/*
 ============================================================================
 Name        : CH56X_gpio.c
 Author      : wch_cc
 Version     : V1.0.0
 Copyright   :
 Description :
 ============================================================================
 */

#include <nds32_isr.h>
#include <nds32_intrinsic.h>
#include "CH568SFR.h"
#include "CH56X_gpio.h"

/*******************************************************************************
* Function Name  : CH56x_GPIOInit
* Description    :
* Input          : GPIOx: where x can be (A or B) to select the GPIO peripheral.
*                  GPIO_Pinx: specifies the port bits.
*                  GPIO_InitStruct: this are some mode, refer to GPIOMode_TpDef
*                  structure.
* Return         : None
*******************************************************************************/
void CH56x_GPIOInit(GPIOPort_TpDef GPIOx, UINT16 GPIO_Pinx, GPIOMode_TpDef GPIO_Mode)
{
	if(GPIOx == GPIOA)
	{
		switch(GPIO_Mode)
		{
		case GPIO_Mode_IN_PU:				//上拉输入
			R32_PA_DIR &= ~GPIO_Pinx;
			R32_PA_PU |= GPIO_Pinx;
			R32_PA_PD &= ~GPIO_Pinx;
			break;

		case GPIO_Mode_IN_PD:				//下拉输入
			R32_PA_DIR &= ~GPIO_Pinx;
			R32_PA_PU &= ~GPIO_Pinx;
			R32_PA_PD |= GPIO_Pinx;
			break;

		case GPIO_Mode_Out_PP_8mA:			//推挽输出最大8mA
			R32_PA_DIR |= GPIO_Pinx;
			R32_PA_DRV &= ~GPIO_Pinx;
			break;

		case GPIO_Mode_Out_PP_16mA:			//推挽输出最大16mA
			R32_PA_DIR |= GPIO_Pinx;
			R32_PA_DRV |= GPIO_Pinx;
			break;

		default :							//浮空输入
			R32_PA_DIR &= ~GPIO_Pinx;
			R32_PA_PU &= ~GPIO_Pinx;
			R32_PA_PD &= ~GPIO_Pinx;
			break;
		}
	}
	else if(GPIOx == GPIOB)
	{
		switch(GPIO_Mode)
		{
		case GPIO_Mode_IN_PU:				//上拉输入
			R32_PB_DIR &= ~GPIO_Pinx;
			R32_PB_PU |= GPIO_Pinx;
			R32_PB_PD &= ~GPIO_Pinx;
			break;

		case GPIO_Mode_IN_PD:				//下拉输入
			R32_PB_DIR &= ~GPIO_Pinx;
			R32_PB_PU &= ~GPIO_Pinx;
			R32_PB_PD |= GPIO_Pinx;
			break;

		case GPIO_Mode_Out_PP_8mA:			//推挽输出最大8mA
			R32_PB_DIR |= GPIO_Pinx;
			R32_PB_DRV &= ~GPIO_Pinx;
			break;

		case GPIO_Mode_Out_PP_16mA:			//推挽输出最大16mA
			R32_PB_DIR |= GPIO_Pinx;
			R32_PB_DRV |= GPIO_Pinx;
			break;

		default :							//浮空输入
			R32_PB_DIR &= ~GPIO_Pinx;
			R32_PB_PU &= ~GPIO_Pinx;
			R32_PB_PD &= ~GPIO_Pinx;
			break;
		}
	}
}


/*******************************************************************************
* Function Name  : CH56x_GPIOSetBits
* Description    :
* Input          : GPIOx: where x can be (A or B) to select the GPIO peripheral.
*                  GPIO_Pinx: specifies the port bits to be set.
*                  This parameter can be any combination of GPIO_Pin_x where x can be (0..15).
* Return         : None
*******************************************************************************/
void CH56x_GPIOSetBits(GPIOPort_TpDef GPIOx, UINT16 GPIO_Pinx)
{
	if(GPIOx == GPIOA){
		R32_PA_OUT |= GPIO_Pinx;
	}
	else{
		R32_PB_OUT |= GPIO_Pinx;
	}
}

/*******************************************************************************
* Function Name  : CH56x_GPIOResetBits
* Description    :
* Input          : GPIOx: where x can be (A or B) to select the GPIO peripheral.
*                  GPIO_Pinx: specifies the port bits to be clear.
*                  This parameter can be any combination of GPIO_Pin_x where x can be (0..15).
* Return         : None
*******************************************************************************/
void CH56x_GPIOResetBits(GPIOPort_TpDef GPIOx, UINT16 GPIO_Pinx)
{
	if(GPIOx == GPIOA){
		R32_PA_CLR = GPIO_Pinx;
	}
	else{
		R32_PB_CLR = GPIO_Pinx;
	}
}

/*******************************************************************************
* Function Name  : CH56x_GPIOInputBit
* Description    : Reads the specified input port pin.
* Input          : GPIO_Pinx: where x can be (A or B) to select the GPIO peripheral.
* 	               GPIO_Pin:  specifies the port bit to read.
*                  This parameter can be GPIO_Pin_x where x can be (0..15).
* Return         : status of port bit
*******************************************************************************/
UINT8 CH56x_GPIOInputBit(GPIOPort_TpDef GPIOx, UINT16 GPIO_Pinx)
{
	if(GPIOx == GPIOA){
		if( R32_PA_PIN&GPIO_Pinx )	return 1;
		else                        return 0;
	}
	else{
		if( R32_PB_PIN&GPIO_Pinx )	return 1;
		else                        return 0;
	}
}

/*******************************************************************************
* Function Name  : CH56x_GPIOInputPort
* Description    : Reads the specified input port .
* Input          : GPIOx: where x can be (A or B) to select the GPIO peripheral.
* Return         : value of port (GPIOA or GPIOB)
*******************************************************************************/
UINT16 CH56x_GPIOInputPort(GPIOPort_TpDef GPIOx)
{
	if(GPIOx == GPIOA){
		return (R32_PA_PIN);
	}
	else{
		return (R32_PB_PIN);
	}
}

/*******************************************************************************
* Function Name  : CH56x_GPIOIntConfig
* Description    :
* Input          : GPIO_ITPinx: refer to GPIO_IntPinx
*                  GPIOITMode_TpDef: this are some mode, refer to GPIOITMode_TpDef
*                  structure.
* Return         : None
*******************************************************************************/
void CH56x_GPIOITConfig(UINT8 GPIO_ITPinx, GPIOITMode_TpDef GPIO_ITMode)
{
	switch( GPIO_ITMode )
	{
	case GPIO_ITMode_HighLevel:				//高电平触发
		R8_GPIO_INT_MODE &= ~GPIO_ITPinx;
		R8_GPIO_INT_POLAR |= GPIO_ITPinx;
		break;

	case GPIO_ITMode_FallEdge:				//下降沿触发
		R8_GPIO_INT_MODE |= GPIO_ITPinx;
		R8_GPIO_INT_POLAR &= ~GPIO_ITPinx;
		break;

	case GPIO_ITMode_RiseEdge:				//上升沿触发
		R8_GPIO_INT_MODE |= GPIO_ITPinx;
		R8_GPIO_INT_POLAR |= GPIO_ITPinx;
		break;

	default:
		R8_GPIO_INT_MODE &= ~GPIO_ITPinx;
		R8_GPIO_INT_POLAR &= ~GPIO_ITPinx;
		break;
	}

	R8_GPIO_INT_STATUS = GPIO_ITPinx;		//清除中断标志
	R8_GPIO_INT_ENABLE |= GPIO_ITPinx;
}

/*******************************************************************************
* Function Name  : CH56x_GPIOITDisable
* Description    :
* Input          : GPIO_ITPinx: refer to GPIO_IntPinx
* Return         : None
*******************************************************************************/
void CH56x_GPIOITDisable( UINT8 GPIO_ITPinx ){
	R8_GPIO_INT_ENABLE &= ~GPIO_ITPinx;
}

/*******************************************************************************
* Function Name  : CH56x_GPIOGetITFlag
* Description    :
* Input          : None
* Return         : Get interrupt flag
*******************************************************************************/
UINT8 CH56x_GPIOGetITFlag( void ){
	return (R8_GPIO_INT_STATUS);
}

/*******************************************************************************
* Function Name  : CH56x_GPIOClearITFlag
* Description    :
* Input          : GPIO_ITPinx: refer to GPIO_IntPinx
* Return         : None
*******************************************************************************/
void CH56x_GPIOClearITFlag( UINT8  GPIO_ITPinx){
	R8_GPIO_INT_STATUS = GPIO_ITPinx;
}

