/*
 ============================================================================
 Name        : CH56X_gpio.h
 Author      : wch_cc
 Version     : V1.0.0
 Copyright   :
 Description :
 ============================================================================
 */


#ifndef __CH56X_GPIO_H__
#define __CH56X_GPIO_H__

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup GPIO_pins_define
  * @{
  */
#define GPIO_Pin_0                 ((UINT16)0x0001)  /*!< Pin 0 selected */
#define GPIO_Pin_1                 ((UINT16)0x0002)  /*!< Pin 1 selected */
#define GPIO_Pin_2                 ((UINT16)0x0004)  /*!< Pin 2 selected */
#define GPIO_Pin_3                 ((UINT16)0x0008)  /*!< Pin 3 selected */
#define GPIO_Pin_4                 ((UINT16)0x0010)  /*!< Pin 4 selected */
#define GPIO_Pin_5                 ((UINT16)0x0020)  /*!< Pin 5 selected */
#define GPIO_Pin_6                 ((UINT16)0x0040)  /*!< Pin 6 selected */
#define GPIO_Pin_7                 ((UINT16)0x0080)  /*!< Pin 7 selected */
#define GPIO_Pin_8                 ((UINT16)0x0100)  /*!< Pin 8 selected */
#define GPIO_Pin_9                 ((UINT16)0x0200)  /*!< Pin 9 selected */
#define GPIO_Pin_10                ((UINT16)0x0400)  /*!< Pin 10 selected */
#define GPIO_Pin_11                ((UINT16)0x0800)  /*!< Pin 11 selected */
#define GPIO_Pin_12                ((UINT16)0x1000)  /*!< Pin 12 selected */
#define GPIO_Pin_13                ((UINT16)0x2000)  /*!< Pin 13 selected */
#define GPIO_Pin_14                ((UINT16)0x4000)  /*!< Pin 14 selected */
#define GPIO_Pin_15                ((UINT16)0x8000)  /*!< Pin 15 selected */
#define GPIO_Pin_All               ((UINT16)0xFFFF)  /*!< All pins selected */


/** @defgroup GPIO_ITPinx define
  * @{
  */
#define GPIO_ITPin_PB10           RB_GPIO_PB10
#define GPIO_ITPin_PB4            RB_GPIO_PB4
#define GPIO_ITPin_PA12           RB_GPIO_PA12
#define GPIO_ITPin_PA11           RB_GPIO_PA11
#define GPIO_ITPin_PA10           RB_GPIO_PA10
#define GPIO_ITPin_PA6            RB_GPIO_PA6
#define GPIO_ITPin_PA4            RB_GPIO_PA4
#define GPIO_ITPin_PA3            RB_GPIO_PA3
#define GPIO_ITPin_All            0xFF


/** @defgroup GPIO_ITFlag define
  * @{
  */
#define GPIO_ITFlag_PB10           RB_GPIO_PB10
#define GPIO_ITFlag_PB4            RB_GPIO_PB4
#define GPIO_ITFlag_PA12           RB_GPIO_PA12
#define GPIO_ITFlag_PA11           RB_GPIO_PA11
#define GPIO_ITFlag_PA10           RB_GPIO_PA10
#define GPIO_ITFlag_PA6            RB_GPIO_PA6
#define GPIO_ITFlag_PA4            RB_GPIO_PA4
#define GPIO_ITFlag_PA3            RB_GPIO_PA3
#define GPIO_ITFlag_All            0xFF


/**
  * @brief  Configuration GPIO port
  */
typedef enum
{
	GPIOA = 0,
	GPIOB = !GPIOA,
}GPIOPort_TpDef;

/**
  * @brief  Configuration GPIO Mode
  */
typedef enum
{
	GPIO_Mode_IN_Floating,			//浮空输入
	GPIO_Mode_IN_PU,				//上拉输入
	GPIO_Mode_IN_PD,				//下拉输入

	GPIO_Mode_Out_PP_8mA,			//推挽输出最大8mA
	GPIO_Mode_Out_PP_16mA,			//推挽输出最大16mA

}GPIOMode_TpDef;

/**
  * @brief  Configuration GPIO output slew rate & input schmitt trigger
  */
typedef enum
{
	GPIO_Mode_FAST,			//快速输出/正常输入
	GPIO_Mode_SLOW,			//斜率输出/斯密特输入
}GPIOSMT_TypeDef;


/**
  * @brief  Configuration GPIO IT Mode
  */
typedef enum
{
	GPIO_ITMode_LowLevel,			//低电平触发
	GPIO_ITMode_HighLevel,			//高电平触发
	GPIO_ITMode_FallEdge,			//下降沿触发
	GPIO_ITMode_RiseEdge,			//上升沿触发

}GPIOITMode_TpDef;


void CH56x_GPIOInit(GPIOPort_TpDef GPIOx, UINT16 GPIO_Pinx, GPIOMode_TpDef GPIO_Mode);
void CH56x_GPIOSetBits(GPIOPort_TpDef GPIOx, UINT16 GPIO_Pinx);
void CH56x_GPIOResetBits(GPIOPort_TpDef GPIOx, UINT16 GPIO_Pinx);
UINT8 CH56x_GPIOInputBit(GPIOPort_TpDef GPIOx, UINT16 GPIO_Pinx);
UINT16 CH56x_GPIOInputPort(GPIOPort_TpDef GPIOx);

void CH56x_GPIOITConfig(UINT8 GPIO_ITPinx, GPIOITMode_TpDef GPIO_ITMode);
void CH56x_GPIOITDisable( UINT8 GPIO_ITPinx );
UINT8 CH56x_GPIOGetITFlag( void );
void CH56x_GPIOClearITFlag( UINT8  GPIO_ITPinx);


#ifdef __cplusplus
}
#endif

#endif		//__CH56X_GPIO_H__
