/*
 ============================================================================
 Name        : ch56x_usb1dev372.c
 Author      : Zoologist
 Version     :
 Copyright   : www.lab-z.com
 Description : SPI1 上的GD25 SPINOR 驱动
 ============================================================================
 */
 
#include <stdio.h>
#include <nds32_isr.h>
#include <nds32_intrinsic.h>

#include "CH568SFR.H"
#include "MYdebug.H"
#include "Ch56x_GDSPI1NOR.h"
#include "Ch56x_GPIO.H"

/*******************************************************************************
* Function Name  : SPIFlash_Enter 4-Byte Address Mode(EN4B)
* Description    : SPI Flash进入4 Bytes 地址模式,对于16MB及其以下的SPI NOR 使用3Bytes 地址模式
				   访问；对于超过16MB 的需要使用4Bytes 地址模式来访问
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
void SPIFlash_EN4B()
{
        SPI1_CS_LOW();
        SPI1_Trans(GD25Q_CMD_EN4B);    //进入4Byte 地址模式
        SPI1_CS_HIGH();

}

/*******************************************************************************
* Function Name  : WriteExtendedRegister
* Description    : 写状态寄存器1
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
void WriteExtendedRegister( UINT8 Status )
{
        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_WRITEEXTREG );
        SPI1_Trans( Status );
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();                               //等待写入结束
        return ;
}

/*******************************************************************************
* Function Name  : SPI1_MASTER_INIT
* Description    : SPI1主机模式初始化
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SPI1_MASTER_INIT ( void )
{
        printf("SPI1 Init!\n");
        R8_SPI1_CTRL_MOD = RB_SPI_MOSI_OE|RB_SPI_SCK_OE;         /* MOSI,SCK输出使能，主机模式，方式0 */
        R8_SPI1_CLOCK_DIV = 0x0a;                                /* 10分频，主频/10 */
        R32_PA_DIR |= (1<<0 | 1<<1);                      		 /* MOSI(PA1),SCK1(PA0),SCS(PB7)为输出*/
        R32_PB_DIR |=  1<<7 ;
        R32_PA_PU  |=  1<<2 ;
		SPI1_CS_HIGH();
        
		printf("SPI1 ID:0x%04x\n",SPIFlash_ReadID() );
		
        printf("Check status reg\n");
        printf("Status1:0x%01x\n",ReadStatus1());
        printf("Status2:0x%01x\n",ReadStatus2());
        printf("Status3:0x%01x\n",ReadStatus3());
        printf("Read Ext:0x%01x\n",ReadExtendedRegister());
      
        WriteExtendedRegister(1);
        SPIFlash_EN4B();

        WriteEnable();
        WriteStatus1(0x00);
        printf("re-Check status reg\n");
        printf("Status1:0x%01x\n",ReadStatus1());
        printf("Status2:0x%01x\n",ReadStatus2());
        printf("Status3:0x%01x\n",ReadStatus3());    
        printf("Read Ext:0x%01x\n",ReadExtendedRegister());
        
}

/*******************************************************************************
* Function Name  : SPI1_Trans
* Description    : 发送一字节数据
* Input          : data -要发送的数据
* Output         : None
* Return         : None
*******************************************************************************/
void SPI1_Trans( UINT8 data )
{

//    R8_SPI1_CTRL_MOD &= ~RB_SPI_FIFO_DIR;
//    R8_SPI1_BUFFER = data;
//    while( !(R8_SPI1_INT_FLAG & RB_SPI_FREE) );

        R32_SPI1_FIFO = data;
        R16_SPI1_TOTAL_CNT = 0x01;
        while( R8_SPI1_FIFO_COUNT != 0 );                                           /* 等待数据发送完毕 */
}

/*******************************************************************************
* Function Name  : SPI1_Recv
* Description    : 接收一字节数据
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
UINT8 SPI1_Recv( void )
{
//    R8_SPI1_CTRL_MOD &= ~RB_SPI_FIFO_DIR;
//    R8_SPI1_BUFFER = 0xFF;                                //启动传输
//    while( !(R8_SPI1_INT_FLAG & RB_SPI_FREE) );
//    return ( R8_SPI1_BUFFER );

        UINT8 data;
        R32_SPI1_FIFO = 0xff;
        R16_SPI1_TOTAL_CNT = 0x01;
        while( R8_SPI1_FIFO_COUNT != 0 );                                           /* 等待数据回来 */
        data = R8_SPI1_BUFFER;
        return data;
}



/*******************************************************************************
* Function Name  : SPIFlash_ReadID
* Description    : SPI Flash读取芯片ID
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
UINT32 SPIFlash_ReadID()
{
        UINT32  temp = 0;

        SPI1_CS_LOW();

        SPI1_Trans(GD25Q_CMD_READID);    //读取ID命令
        temp = SPI1_Recv();
        temp = temp<<8;
        temp |= SPI1_Recv();
        temp = temp<<8;
        temp |= SPI1_Recv();

        SPI1_CS_HIGH();

        return temp;
}

/*******************************************************************************
* Function Name  : WriteEnable
* Description    : 写使能
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void WriteEnable( void )
{
        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_WRITEENABLE );   //发送写使能命令
        SPI1_CS_HIGH();
}

/*******************************************************************************
* Function Name  : WriteDisable
* Description    : 关闭写
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void WriteDisable( void )
{
        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_WRITEDISABLE );   //发送写禁止命令
        SPI1_CS_HIGH();
}

/*******************************************************************************
* Function Name  : ReadStatus1
* Description    : 读取状态寄存器1
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
UINT8 ReadStatus1()
{
        UINT8 Status;

        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_READSTATUS1 ); //发送读状态寄存器的命令
        Status = SPI1_Recv();      //读取状态寄存器
        SPI1_CS_HIGH();

        return Status;
}

/*******************************************************************************
* Function Name  : ReadStatus2
* Description    : 读取状态寄存器2
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
UINT8 ReadStatus2()
{
        UINT8 Status;

        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_READSTATUS2 ); //发送读状态寄存器的命令
        Status = SPI1_Recv();      //读取状态寄存器
        SPI1_CS_HIGH();

        return Status;
}

/*******************************************************************************
* Function Name  : ReadStatus3
* Description    : 读取状态寄存器3
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
UINT8 ReadStatus3()
{
        UINT8 Status;

        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_READSTATUS3 ); //发送读状态寄存器的命令
        Status = SPI1_Recv();      //读取状态寄存器
        SPI1_CS_HIGH();

        return Status;
}

/*******************************************************************************
* Function Name  : WriteStatus1
* Description    : 写状态寄存器1
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
void WriteStatus1( UINT8 Status )
{
        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_WRITESTATUS1 );     //发送写状态寄存器的命令
        SPI1_Trans( Status );                     //发状态寄存器的数值
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();                               //等待写入结束
        return ;
}

/*******************************************************************************
* Function Name  : WriteStatus2
* Description    : 写状态寄存器2
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
void WriteStatus2( UINT8 Status )
{
        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_WRITESTATUS2 );     //发送写状态寄存器的命令
        SPI1_Trans( Status );                     //发状态寄存器的数值
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();                               //等待写入结束
        return ;
}

/*******************************************************************************
* Function Name  : WriteStatus3
* Description    : 写状态寄存器3
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
void WriteStatus3( UINT8 Status )
{
        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_WRITESTATUS3 );     //发送写状态寄存器的命令
        SPI1_Trans( Status );                     //发状态寄存器的数值
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();                               //等待写入结束
        return ;
}

/*******************************************************************************
* Function Name  : ReadExternalFlash_SPI
* Description    : 读数据，注意：地址低8位必须为0
*******************************************************************************/
void ReadSPINorData( UINT32 StarAddr, UINT16 Len, PUINT8 RcvBuffer )
{
		CH56x_GPIOSetBits(GPIOA,  GPIO_Pin_4);
        UINT16 i;
        SPI1_CS_LOW();
        SPI1_Trans(GD25Q_CMD_READDATA);              //读命令
        SPI1_Trans(( StarAddr             >> 24));   //发送4字节地址
        SPI1_Trans(((StarAddr & 0xFFFFFF) >> 16));
        SPI1_Trans(((StarAddr & 0xFFFF) >> 8));
        SPI1_Trans(StarAddr & 0xFF);
        for (i=0; i<Len; i++)
        {
        	RcvBuffer[i]=SPI1_Recv();
        }
        SPI1_CS_HIGH();
        CH56x_GPIOResetBits(GPIOA,  GPIO_Pin_4);
}

/*******************************************************************************
* Function Name  : WaitExternalFlashIfBusy
* Description    : 等待芯片空闲(在执行Byte-Program, Sector-Erase, Block-Erase, Chip-Erase操作后)
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void WaitExternalFlashIfBusy( void )
{
        while ((ReadStatus1()&0x01) == 0x01 )
        {
        	//printf("Busy\n");
                //;    //等待直到Flash空闲
        }
}
/*******************************************************************************
* Function Name  : PageProgram
* Description    : 对一个page 进行写入
*******************************************************************************/
void PageProgram( UINT32 StarAddr, UINT16 Len, PUINT8 RcvBuffer )
{
		CH56x_GPIOSetBits(GPIOA,  GPIO_Pin_4);
        int i;

        printf("PpX %lx [%X]\n",StarAddr,Len);
                                                                        
        WriteEnable();

        SPI1_CS_LOW();
        SPI1_Trans(GD25Q_CMD_PAGEPROGRAM);              //写命令
        SPI1_Trans(( StarAddr             >> 24));      //发送4字节地址
        SPI1_Trans(((StarAddr & 0xFFFFFF) >> 16));
        SPI1_Trans(((StarAddr & 0xFFFF) >> 8));
        SPI1_Trans(StarAddr & 0xFF);
        for (i=0; i<Len; i++)
        {
                SPI1_Trans(RcvBuffer[i]);
        }
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();                       //等待写入结束
        CH56x_GPIOResetBits(GPIOA,  GPIO_Pin_4);
}

/*******************************************************************************
* Function Name  : SectorErase
* Description    : 擦除4K Flash  擦除一个扇区
* Input          : Dst_Addr 0-1 ffff ffff ,清除任意地址所在的扇区
* Output         : None
* Return         : None
*******************************************************************************/
void SectorErase( UINT32 Dst_Addr )
{
        printf("SectorErase :0x%lx\n",Dst_Addr);
        WriteEnable();
        WaitExternalFlashIfBusy();
        printf("Status1:0x%01x\n",ReadStatus1());
        printf("Status2:0x%01x\n",ReadStatus2());
        printf("Status3:0x%01x\n",ReadStatus3());

        SPI1_CS_LOW();
        SPI1_Trans(GD25Q_CMD_SECTORERASE);                                 //扇区擦除命令
        SPI1_Trans(( Dst_Addr             >> 24));                         //发送4字节地址
        SPI1_Trans(((Dst_Addr & 0xFFFFFF) >> 16));                         
        SPI1_Trans(((Dst_Addr & 0xFFFF) >> 8));
        SPI1_Trans(Dst_Addr & 0xFF);

        printf("addr:0x%lx\n",( Dst_Addr             >> 24));
        printf("addr:0x%lx\n",((Dst_Addr & 0xFFFFFF) >> 16));
        printf("addr:0x%lx\n",((Dst_Addr & 0xFFFF) >> 8));
        printf("addr:0x%lx\n",(Dst_Addr & 0xFF));
        
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();
}

/*******************************************************************************
* Function Name  : Block32KErase
* Description    : 擦除32K Flash
* Input          : Dst_Addr 0-1 ffff ffff ,清除任意地址所在的扇区
* Output         : None
* Return         : None
*******************************************************************************/
void Block32KErase( UINT32 Dst_Addr )
{
        WriteEnable();

        SPI1_CS_LOW();
        SPI1_Trans(GD25Q_CMD_BLOCKERASE32K);                                      //扇区擦除命令
        SPI1_Trans(( Dst_Addr             >> 24));   //发送4字节地址
        SPI1_Trans(((Dst_Addr & 0xFFFFFF) >> 16));
        SPI1_Trans(((Dst_Addr & 0xFFFF) >> 8));
        SPI1_Trans(Dst_Addr & 0xFF);
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();
}

/*******************************************************************************
* Function Name  : Block64KErase
* Description    : 擦除64K Flash
* Input          : Dst_Addr 0-1 ffff ffff ,清除任意地址所在的扇区
* Output         : None
* Return         : None
*******************************************************************************/
void Block64KErase( UINT32 Dst_Addr )
{
        WriteEnable();

        SPI1_CS_LOW();
        SPI1_Trans(GD25Q_CMD_BLOCKERASE64K);                                      //扇区擦除命令
        SPI1_Trans(( Dst_Addr             >> 24));   //发送4字节地址
        SPI1_Trans(((Dst_Addr & 0xFFFFFF) >> 16));
        SPI1_Trans(((Dst_Addr & 0xFFFF) >> 8));
        SPI1_Trans(Dst_Addr & 0xFF);
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();
}

/*******************************************************************************
* Function Name  : ChipErase
* Description    : 擦除整片
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void ChipErase()
{
        WriteEnable();

        SPI1_CS_LOW();
        SPI1_Trans(GD25Q_CMD_CHIPERASE);     //整片擦除
        SPI1_CS_HIGH();

        WaitExternalFlashIfBusy();
}

/*******************************************************************************
* Function Name  : ReadExtendedRegister
* Description    : 读取Extended Register
* Input          : None
* Output         : None
* Return         :
*******************************************************************************/
UINT8 ReadExtendedRegister()
{
        UINT8 Status;

        SPI1_CS_LOW();
        SPI1_Trans( GD25Q_CMD_READEXTREG );
        Status = SPI1_Recv();
        SPI1_CS_HIGH();

        return Status;
}


