/*
 ============================================================================
 Name        : ch56x_usb1dev372.c
 Author      : Zoologist
 Version     :
 Copyright   : www.lab-z.com
 Description : SPI0 上的PSRAM 驱动
 ============================================================================
 */
 
#include <stdio.h>
#include <nds32_isr.h>
#include <nds32_intrinsic.h>

#include "CH568SFR.H"
#include "MYdebug.H"
#include "SPI0PSRAM.h"
#include "Ch56x_GPIO.H"

/*******************************************************************************
* Function Name  : SPI0_MASTER_INIT
* Description    : SPI0主机模式初始化
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SPI0_MASTER_INIT ( void )
{
  R8_SPI0_CTRL_MOD = RB_SPI_MOSI_OE|RB_SPI_SCK_OE;                              /* MOSI,SCK输出使能，主机模式，方式0 */
  R8_SPI0_CLOCK_DIV = 0x02;                                                     /* 10分频，主频/10*/
  R32_PA_DIR |= (1<<14 | 1<<13 | 1<<12);                                        /* MOSI(PA14),SCK0(PA13),SCS(PA12)为输出*/
  R32_PA_PU  |=  1<<12 ;

  R8_SPI0_CTRL_CFG &= ~RB_SPI_DMA_ENABLE;
  printf("SPI0 ID:0x%02x\n",SPI0Flash_ReadID() );

}

/*******************************************************************************
* Function Name  : SPI0_Trans
* Description    : 发送一字节数据
* Input          : data -要发送的数据
* Output         : None
* Return         : None
*******************************************************************************/
void SPI0_Trans( UINT8 data )
{

//    R8_SPI0_CTRL_MOD &= ~RB_SPI_FIFO_DIR;
//    R8_SPI0_BUFFER = data;
//    while( !(R8_SPI0_INT_FLAG & RB_SPI_FREE) );

    R32_SPI0_FIFO = data;
    R16_SPI0_TOTAL_CNT = 0x01;
    while( R8_SPI0_FIFO_COUNT != 0 );                                           /* 等待数据发送完毕 */
}

/*******************************************************************************
* Function Name  : SPI0_Recv
* Description    : 接收一字节数据
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
UINT8 SPI0_Recv( void )
{
//    R8_SPI0_CTRL_MOD &= ~RB_SPI_FIFO_DIR;
//    R8_SPI0_BUFFER = 0xFF;                                //启动传输
//    while( !(R8_SPI0_INT_FLAG & RB_SPI_FREE) );
//    return ( R8_SPI0_BUFFER );

    UINT8 data;
    R32_SPI0_FIFO = 0xff;
    R16_SPI0_TOTAL_CNT = 0x01;
    while( R8_SPI0_FIFO_COUNT != 0 );                                           /* 等待数据回来 */
    data = R8_SPI0_BUFFER;
    return data;
}

/*******************************************************************************
* Function Name  : SPI0Flash_ReadID
* Description    : SPI Flash读取芯片ID
* Input          : None
* Output         : None
* Return         : 
*******************************************************************************/
UINT16 SPI0Flash_ReadID(void)
{
    UINT32  temp = 0;

    SPI0_CS_LOW();
    SPI0_Trans(CMD_READID);  //读取ID命令
    SPI0_Trans(0x00);
    SPI0_Trans(0x00);
    SPI0_Trans(0x00);

    temp = SPI0_Recv();
    temp = temp<<8;
    temp |= SPI0_Recv();

    SPI0_CS_HIGH();

    return temp;
}

/*******************************************************************************
* Function Name  : ReadPSRAMData_SPI0
* Description    : 读取地址的数据
*******************************************************************************/
void ReadPSRAMData_SPI0( UINT32 StarAddr, UINT16 Len, PUINT8 RcvBuffer )
{
	CH56x_GPIOSetBits(GPIOA,  GPIO_Pin_4);
    UINT16 i;
    
    SPI0_CS_LOW();
    SPI0_Trans(CMD_FASTREAD);                   //读命令
    SPI0_Trans(((StarAddr & 0xFFFFFF) >> 16));  //发送地址高位
    SPI0_Trans(((StarAddr & 0xFFFF) >> 8));
    SPI0_Trans(StarAddr & 0xFF);
    SPI0_Trans(0x00);  //Wait Cycles
    for (i=0;i<Len; i++) {
            RcvBuffer[i]= SPI0_Recv();
    }
    
    SPI0_CS_HIGH();
    CH56x_GPIOResetBits(GPIOA,  GPIO_Pin_4);
}

/*******************************************************************************
* Function Name  : BlukWriteExternalFlash_SPI
* Description    : 无检验写SPI FLASH
* Input          : SendBuffer:数据存储区
*                  StarAddr:开始写入的地址
*                  Len:要写入的字节数(最大1024)
* Output         : None
* Return         : None
*******************************************************************************/
void WritesPSRAMData_SPI0(UINT32 StarAddr,UINT16 Len,PUINT8 SendBuffer)
{
	CH56x_GPIOSetBits(GPIOA,  GPIO_Pin_4);
    UINT16 i;
    SPI0_CS_LOW();
    SPI0_Trans(0x02);
    SPI0_Trans(((StarAddr & 0xFFFFFF) >> 16));  //发送地址高位
    SPI0_Trans(((StarAddr & 0xFFFF) >> 8));
    SPI0_Trans(StarAddr & 0xFF);
    for(i=0; i<Len; i++)
    {
        SPI0_Trans(SendBuffer[i]);
    }
    SPI0_CS_HIGH();
    CH56x_GPIOResetBits(GPIOA,  GPIO_Pin_4);
}
