/*
 ============================================================================
 Name        : ch56x_usb1dev372.c
 Author      : CC
 Version     :
 Copyright   : www.lab-z.com
 Description : USB0&1 USB Mass Storage device
 ============================================================================
 */

#include <stdio.h>
#include <nds32_isr.h>
#include <nds32_intrinsic.h>

#include "CH568SFR.H"
#include "MYdebug.H"
#include "USBShare.H"
#include "USB1.h"

// USB 设备1

__attribute__ ((aligned(4)))UINT8 Usb1Ep0Buffer[64]; // EP1+EP4 OUT&IN, must 4×??ú????
__attribute__ ((aligned(4)))UINT8 Usb1Ep1Buffer[512 * 2]; // EP1 IN+OUT, must 4×??ú????
#define Usb1SetupBuf         ((PUSB_SETUP_REQ)Usb1Ep0Buffer)
#define Usb1Ep1OUTBuf        ((PUINT8)&Usb1Ep1Buffer[0])
#define Usb1Ep1INBuf          ((PUINT8)&Usb1Ep1Buffer[512])

UINT16 Usb1SetupLen = 0; /* USB控制传输传输长度 */
UINT8V Usb1SetupReqCode; /* USB控制传输命令码 */
PUINT8 Usb1pDescr; /* 上传指针 */
UINT8 Usb1DevAddr; /* 设备地址 */
UINT8 Usb1Configed = 0;                // USB配置标志

UINT8 Usb1CurrentRequest = 0;            //记录当前正在处理的 Request
UINT32 Usb1BytesLeft; /*剩余需要传输的字节数*/
UINT32 Usb1BytesSend; /*已经发送的字节数*/

/*******************************************************************************
 * Function Name  : USB1DeviceInit
 * Description    : USB1 设备模式初始化
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void USB1DeviceInit(void)  // 初始化USB设备
{
        /* 全局寄存器 */
        R8_USB1_CTRL = bUC_CLR_ALL;       //清空中断标志和FIFO
        R8_USB1_CTRL = UCST_HS | bUC_INT_BUSY | bUC_DMA_EN;  //设备模式，全速，
        R8_USB1_SUSPEND = 0;                //不挂起
        R8_USB1_DEV_AD = 0x00;              //设备地址0
        R8_USB1_INT_FG = 0xff;              //清除中断标志
        R8_USB1_INT_EN = bUIE_FIFO_OV | bUIE_SUSPEND | bUIE_TRANSFER | bUIE_BUS_RST; //中断使能
        R8_UEP4_1_MOD1 = 0;
        R8_UEP2_3_MOD1 = 0;

        /* 设备端点设置 */
//ENDP0
        R8_UEP0_TX_CTRL1 = UEP_T_RES_NAK;
        R8_UEP0_RX_CTRL1 = UEP_R_RES_ACK;
        R16_UEP0_MAX_LEN1 = THIS_ENDP0_SIZE;             //端点0接收包的最大长度为64字节
        R16_UEP0_DMA1 = (UINT32) Usb1Ep0Buffer;

//ENDP1
        R8_UEP1_TX_CTRL1 = UEP_T_RES_NAK | bUEP_AUTO_TOG;
        R8_UEP1_RX_CTRL1 = UEP_R_RES_ACK | bUEP_AUTO_TOG;
        R8_UEP4_1_MOD1 |= bUEP1_TX_EN | bUEP1_RX_EN;
        R16_UEP1_MAX_LEN1 = 512;
        R16_UEP1_DMA1 = (UINT32) Usb1Ep1Buffer;

//ENDP2
        R8_UEP2_TX_CTRL1 = UEP_T_RES_NAK | bUEP_AUTO_TOG;
        R8_UEP2_RX_CTRL1 = UEP_R_RES_ACK | bUEP_AUTO_TOG;
        R8_UEP2_3_MOD1 |= bUEP2_RX_EN | bUEP2_TX_EN;
        R16_UEP2_MAX_LEN1 = 0;
        R16_UEP2_DMA1 = 0;

//ENDP3
        R8_UEP3_TX_CTRL1 = UEP_T_RES_NAK | bUEP_AUTO_TOG;
        R8_UEP3_RX_CTRL1 = UEP_R_RES_ACK | bUEP_AUTO_TOG;
        R8_UEP2_3_MOD1 |= bUEP3_RX_EN | bUEP3_TX_EN;
        R16_UEP3_MAX_LEN1 = 0;
        R16_UEP3_DMA1 = 0;

//ENDP4
        R8_UEP4_TX_CTRL1 = UEP_T_RES_NAK;        //不支持自动翻转
        R8_UEP4_RX_CTRL1 = UEP_R_RES_ACK;
        R8_UEP4_1_MOD1 |= bUEP4_RX_EN | bUEP4_TX_EN;
        R16_UEP4_MAX_LEN1 = 0;

        R8_USB1_CTRL |= bUC_DEV_PU_EN;        //设备连接
}

/*******************************************************************************
 * Function Name  : USB1DevIntDeal
 * Description    : USB1 设备中断处理
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void USB1DevIntDeal(void)
{
        UINT32 i, l, len;
        PBULK_ONLY_CMD CMD;
        static UINT32 LBA;
        static UINT32 SPINorPosition; /*要发送的数据在 SPI NOR 中的位置*/
        UINT8 intstatus;

        if ( R8_USB1_INT_FG & UIF_TRANSFER)    // USB传输完成
        {
                if ( R8_USB1_INT_ST & bUIS_IS_NAK)    //接收到主机IN包
                {
                        printf("NAK\n");
                }
                else
                {
                        intstatus = R8_USB1_INT_ST & (MASK_UIS_TOKEN | MASK_UIS_ENDP);

                        if (intstatus == (UIS_TOKEN_OUT | 4)) /* endpoint 4  下传 */
                        {
                                if (R8_USB1_INT_ST & bUIS_TOG_OK)
                                {
                                        R8_UEP4_TX_CTRL1 = (R8_UEP4_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                           | UEP_T_RES_ACK;
                                }
                        }
                        if (intstatus == (UIS_TOKEN_OUT | 3)) /* endpoint 3 下传 */
                        {
                                if (R8_USB1_INT_ST & bUIS_TOG_OK)
                                {
                                        R8_UEP3_TX_CTRL1 = (R8_UEP3_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                           | UEP_T_RES_ACK;
                                }
                        }
                        if (intstatus == (UIS_TOKEN_OUT | 2)) /* endpoint 2 下传 */
                        {
                                if (R8_USB1_INT_ST & bUIS_TOG_OK)
                                {
                                        R8_UEP2_TX_CTRL1 = (R8_UEP2_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                           | UEP_T_RES_ACK;
                                }
                        }
                        if (intstatus == (UIS_TOKEN_OUT | 1)) /* endpoint 1 下传 */
                        {
                                if (R8_USB1_INT_ST & bUIS_TOG_OK)
                                {
                                        //R8_UEP1_RX_CTRL1 = R8_UEP1_RX_CTRL1 ^ 0x08;    //翻转
                                        /*
                                         l = R16_USB1_RX_LEN;
                                         for(i=0; i<l; i++)
                                         Usb1Ep1INBuf[i] = ~Usb1Ep1OUTBuf[i];        //按位取反上传
                                         R16_UEP1_T_LEN1 = l;
                                         R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;

                                         printf("OUTBUFFERA:");
                                         for (i=0; i<R16_USB1_RX_LEN; i++)
                                         {
                                         printf("%X ",Usb1Ep1Buffer[i]);
                                         }
                                         printf("\n");
                                         */
                                        if (Usb1CurrentRequest == 0x2A)     // WRITE 10
                                        {
                                                len = (Usb1BytesLeft < ENDP1_SIZE) ?
                                                      Usb1BytesLeft : ENDP1_SIZE;
                                                // 实际写入数据
                                                // 有可能收到请求 LBA 0, Length=8096 这种超过1个扇区的请求
                                                printf("Write10 %lu(D) 0x%lu\n", len,SPINorPosition);
                                                if (SPINorPosition >= 0xFFFFFFFE) // 如果访问超过SPINOR 范围
                                                {
													if (SPINorPosition==0xFFFFFFFE) { //表示当前访问TranHeader
														printf("WTH %x\n",Usb1Ep1OUTBuf[0]);
														MEMCPY(&TranHeader, &Usb1Ep1OUTBuf[0], sizeof(TranHeader));
													}
                                                }
                                                else
                                                {
                                                        // 落在 SPI NOR 范围
                                                        if (SPINorPosition<MAXSPINORADDRESS)
                                                        {
                                                                printf("wSPI \n");

                                                                // 访问的地址在SPI NOR范围以内直接写入
                                                                if (len > 256)   //如果要写入的长度超过一个Page的256 Bytes
                                                                {
                                                                        for (i = 0; i < len / 256; i++) //分成几次写入
                                                                        {
                                                                                PageProgram(SPINorPosition + i * 256, 256,
                                                                                            &Usb1Ep1OUTBuf[i * 256]);
                                                                        }
                                                                }
                                                                else
                                                                {
                                                                        PageProgram(SPINorPosition, len, Usb1Ep1OUTBuf);
                                                                }
                                                                SPINorPosition = SPINorPosition + len;
                                                                printf("PageProgram complete\n");
                                                                //printf("id:0x%04x\n",SPIFlash_ReadID() );
                                                        }else {
															     printf("wPSRAM %d\n",SPINorPosition-(MAXDISKADDRESS-MAXSPSRAMADDRESS));
                                                                // 落在 PSRAM 范围
                                                                WritesPSRAMData_SPI0(SPINorPosition-(MAXDISKADDRESS-MAXSPSRAMADDRESS),len,&Usb1Ep1OUTBuf[0]);
                                                                SPINorPosition = SPINorPosition + len;
                                                        }

                                                }
                                                Usb1BytesLeft = Usb1BytesLeft - len;

                                                R8_UEP1_RX_CTRL1 = (R8_UEP1_RX_CTRL1 & ~ MASK_UEP_R_RES)
                                                                   | UEP_R_RES_ACK;

                                        }
                                        else
                                        {
                                                CMD = &Usb1Ep1OUTBuf[0];
                                                if (CMD->mCBW.mCBW_CB_Buf[0] == 0x12)   // INQUIRY CMD
                                                {
                                                        printf("INQUIRY CMD %d tag %X\n",
                                                               CMD->mCBW.mCBW_DataLen, CMD->mCBW.mCBW_Tag);
                                                        Usb1CurrentRequest = 0x12;   //当前是INQUIRY CMD
                                                        if (CMD->mCBW.mCBW_DataLen == 0xFF) //两种 INQUIRY
                                                        {
                                                                MEMCPY( Usb1Ep1INBuf, &InquiryReply[0],
                                                                        sizeof(InquiryReply));
                                                                R16_UEP1_T_LEN1 = sizeof(InquiryReply);
                                                                R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1
                                                                                    & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;
                                                        }
                                                        else
                                                        {
                                                                MEMCPY( Usb1Ep1INBuf, &InquiryReply[0],
                                                                        sizeof(InquiryReply));
                                                                R16_UEP1_T_LEN1 = sizeof(InquiryReply);
                                                                R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1
                                                                                    & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;
                                                        }
                                                        for (i = 0; i < sizeof(InquiryReply); i++)
                                                        {
                                                                printf("%X ", Usb1Ep1INBuf[i]);
                                                        }
                                                        printf("\n");
                                                }

                                                if (CMD->mCBW.mCBW_CB_Buf[0] == 0x23) // READ FORMAT CAPACITIES
                                                {
                                                        Usb1CurrentRequest = 0x23; //当前是 READ FORMAT CAPACITIES
                                                        MEMCPY( Usb1Ep1INBuf, &RDFMCAPReply[0],
                                                                sizeof(RDFMCAPReply)); /* 加载上传数据 */
                                                        R16_UEP1_T_LEN1 = sizeof(RDFMCAPReply);
                                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1
                                                                            & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;

                                                }

                                                if (CMD->mCBW.mCBW_CB_Buf[0] == 0x25)   // READ CAPACITY
                                                {
                                                        Usb1CurrentRequest = 0x25;   //当前是 READ CAPACITY
                                                        MEMCPY( Usb1Ep1INBuf, &RDCAPREPLY[0],
                                                                sizeof(RDCAPREPLY)); /* 加载上传数据 */
                                                        R16_UEP1_T_LEN1 = sizeof(RDCAPREPLY);
                                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1
                                                                            & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;

                                                }

                                                if (CMD->mCBW.mCBW_CB_Buf[0] == 0x1A)   // MODE SENSE 6
                                                {
                                                        Usb1CurrentRequest = 0x1A;   //当前是 MODE SENSE 6
                                                        MEMCPY( Usb1Ep1INBuf, &MODESENE6REPLY[0],
                                                                sizeof(MODESENE6REPLY)); /* 加载上传数据 */
                                                        R16_UEP1_T_LEN1 = sizeof(MODESENE6REPLY);
                                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1
                                                                            & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;
                                                }
                                                /*
                                                 if (CMD->mCBW.mCBW_CB_Buf[0]==0x1E)   //  SCSI PREVENT ALLOW MEDIUM REMOVAL command
                                                 {
                                                 Usb1CurrentRequest=0x1A;//当前是 MODE SENSE 6
                                                 MEMCPY( Usb1Ep1INBuf, &MODESENE6REPLY[0], sizeof(MODESENE6REPLY) );  //加载上传数据
                                                 R16_UEP1_T_LEN1 = sizeof(MODESENE6REPLY);
                                                 R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;
                                                 }*/
                                                if (CMD->mCBW.mCBW_CB_Buf[0] == 0x00) // TEST UNIT READY
                                                {
                                                        printf("Tag: %X\n", CMD->mCBW.mCBW_Tag);
                                                        MEMCPY(&TESTUNITREADYREPLY[4], &CMD->mCBW.mCBW_Tag,
                                                               4);
                                                        MEMCPY( Usb1Ep1INBuf, &TESTUNITREADYREPLY[0],
                                                                sizeof(TESTUNITREADYREPLY)); /* 加载上传数据 */
                                                        R16_UEP1_T_LEN1 = sizeof(TESTUNITREADYREPLY);
                                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1
                                                                            & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;
                                                }
                                                if (CMD->mCBW.mCBW_CB_Buf[0] == 0x28)   // READ 10
                                                {
                                                        Usb1CurrentRequest = 0x28;   //当前是 READ 10
                                                        Usb1BytesLeft = CMD->mCBW.mCBW_DataLen; //要发送的数据长度
                                                        Usb1BytesSend = 0;
                                                        LBA = (CMD->mCBW.mCBW_CB_Buf[2]);
                                                        LBA = (LBA << 8) | (CMD->mCBW.mCBW_CB_Buf[3]);
                                                        LBA = (LBA << 8) | (CMD->mCBW.mCBW_CB_Buf[4]);
                                                        LBA = (LBA << 8) | (CMD->mCBW.mCBW_CB_Buf[5]);

                                                        printf("xRead10 LBA: %X Length: %X\n", LBA,
                                                               Usb1BytesLeft);
                                                        len = (Usb1BytesLeft < ENDP1_SIZE) ?
                                                              Usb1BytesLeft : ENDP1_SIZE;
                                                        // 实际读取数据
                                                        //SendIndex=findLBA(LBA);
                                                        //printf("SendIndex: %d(D)\n",SendIndex);
                                                        SPINorPosition = LBA * DISK_SECTOR_SIZE;
                                                        if (LBA < MAXSPINORADDRESS / DISK_SECTOR_SIZE) // 如果读取落在 SPI NOR 访问，直接访问 SPI NOR
                                                        {
																if (SPINorPosition==0x1800) { //如果读取位置在 0x1800
																    printf("RD TH %x\n",Usb1Ep1INBuf[1]);
																	MEMCPY(Usb1Ep1INBuf,&TranHeader,sizeof(TranHeader)); // 发送 TranHeader
																	SPINorPosition = SPINorPosition + len;
																} else {
																printf("Read SPI NOR @%X\n", SPINorPosition);
                                                                ReadSPINorData(SPINorPosition, len,
                                                                               Usb1Ep1INBuf);
                                                                SPINorPosition = SPINorPosition + len;
																}
                                                        }
                                                        else
                                                        {
                                                                // 处理 PSRAM 的读取
                                                                if (LBA>= (MAXDISKADDRESS-MAXSPSRAMADDRESS) / DISK_SECTOR_SIZE)
                                                                {
																		printf("Read PSRAM %d\n",SPINorPosition - (MAXDISKADDRESS-MAXSPSRAMADDRESS));
                                                                        ReadPSRAMData_SPI0(
                                                                                SPINorPosition - (MAXDISKADDRESS-MAXSPSRAMADDRESS),
                                                                                len,
                                                                                Usb1Ep1INBuf);
                                                                        SPINorPosition = SPINorPosition + len;
                                                                }
                                                                else
                                                                {
                                                                        // 其余的用 0 回复
                                                                        MEMSET(Usb1Ep1INBuf, 0, len);
                                                                        SPINorPosition = 0xFFFFFFFF;
                                                                }

                                                        }

                                                        Usb1BytesLeft = Usb1BytesLeft - len;
                                                        Usb1BytesSend = Usb1BytesSend + len;
                                                        R16_UEP1_T_LEN1 = len;
                                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1
                                                                            & ~ MASK_UEP_T_RES) | UEP_T_RES_ACK;
                                                        //printf("Send: %X Left %X\n",len,BytesLeft);
                                                }
                                                if (CMD->mCBW.mCBW_CB_Buf[0] == 0x2A)   // WRITE 10
                                                {
                                                        Usb1CurrentRequest = 0x2A;   //当前是 WRITE 10
                                                        Usb1BytesLeft = CMD->mCBW.mCBW_DataLen; //要接收的数据长度

                                                        LBA = (CMD->mCBW.mCBW_CB_Buf[2]);
                                                        LBA = (LBA << 8) | (CMD->mCBW.mCBW_CB_Buf[3]);
                                                        LBA = (LBA << 8) | (CMD->mCBW.mCBW_CB_Buf[4]);
                                                        LBA = (LBA << 8) | (CMD->mCBW.mCBW_CB_Buf[5]);

                                                        printf("U1Write10 LBA: %lX Length: %lX %X\n", LBA,
                                                               Usb1BytesLeft,TranHeader.Flag);
                                                        //printf("id:0x%04x\n", SPIFlash_ReadID());

                                                        // 输出 Status Register
                                                        //printf("Status1:0x%01x\n", ReadStatus1());
                                                        //printf("Status2:0x%01x\n", ReadStatus2());
                                                        //printf("Status3:0x%01x\n", ReadStatus3());

                                                        SPINorPosition = LBA * DISK_SECTOR_SIZE;
														
														// 检查要写入的目标是是否在SPI NOR上
														if (LBA < MAXSPINORADDRESS / DISK_SECTOR_SIZE) {
															// 检查是否只读和4K尺寸
															if (((TranHeader.Flag&0x02)==0x02)&&(Usb1BytesLeft == 4 * 1024)) {
																WriteEnable();
																WaitExternalFlashIfBusy();
																WriteStatus1(0x02);
																WaitExternalFlashIfBusy();
																
                                                                printf("Erase Sector\n");

                                                                SectorErase(SPINorPosition);
															} else {
																if ((SPINorPosition==0x1800)&&(Usb1BytesLeft==512)) { //如果写入位置在 0x1800 并且只写入 512Bytes
																	printf("WR TH\n");
																	SPINorPosition = 0xFFFFFFFE; //特别标记
																} else {
																	// 只接收数据，不处理
																	SPINorPosition = 0xFFFFFFFF;
																}
															}
														} else {
															// 不在在 PSRAM 上
															if ((SPINorPosition < MAXDISKADDRESS-MAXSPSRAMADDRESS)) {
																// 只接收数据，不处理
																SPINorPosition = 0xFFFFFFFF;
															}
														}
														
														/*
                                                        // 如果写入目标在SPINOR 范围，并且以4K写入，那么直接写入
                                                        if ((LBA < MAXSPINORADDRESS / DISK_SECTOR_SIZE)
                                                                        && (Usb1BytesLeft == 4 * 1024)
																		&& (ReadOnly==FALSE))
                                                        {
															                                                        // 在所有的写入操作之前都要设置起来 WEL
																WriteEnable();
																WaitExternalFlashIfBusy();
																WriteStatus1(0x02);
																WaitExternalFlashIfBusy();
																
                                                                printf("Erase Sector\n");

                                                                SectorErase(SPINorPosition);
                                                        }
                                                        else
                                                        {
                                                                if ((SPINorPosition
                                                                                < MAXSPINORADDRESS + MAXSPSRAMADDRESS)||)
                                                                {
                                                                        SPINorPosition = 0xFFFFFFFF;
                                                                }
                                                        }
														*/
                                                        // 这里 SPINorPosition 会指向 SPINOR 或者 PSRAM 的位置
                                                }
                                        }
                                }
                        }
                        if (intstatus == (UIS_TOKEN_IN | 4)) /* endpoint 4 上传 */
                        {
                                R8_UEP4_TX_CTRL1 = R8_UEP4_TX_CTRL1 ^ 0x08;     //手动翻转
                                R8_UEP4_TX_CTRL1 = (R8_UEP4_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                   | UEP_T_RES_NAK;
                        }
                        if (intstatus == (UIS_TOKEN_IN | 3)) /* endpoint 3 上传 */
                        {
                                R8_UEP3_TX_CTRL1 = (R8_UEP3_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                   | UEP_T_RES_NAK;
                        }
                        if (intstatus == (UIS_TOKEN_IN | 2)) /* endpoint 2 上传 */
                        {
                                R8_UEP2_TX_CTRL1 = (R8_UEP2_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                   | UEP_T_RES_NAK;
                        }
                        if (intstatus == (UIS_TOKEN_IN | 1)) /* endpoint 1 上传 */
                        {
                                //printf("In Toaken\n");
                                /*printf("OUTBUFFERB:");
                                 for (i=0; i<20; i++)
                                 {
                                 printf("%X ",Usb1Ep1Buffer[i]);
                                 }
                                 printf("\n");*/
                                //R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES) | UEP_T_RES_NAK;
                                if (Usb1CurrentRequest == 0x12)
                                {
                                        printf("Tag12: %X\n", CMD->mCBW.mCBW_Tag);
                                        MEMCPY(&InquirySts[4], &CMD->mCBW.mCBW_Tag, 4);
                                        MEMCPY( Usb1Ep1INBuf, &InquirySts[0], sizeof(InquirySts));

                                        R16_UEP1_T_LEN1 = sizeof(InquirySts);
                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                           | UEP_T_RES_ACK;
                                        Usb1CurrentRequest = 0x0;
                                }
                                else if (Usb1CurrentRequest == 0x23)
                                {
                                        printf("Tag23: %X\n", CMD->mCBW.mCBW_Tag);
                                        MEMCPY(&RDFMCAPSts[4], &CMD->mCBW.mCBW_Tag, 4);
                                        MEMCPY( Usb1Ep1INBuf, &RDFMCAPSts[0], sizeof(RDFMCAPSts)); /* 加载上传数据 */
                                        R16_UEP1_T_LEN1 = sizeof(RDFMCAPSts);
                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                           | UEP_T_RES_ACK;
                                        Usb1CurrentRequest = 0x0;
                                }
                                else if (Usb1CurrentRequest == 0x25)      // READ CAPACITY
                                {
                                        printf("U1Tag25: %X\n", CMD->mCBW.mCBW_Tag);
                                        MEMCPY(&RDCAPSts[4], &CMD->mCBW.mCBW_Tag, 4);
                                        MEMCPY( Usb1Ep1INBuf, &RDCAPSts[0], sizeof(RDCAPSts)); /* 加载上传数据 */
                                        R16_UEP1_T_LEN1 = sizeof(RDCAPSts);
                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                           | UEP_T_RES_ACK;
                                        Usb1CurrentRequest = 0x0;
                                }
                                else if (Usb1CurrentRequest == 0x1A)     // MODE SENSE 6
                                {
                                        printf("Tag1A: %X\n", CMD->mCBW.mCBW_Tag);
                                        MEMCPY(&MODESENE6Sts[4], &CMD->mCBW.mCBW_Tag, 4);
                                        MEMCPY( Usb1Ep1INBuf, &MODESENE6Sts[0],
                                                sizeof(MODESENE6Sts)); /* 加载上传数据 */
                                        R16_UEP1_T_LEN1 = sizeof(MODESENE6Sts);
                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                           | UEP_T_RES_ACK;
                                        Usb1CurrentRequest = 0x0;
                                }
                                else if (Usb1CurrentRequest == 0x28)     // READ 10
                                {
                                        if (Usb1BytesLeft != 0)
                                        {
                                                len = (Usb1BytesLeft < ENDP1_SIZE) ?
                                                      Usb1BytesLeft : ENDP1_SIZE;
                                                // 实际写入数据
                                                // 有可能收到请求 LBA 0, Length=8096 这种超过1个扇区的请求
                                                //SendIndex=findLBA(LBA+(BytesSend / DISK_SECTOR_SIZE));
                                                //printf("SendIndex2: %d(D)\n",SendIndex);
                                                if (SPINorPosition == 0xFFFFFFFF) // 如果访问超过SPINOR+PSRAM 范围
                                                {
                                                        // 直接返回 0
                                                        MEMSET( Usb1Ep1INBuf, 0, len);
                                                }
                                                else
                                                {
                                                        // 要访问 SPINOR
                                                        if (SPINorPosition < MAXSPINORADDRESS)
                                                        {
                                                                ReadSPINorData(SPINorPosition, len,
                                                                               Usb1Ep1INBuf);
                                                        }
                                                        else
                                                        {
                                                                // 访问 PSRAM
                                                                ReadPSRAMData_SPI0(
                                                                        SPINorPosition - (MAXDISKADDRESS-MAXSPSRAMADDRESS), len,
                                                                        Usb1Ep1INBuf);
                                                        }

                                                        SPINorPosition = SPINorPosition + len;
                                                }
                                                Usb1BytesLeft = Usb1BytesLeft - len;
                                                Usb1BytesSend = Usb1BytesSend + len;

                                                R16_UEP1_T_LEN1 = len;
                                                R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                                   | UEP_T_RES_ACK;
                                                //printf("Send: %X Left %X\n",len,BytesLeft);
                                        }
                                        else
                                        {
                                                //printf("Tag28: %X\n",CMD->mCBW.mCBW_Tag);
                                                MEMCPY(&READ10REPLY[4], &CMD->mCBW.mCBW_Tag, 4);
                                                MEMCPY( Usb1Ep1INBuf, &READ10REPLY[0],
                                                        sizeof(READ10REPLY)); /* 加载上传数据 */
                                                R16_UEP1_T_LEN1 = sizeof(READ10REPLY);
                                                R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                                   | UEP_T_RES_ACK;
                                                Usb1CurrentRequest = 0;
                                        }
                                }
                                else if (Usb1CurrentRequest == 0x2A)     // WRITE 10
                                {
                                        // 回复WRITE10REPLY 表示已经完成
                                        MEMCPY(&WRITE10REPLY[4], &CMD->mCBW.mCBW_Tag, 4);
                                        MEMCPY( Usb1Ep1INBuf, &WRITE10REPLY[0],
                                                sizeof(WRITE10REPLY)); /* 加载上传数据 */
                                        R16_UEP1_T_LEN1 = sizeof(READ10REPLY);
                                        R8_UEP1_TX_CTRL1 = (R8_UEP1_TX_CTRL1 & ~ MASK_UEP_T_RES)
                                                           | UEP_T_RES_ACK;
                                        Usb1CurrentRequest = 0;
                                        // 输出 Status Register
                                        // printf("Status1:0x%01x\n",ReadStatus1());
                                        // printf("Status2:0x%01x\n",ReadStatus2());
                                        // printf("Status3:0x%01x\n",ReadStatus3());
                                }

                        }
                        if (intstatus == (UIS_TOKEN_SETUP)) /* endpoint 0# SETUP */
                        {
                                USB1Dev_EDP0_Setup_Deal();
                        }
                        if (intstatus == (UIS_TOKEN_IN)) /* endpoint 0# IN */
                        {
                                USB1Dev_EDP0_IN_Deal();
                        }
                        if (intstatus == (UIS_TOKEN_OUT)) /* endpoint 0# OUT */
                        {
                                USB1Dev_EDP0_OUT_Deal();
                        }
                }
                R8_USB1_INT_FG = UIF_TRANSFER;
        }
        else if ( R8_USB1_INT_FG & UIF_SUSPEND)     // USB总线挂起/唤醒完成
        {
                R8_USB1_INT_FG = UIF_SUSPEND;
                printf("Suspend\n");
        }
        else if ( R8_USB1_INT_FG & UIF_BUS_RST)       //USB设备复位
        {
                R8_UEP0_TX_CTRL1 = UEP_T_RES_NAK;
                R8_UEP0_RX_CTRL1 = UEP_R_RES_ACK;
                R8_UEP1_TX_CTRL1 = UEP_T_RES_NAK | bUEP_AUTO_TOG;
                R8_UEP1_RX_CTRL1 = UEP_R_RES_ACK | bUEP_AUTO_TOG;
                R8_UEP2_TX_CTRL1 = UEP_T_RES_NAK | bUEP_AUTO_TOG;
                R8_UEP2_RX_CTRL1 = UEP_R_RES_ACK | bUEP_AUTO_TOG;
                R8_UEP3_TX_CTRL1 = UEP_T_RES_NAK | bUEP_AUTO_TOG;
                R8_UEP3_RX_CTRL1 = UEP_R_RES_ACK | bUEP_AUTO_TOG;
                R8_UEP4_TX_CTRL1 = UEP_T_RES_NAK;
                R8_UEP4_RX_CTRL1 = UEP_R_RES_ACK;
                R8_USB1_DEV_AD = 0;
                Usb1DevAddr = 0;
                R8_USB1_INT_FG = UIF_BUS_RST;
                printf("Reset!\n");
        }
        else if ( R8_USB1_INT_FG & UIF_FIFO_OV)       //FIFO溢出中断
        {
                R8_USB1_INT_FG = UIF_FIFO_OV;
                printf("FIFOOV!\n");
        }
}

/*******************************************************************************
 * Function Name  : USB1Dev_EDP0_Setup_Deal
 * Description    : USB1 设备模式端点0 setup处理
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void USB1Dev_EDP0_Setup_Deal(void)
{
        UINT16 len;
        UINT8 status, i;
        UINT8 buf[8];
        static UINT8V UsbInterCfg;

        /* 获取SETUP包 */
        len = R16_USB1_RX_LEN;
        if (len == sizeof(USB_SETUP_REQ))
        {
//        for(i=0; i<8; i++)    printf("%02x ", Usb1Ep0Buffer[i]);    printf("\n");

                Usb1SetupLen = (UINT16) (Usb1SetupBuf->wLengthH) << 8
                               | (Usb1SetupBuf->wLengthL);        //请求数据长度
                Usb1SetupReqCode = Usb1SetupBuf->bRequest;                         //命令码

                /* 分析并处理当前的SETUP包 */
                len = 0;                                                 // 默认为成功并且上传0长度
                status = 0;
                if (( Usb1SetupBuf->bRequestType & USB_REQ_TYP_MASK)
                                != USB_REQ_TYP_STANDARD) /* 非标准请求 */
                {
                        // 处理 Get Max LUN
                        if ((Usb1SetupBuf->bRequestType == 0xA1)
                                        && (Usb1SetupReqCode == GET_MAX_LUN))
                        {
                                printf("SETUP GET_MAX_LUN\n");
                                Usb1Ep0Buffer[0] = 0;
                                R16_UEP0_T_LEN1 = 1;
                                R8_UEP0_TX_CTRL1 = UEP_DATA1 | UEP_T_RES_ACK;
                                return;
                        }

                        status = 0xFF;  // 操作失败
                }
                else   /* 标准请求 */
                {
                        switch (Usb1SetupReqCode)            // 请求码
                        {
                        case USB_GET_DESCRIPTOR:
                                switch ( Usb1SetupBuf->wValueH)
                                {
                                case 0x01:                                              // 设备描述符
                                        Usb1pDescr = (PUINT8) (&MyDevDescr[0]);
                                        len = sizeof(MyDevDescr);
                                        break;
                                case 0x02:                                              // 配置描述符
                                        Usb1pDescr = (PUINT8) (&MyCfgDescr[0]);
                                        len = sizeof(MyCfgDescr);
                                        printf("Send Config desc\n");
                                        break;
                                case 0x03:                                  // 字符串描述符
                                        switch ( Usb1SetupBuf->wValueL)
                                        {
                                        case 1:
                                                //厂商字符串描述符
                                                printf("iManu\n");
                                                printf(&MyManuInfo[0]);
                                                Usb1pDescr = (PUINT8) (&MyManuInfo[0]);
                                                len = sizeof(MyManuInfo);
                                                break;
                                        case 2:                                //产品字符串描述符
                                                printf("iProduct\n");
                                                printf(&MyProdInfo[0]);
                                                Usb1pDescr = (PUINT8) (&MyProdInfo[0]);
                                                len = sizeof(MyProdInfo);
                                                break;
                                        case 0:                                //语言描述符
                                                printf("iLang\n");
                                                Usb1pDescr = (PUINT8) (&MyLangDescr[0]);
                                                len = sizeof(MyLangDescr);
                                                break;
                                        case 3:
                                                printf("iSerial\n");
                                                printf(&MySerial[0]);
                                                Usb1pDescr = (PUINT8) (&MySerial[0]);
                                                len = sizeof(MySerial);
                                                break;
                                        case 4:
                                                printf("String4\n");
                                                Usb1pDescr = (PUINT8) (&String4[0]);
                                                len = sizeof(String4);
                                                break;
                                        case 5:
                                                printf("String5\n");
                                                Usb1pDescr = (PUINT8) (&String5[0]);
                                                len = sizeof(String5);
                                                break;
                                        default:
                                                printf("Not support\n");
                                                status = 0xFF;  // 不支持的字符串描述符
                                                break;
                                        }
                                        break;

                                case USB_DESCR_TYP_QUALIF:
                                        printf("USB_DESCR_TYP_QUALIF\n");
                                        status = 0xFF;
                                        break;
                                default:
                                        status = 0xFF;  // 不支持的描述符类型
                                        break;
                                }
                                break;
                        case USB_SET_ADDRESS:
                                Usb1DevAddr = Usb1SetupBuf->wValueL;  // 暂存USB设备地址
                                break;
                        case USB_GET_CONFIGURATION:
                                buf[0] = Usb1Configed;
                                Usb1pDescr = buf;
                                if (Usb1SetupLen >= 1)
                                        len = 1;
                                break;
                        case USB_SET_CONFIGURATION:
                                Usb1Configed = Usb1SetupBuf->wValueL;
                                break;
                        case USB_CLEAR_FEATURE:
                                if (( Usb1SetupBuf->bRequestType & USB_REQ_RECIP_MASK)
                                                == USB_REQ_RECIP_ENDP)    // 清除端点
                                {
                                        switch ( Usb1SetupBuf->wIndexL)          //复位同步触发位，清除STALL
                                        {
                                        case 0x82:
                                                R8_UEP2_TX_CTRL1 = R8_UEP2_TX_CTRL1
                                                                   & ~( MASK_UEP_T_RES)| UEP_T_RES_NAK;
                                                break;
                                        case 0x02:
                                                R8_UEP2_RX_CTRL1 = R8_UEP2_RX_CTRL1
                                                                   & ~( MASK_UEP_R_RES)| UEP_R_RES_ACK;
                                                break;
                                        case 0x81:
                                                R8_UEP1_TX_CTRL1 = R8_UEP1_TX_CTRL1
                                                                   & ~( MASK_UEP_T_RES)| UEP_T_RES_NAK;
                                                break;
                                        case 0x01:
                                                R8_UEP1_RX_CTRL1 = R8_UEP1_RX_CTRL1
                                                                   & ~( MASK_UEP_R_RES)| UEP_R_RES_ACK;
                                                break;
                                        default:
                                                status = 0xFF;  // 不支持的端点
                                                break;
                                        }
                                }
                                else if (( Usb1SetupBuf->bRequestType & USB_REQ_RECIP_MASK)
                                                == USB_REQ_RECIP_DEVICE)         //清除设备
                                {
                                        /* 设备休眠 */
                                }
                                else
                                        status = 0xFF;  // 不是端点不支持
                                break;
                        case USB_SET_FEATURE:
                                if (( Usb1SetupBuf->bRequestType & USB_REQ_RECIP_MASK)
                                                == USB_REQ_RECIP_ENDP)    // 设置端点
                                {
                                        switch ((UINT16) (Usb1SetupBuf->wIndexH) << 8
                                                        | (Usb1SetupBuf->wIndexL))         //复位同步触发位，清除STALL
                                        {
                                        case 0x82:
                                                R8_UEP2_TX_CTRL1 |= UEP_T_RES_STALL;
                                                break;
                                        case 0x02:
                                                R8_UEP2_RX_CTRL1 |= UEP_T_RES_STALL;
                                                break;
                                        case 0x81:
                                                R8_UEP1_TX_CTRL1 |= UEP_T_RES_STALL;
                                                break;
                                        case 0x01:
                                                R8_UEP1_RX_CTRL1 |= UEP_T_RES_STALL;
                                                break;
                                        default:
                                                status = 0xFF;  // 不支持的端点
                                                break;
                                        }
                                }
                                else if (( Usb1SetupBuf->bRequestType & USB_REQ_RECIP_MASK)
                                                == USB_REQ_RECIP_DEVICE)         //设置设备
                                {
                                        /* 设备唤醒 */
                                }
                                else
                                        status = 0xFF;  // 不是端点不支持
                                break;
                        case USB_SET_INTERFACE:
                                UsbInterCfg = Usb1SetupBuf->wIndexL;
                                break;
                        case USB_GET_INTERFACE:
                                buf[0] = UsbInterCfg;
                                Usb1pDescr = buf;
                                if (Usb1SetupLen >= 1)
                                        len = 1;
                                break;
                        case USB_GET_STATUS:
                                if (( Usb1SetupBuf->bRequestType & USB_REQ_RECIP_MASK)
                                                == USB_REQ_RECIP_ENDP)    // 获取端点状态
                                {
                                        switch ((UINT16) (Usb1SetupBuf->wIndexH) << 8
                                                        | (Usb1SetupBuf->wIndexL))
                                        {
                                        case 0x82:
                                                buf[0] =
                                                        ((R8_UEP2_TX_CTRL1 & 0x11) == UEP_T_RES_STALL) ?
                                                        0x01 : 0x00;
                                                break;
                                        case 0x02:
                                                buf[0] =
                                                        ((R8_UEP2_RX_CTRL1 & 0x11) == UEP_T_RES_STALL) ?
                                                        0x01 : 0x00;
                                                break;
                                        case 0x81:
                                                buf[0] =
                                                        ((R8_UEP1_TX_CTRL1 & 0x11) == UEP_T_RES_STALL) ?
                                                        0x01 : 0x00;
                                                break;
                                        case 0x01:
                                                buf[0] =
                                                        ((R8_UEP1_RX_CTRL1 & 0x11) == UEP_T_RES_STALL) ?
                                                        0x01 : 0x00;
                                                break;
                                        default:
                                                status = 0xFF;  // 不支持的端点
                                                break;
                                        }
                                        buf[1] = 0x00;
                                        Usb1pDescr = buf;
                                        len = 2;
                                }
                                else if (( Usb1SetupBuf->bRequestType & USB_REQ_RECIP_MASK)
                                                == USB_REQ_RECIP_DEVICE)      // 获取设备状态
                                {
                                        buf[0] = 0x00;        //填充当前设备状态
                                        buf[1] = 0x00;
                                        Usb1pDescr = buf;
                                        len = 2;
                                }
                                break;
                        default:
                                status = 0xFF;  // 操作失败
                                break;
                        }
                }
        }
        else
        {
                status = 0xFF;      // SETUP包长度错误
        }
        if (status == 0xFF)    // 操作失败
        {
                Usb1SetupReqCode = 0xFF;
                //R8_UEP0_RX_CTRL1 = UEP_DATA1 | UEP_R_RES_STALL;
                R8_UEP0_TX_CTRL1 = UEP_DATA1 | UEP_T_RES_STALL;
        }
        else
        {
                Usb1SetupLen = (Usb1SetupLen < len) ? Usb1SetupLen : len;   //这次传输上传数据长度
                if (Usb1SetupLen)
                {
                        if (Usb1SetupBuf->bRequestType & 0x80)                        //上传数据
                        {
                                len = (Usb1SetupLen < THIS_ENDP0_SIZE) ?
                                      Usb1SetupLen : THIS_ENDP0_SIZE;        //本次数据上传长度
                                MEMCPY(Usb1Ep0Buffer, Usb1pDescr, len); /* 加载上传数据 */
                                R16_UEP0_T_LEN1 = len;
                                R8_UEP0_TX_CTRL1 = UEP_DATA1 | UEP_T_RES_ACK;     // 默认数据包是DATA1
                                Usb1SetupLen -= len;
                                Usb1pDescr += len;
                        }
                        else                                                          //下传数据
                        {
                                R16_UEP0_T_LEN1 = 0;  // 虽然尚未到状态阶段，但是提前预置上传0长度数据包以防主机提前进入状态阶段
                                R8_UEP0_RX_CTRL1 = UEP_DATA1 | UEP_R_RES_ACK;     // 默认数据包是DATA1
                        }
                }
                else
                {
                        R16_UEP0_T_LEN1 = 0;
                        R8_UEP0_TX_CTRL1 = UEP_DATA1 | UEP_T_RES_ACK;        //状态阶段
                }
        }
}

/*******************************************************************************
 * Function Name  : USB1Dev_EDP0_IN_Deal
 * Description    : USB1 设备模式端点0 IN处理
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void USB1Dev_EDP0_IN_Deal(void)
{
        UINT16 len;
        switch (Usb1SetupReqCode)
        {
        case USB_GET_DESCRIPTOR:
                printf("USB_GET_DESCRIPTOR");
                len = (Usb1SetupLen < THIS_ENDP0_SIZE) ? Usb1SetupLen : THIS_ENDP0_SIZE; //本次数据上传长度
                MEMCPY(Usb1Ep0Buffer, Usb1pDescr, len); /* 加载上传数据 */
                R16_UEP0_T_LEN1 = len;
                R8_UEP0_TX_CTRL1 ^= UEP_DATA1;  // 翻转
                Usb1SetupLen -= len;
                Usb1pDescr += len;
                break;
        case USB_SET_ADDRESS:
                R16_UEP0_T_LEN1 = 0;
                R8_UEP0_TX_CTRL1 = UEP_DATA0 | UEP_T_RES_NAK;
                R8_USB1_DEV_AD = Usb1DevAddr;
                break;
        default:
                R16_UEP0_T_LEN1 = 0;
                R8_UEP0_TX_CTRL1 = UEP_DATA0 | UEP_T_RES_NAK;
                R8_UEP0_RX_CTRL1 = UEP_DATA0 | UEP_R_RES_ACK;
                break;
        }
}

/*******************************************************************************
 * Function Name  : USB1Dev_EDP0_OUT_Deal
 * Description    : USB1 设备模式端点0 OUT处理
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
UINT16 USB1Dev_EDP0_OUT_Deal(void)
{
        UINT16 len;
        len = R16_USB1_RX_LEN;

        return (len);
}
