// 2016.04.26.16:46  2016.05.26.14:44
/* Define for CH568/CH567/CH566/CH565  */
/* Website:  http://wch.cn  */
/* Email:    tech@wch.cn    */
/* Author:   W.ch 2016.10   */
/* V1.0 SpecialFunctionRegister */

// multi-blocks: __BASE_TYPE__, __CH568GPSFR_H__, __CH56USBSFR_H__, __USB_TYPE__, __CH568SATASFR_H__, __SATA_TYPE__, __CH568SDC4SFR_H__, __CH568ECDCSFR_H__, __SDIO_TYPE__,

#ifndef __BASE_TYPE__
#define __BASE_TYPE__

#ifdef __cplusplus
extern "C" {
#endif

/* ********************************************************************************************************************* */
/* Base types & constants */

#ifndef TRUE
#define TRUE                    1
#define FALSE                   0
#endif
#ifndef NULL
#define NULL                    0
#endif

#ifndef VOID
#define VOID                    void
#endif
#ifndef CONST
#define CONST                   const
#endif
#ifndef BOOL
typedef unsigned char           BOOL;
#endif
#ifndef BOOLEAN
typedef unsigned char           BOOLEAN;
#endif
#ifndef CHAR
typedef char                    CHAR;
#endif
#ifndef INT8
typedef char                    INT8;
#endif
#ifndef INT16
typedef short                   INT16;
#endif
#ifndef INT32
typedef long                    INT32;
#endif
#ifndef UINT8
typedef unsigned char           UINT8;
#endif
#ifndef UINT16
typedef unsigned short          UINT16;
#endif
#ifndef UINT32
typedef unsigned long           UINT32;
#endif
#ifndef UINT8V
typedef unsigned char volatile  UINT8V;
#endif
#ifndef UINT16V
typedef unsigned short volatile UINT16V;
#endif
#ifndef UINT32V
typedef unsigned long volatile  UINT32V;
#endif

#ifndef PVOID
typedef void                    *PVOID;
#endif
#ifndef PCHAR
typedef char                    *PCHAR;
#endif
#ifndef PCHAR
typedef const char              *PCCHAR;
#endif
#ifndef PINT8
typedef char                    *PINT8;
#endif
#ifndef PINT16
typedef short                   *PINT16;
#endif
#ifndef PINT32
typedef long                    *PINT32;
#endif
#ifndef PUINT8
typedef unsigned char           *PUINT8;
#endif
#ifndef PUINT16
typedef unsigned short          *PUINT16;
#endif
#ifndef PUINT32
typedef unsigned long           *PUINT32;
#endif
#ifndef PUINT8V
typedef volatile unsigned char  *PUINT8V;
#endif
#ifndef PUINT16V
typedef volatile unsigned short *PUINT16V;
#endif
#ifndef PUINT32V
typedef volatile unsigned long  *PUINT32V;
#endif

/* ********************************************************************************************************************* */
/* Base macros */

#define	FREQ_SYS			96000000						// 系统时钟


#ifndef min
#define min(a,b)                (((a) < (b)) ? (a) : (b))
#endif
#ifndef max
#define max(a,b)                (((a) > (b)) ? (a) : (b))
#endif

#if		DEBUG
#define PRINT(X...) printf(X)
#else
#define PRINT(X...)
#endif

/* Calculate the byte offset of a field in a structure of type */
#define FIELD_OFFSET(Type, Field)    ((UINT16)&(((Type *)0)->Field))

/* Calculate the size of a field in a structure of type */
#define FIELD_SIZE(Type, Field)      (sizeof(((Type *)0)->Field))

/* An expression that yields the type of a field in a struct */
#define FIELD_TYPE(Type, Field)      (((Type *)0)->Field)

/* Return the number of elements in a statically sized array */
#define NUMBER_OF(Array)             (sizeof(Array)/sizeof((Array)[0]))
#define NUMBER_OF_FIELD(Type, Field) (NUMBER_OF(FIELD_TYPE(Type, Field)))

#ifdef __cplusplus
}
#endif

#endif  // __BASE_TYPE__


#ifndef __CH568GPSFR_H__
#define __CH568GPSFR_H__

#ifdef __cplusplus
extern "C" {
#endif

/* ********************************************************************************************************************* */

// Address Space
//    CODE:   00000000H - 0007FFFFH   256K/512KB/1MB
//    DATA:   00200000H - 00207FFFH   32KB/1MB
//    SFR:    00400000H - 0040FFFFH   64KB/1MB
//    BUS:    00800000H - 00807FFFH   32KB/1MB
//
//    SFR:    00400000H - 0040FFFFH, 64KB
//      SYS:     +1000H - 13FFH, include base configuration, interrupt, GPIO, etc...
//      TMR0:    +2000H - 23FFH
//      TMR1:    +2400H - 27FFH
//      TMR2:    +2800H - 2BFFH
//      UART0:   +3000H - 33FFH
//      UART1:   +3400H - 37FFH
//      UART2:   +3800H - 3BFFH
//      UART3:   +3C00H - 3FFFH
//      SPI0:    +4000H - 43FFH
//      SPI1:    +4400H - 47FFH
//      PWMX:    +5000H - 53FFH
//      LED:     +6000H - 63FFH
//      USB0:    +8000H - 83FFH
//      USB1:    +9000H - 93FFH
//      SDC:     +A000H - A3FFH
//      SATA:    +B000H - B3FFH
//      ECDC1:   +C400H - C7FFH, SM4/AES

// Register Bit Attribute / Bit Access Type
//   RF:    Read only for Fixed value
//   RO:    Read Only (internal change)
//   RZ:    Read only with auto clear Zero
//   WO:    Write Only (read zero or different)
//   WA:    Write only under safe Accessing mode (read zero or different)
//   WZ:    Write only with auto clear Zero
//   RW:    Read / Write
//   RWA:   Read / Write under safe Accessing mode
//   RW1:   Read / Write 1 to Clear

/* Register name rule:
   R32_* for 32 bits register (UINT32,ULONG)
   R16_* for 16 bits register (UINT16,USHORT)
   R8_*  for  8 bits register (UINT8,UCHAR)
   RB_*  for bit or bit mask of 8 bit register
   BA_*  for base address point
   b*    for GPIO bit mask
   Others for register address offset */

/* ********************************************************************************************************************* */

/* System: safe accessing register */
#define R32_SAFE_ACCESS     (*((PUINT32V)0x00401000)) // RW, safe accessing
#define R8_SAFE_ACCESS_SIG  (*((PUINT8V)0x00401000))  // WO, safe accessing sign register, must write 0x57 then 0xA8 to enter safe accessing mode
#define  RB_SAFE_ACC_MODE   0x03                      // RO, current safe accessing mode: 11=safe/unlocked (SAM), other=locked (00..01..10..11)
#define  RB_SAFE_ACC_TIMER  0x70                      // RO, safe accessing timer bit mask (16*clock number)
#define R8_CHIP_ID          (*((PUINT8V)0x00401001))  // RF, chip ID register, always is ID_CH56*
#define R8_SAFE_ACCESS_ID   (*((PUINT8V)0x00401002))  // RF, safe accessing ID register, always 0x02

/* System: global configuration register */
#define R32_GLOBAL_CONFIG   (*((PUINT32V)0x00401004)) // RW, global configuration
#define R8_GLOB_ROM_CFG     (*((PUINT8V)0x00401004))  // RWA, flash ROM configuration, SAM and bit7:6 must write 1:0
#define  RB_ROM_EXT_RE      0x01                      // RO, enable flash ROM being read by external programmer: 0=reading protect, 1=enable read
#define  RB_CODE_RAM_WE     0x02                      // RWA, enable code RAM being write: 0=writing protect, 1=enable write
#define  RB_ROM_DATA_WE     0x04                      // RWA, enable flash ROM data area being erase/write: 0=writing protect, 1=enable program and erase
#define  RB_ROM_CODE_WE     0x08                      // RWA, enable flash ROM code & data area being erase/write: 0=writing protect, 1=enable program and erase
//#define  RB_GLOB_BOOT_APP   0x10                      // RO, boot loader action status
#define R8_RST_BOOT_STAT    (*((PUINT8V)0x00401005))  // RO, reset status and boot/debug status, bit7:6 always are 1:1
#define  RB_RESET_FLAG      0x03                      // RO: recent reset flag
#define  RST_FLAG_SW        0x00
#define  RST_FLAG_POR       0x01
#define  RST_FLAG_PIN       0x03
// RB_RESET_FLAG: recent reset flag
//    00 - software reset, by RB_SOFTWARE_RESET=1 @RB_BOOT_LOADER=0
//    01 - power on reset
//    11 - external manual reset by RST# pin input low
#define  RB_CFG_RESET_EN    0x04                      // RO, manual reset input enable status
#define  RB_CFG_BOOT_EN     0x08                      // RO, boot-loader enable status
#define  RB_CFG_DEBUG_EN    0x10                      // RO, debug enable status
#define  RB_BOOT_LOADER     0x20                      // RO, indicate boot loader status: 0=application status (by software reset), 1=boot loader status
#define R8_RST_WDOG_CTRL    (*((PUINT8V)0x00401006))  // RWA, reset control, SAM and bit7:6 must write 0:1
#define  RB_SOFTWARE_RESET  0x01                      // WA/WZ, global software reset, high action, auto clear
#define R8_GLOB_RESET_KEEP  (*((PUINT8V)0x00401007))  // RW, value keeper during global reset

/* System: clock configuration register */
#define R32_CLOCK_CONFIG    (*((PUINT32V)0x00401008)) // RWA, clock configuration, SAM
#define R8_CLK_PLL_DIV      (*((PUINT8V)0x00401008))  // RWA, output clock divider from PLL, only low 4 bit, SAM and bit7:6 must write 0:1
#define R8_CLK_CFG_CTRL     (*((PUINT8V)0x0040100A))  // RWA, clock control, SAM and bit7:6 must write 1:0
#define  RB_CLK_PLL_SLEEP   0x01                      // RWA, PLL sleep control: 0=PLL run, 1=PLL sleep
#define  RB_CLK_SEL_PLL     0x02                      // RWA, clock source selection: 0=oscillator 30MHz, 1=PLL 480MHz
// OSC_FREQ = 30MHz
// PLL_FREQ = 480MHz
// SRC_FREQ = RB_CLK_SEL_PLL ? PLL_FREQ : OSC_FREQ
// SYS_FREQ = SRC_FREQ / R8_CLK_PLL_DIV, 2MHz ~ 120MHz
// default: SRC_FREQ = 30MHz, SYS_FREQ = 15MHz

/* System: sleep control register */
#define R32_SLEEP_CONTROL   (*((PUINT32V)0x0040100C)) // RWA, sleep control, SAM
#define R8_SLP_CLK_OFF0     (*((PUINT8V)0x0040100C))  // RWA, sleep clock off control byte 0, SAM
#define  RB_SLP_CLK_TMR0    0x01                      // RWA, sleep TMR0 clock
#define  RB_SLP_CLK_TMR1    0x02                      // RWA, sleep TMR1 clock
#define  RB_SLP_CLK_TMR2    0x04                      // RWA, sleep TMR2 clock
#define  RB_SLP_CLK_PWMX    0x08                      // RWA, sleep PWMX clock
#define  RB_SLP_CLK_UART0   0x10                      // RWA, sleep UART0 clock
#define  RB_SLP_CLK_UART1   0x20                      // RWA, sleep UART1 clock
#define  RB_SLP_CLK_UART2   0x40                      // RWA, sleep UART2 clock
#define  RB_SLP_CLK_UART3   0x80                      // RWA, sleep UART3 clock
#define R8_SLP_CLK_OFF1     (*((PUINT8V)0x0040100D))  // RWA, sleep clock off control byte 1, SAM
#define  RB_SLP_CLK_SPI0    0x01                      // RWA, sleep SPI0 clock
#define  RB_SLP_CLK_SPI1    0x02                      // RWA, sleep SPI1 clock
#define  RB_SLP_CLK_SDC     0x04                      // RWA, sleep SDC clock
#define  RB_SLP_CLK_LED     0x08                      // RWA, sleep LED clock
#define  RB_SLP_CLK_USB0    0x10                      // RWA, sleep USB0 clock
#define  RB_SLP_CLK_USB1    0x20                      // RWA, sleep USB1 clock
#define  RB_SLP_CLK_SATA    0x40                      // RWA, sleep SATA clock
#define  RB_SLP_CLK_ECDC    0x80                      // RWA, sleep ECDC clock
#define R8_SLP_WAKE_CTRL    (*((PUINT8V)0x0040100E))  // RWA, wake control, SAM
#define  RB_SLP_USB0_WAKE   0x01                      // RWA, enable USB0 waking
#define  RB_SLP_USB1_WAKE   0x02                      // RWA, enable USB1 waking
#define  RB_SLP_SATA_WAKE   0x04                      // RWA, enable SATA waking
#define  RB_SLP_GPIO_WAKE   0x10                      // RWA, enable GPIO waking
#define R8_SLP_POWER_CTRL   (*((PUINT8V)0x0040100F))  // RWA, power control, SAM
#define  RB_SLP_USB0_PWRDN  0x01                      // RWA, enable USB0 power down
#define  RB_SLP_USB1_PWRDN  0x02                      // RWA, enable USB1 power down
#define  RB_SLP_SATA_PWRDN  0x04                      // RWA, enable SATA power down
#define  RB_SLP_STANDBY     0x80                      // RWA, request standby, set 1 to request and auto clear if accepted

/* System: port and bus configuration register */
#define R32_PORT_CONFIG     (*((PUINT32V)0x00401010)) // RW, port and bus configuration
#define R8_XBUS_CONFIG      (*((PUINT8V)0x00401010))  // RW, external bus configuration
#define  RB_XBUS_ENABLE     0x01                      // RW, external bus enable: 0=disable, 1=enable bRD and bWR, enable PB[7:0] as parallel port
#define  RB_XBUS_ADDR_OE    0x0C                      // RW, external bus address output enable: 00=none, 01=PA[5:0], 10=PA[9:0], 11=PA[14:0]
#define  RB_XBUS_WIDTH      0x30                      // RW, external bus access pulse width: 00=3 clocks, 01=5 clocks, 10= 9 clocks, 11=16 clocks
#define  RB_XBUS_HOLD       0x40                      // RW, external bus hold time: 0=2 clocks, 1=3 clocks
#define  RB_XBUS_SETUP      0x80                      // RW, external bus setup time: 0=2 clocks, 1=3 clocks
#define R8_PORT_PIN         (*((PUINT8V)0x00401012))  // RW, alternate pin control
#define  RB_PIN_TMR1        0x02                      // RW, TMR1 alternate pin enable: 0=TMR1/PWM5/CAP1 on PA[10], 1=TMR1_/PWM5_/CAP1_ on PB[2]
#define  RB_PIN_TMR2        0x04                      // RW, TMR2 alternate pin enable: 0=TMR2/PWM6/CAP2 on PA[11], 1=TMR2_/PWM6_/CAP2_ on PB[11]
#define  RB_PIN_UART0       0x10                      // RW, RXD0/TXD0 alternate pin enable: 0=RXD0/TXD0 on PB[4]/PB[7], 1=RXD0_/TXD0_ on PA[15]/PA[14]
#define  RB_PIN_UART1       0x20                      // RW, RXD1/TXD1 alternate pin enable: 0=RXD1/TXD1 on PA[8]/PA[9], 1=RXD1_/TXD1_ on PB[8]/PB[9]

/* System: GPIO interrupt control register */
#define R32_GPIO_INT_CTRL   (*((PUINT32V)0x0040101C)) // RW, GPIO interrupt control
#define R8_GPIO_INT_STATUS  (*((PUINT8V)0x0040101C))  // RW1, GPIO interrupt flag
#define R8_GPIO_INT_ENABLE  (*((PUINT8V)0x0040101D))  // RW, GPIO interrupt enable
#define R8_GPIO_INT_MODE    (*((PUINT8V)0x0040101E))  // RW, GPIO interrupt mode: 0=level action, 1=edge action
#define R8_GPIO_INT_POLAR   (*((PUINT8V)0x0040101F))  // RW, GPIO interrupt polarity: 0=normal(low_level/fall_edge), 1=invert(high_level/rise_edge)

/* GPIO interrupt control register address offset and bit define */
#define BA_GPIO_INT_CTRL    ((PUINT8V)0x0040101C)     // point GPIO interrupt base address
#define GPIO_INT_STATUS     0x00
#define GPIO_INT_ENABLE     0x01
#define GPIO_INT_MODE       0x02
#define GPIO_INT_POLAR      0x03
#define  RB_GPIO_PA3        0x01
#define  RB_GPIO_PA4        0x02
#define  RB_GPIO_PA6        0x04
#define  RB_GPIO_PA10       0x08
#define  RB_GPIO_PA11       0x10
#define  RB_GPIO_PA12       0x20
#define  RB_GPIO_PB4        0x40
#define  RB_GPIO_PB10       0x80

/* GPIO PA register */
#define R32_PA_DIR          (*((PUINT32V)0x00401040)) // RW, GPIO PA I/O direction: 0=in, 1=out
#define R8_PA_DIR_0         (*((PUINT8V)0x00401040))  // RW, GPIO PA I/O direction byte 0
#define R8_PA_DIR_1         (*((PUINT8V)0x00401041))  // RW, GPIO PA I/O direction byte 1
#define R32_PA_PIN          (*((PUINT32V)0x00401044)) // RO, GPIO PA input
#define R8_PA_PIN_0         (*((PUINT8V)0x00401044))  // RO, GPIO PA input byte 0
#define R8_PA_PIN_1         (*((PUINT8V)0x00401045))  // RO, GPIO PA input byte 1
#define R32_PA_OUT          (*((PUINT32V)0x00401048)) // RW, GPIO PA output
#define R8_PA_OUT_0         (*((PUINT8V)0x00401048))  // RW, GPIO PA output byte 0
#define R8_PA_OUT_1         (*((PUINT8V)0x00401049))  // RW, GPIO PA output byte 1
#define R32_PA_CLR          (*((PUINT32V)0x0040104C)) // WZ, GPIO PA clear output: 0=keep, 1=clear
#define R8_PA_CLR_0         (*((PUINT8V)0x0040104C))  // WZ, GPIO PA clear output byte 0
#define R8_PA_CLR_1         (*((PUINT8V)0x0040104D))  // WZ, GPIO PA clear output byte 1
#define R32_PA_PU           (*((PUINT32V)0x00401050)) // RW, GPIO PA pullup resistance enable
#define R8_PA_PU_0          (*((PUINT8V)0x00401050))  // RW, GPIO PA pullup resistance enable byte 0
#define R8_PA_PU_1          (*((PUINT8V)0x00401051))  // RW, GPIO PA pullup resistance enable byte 1
#define R32_PA_PD           (*((PUINT32V)0x00401054)) // RW, GPIO PA output open-drain & input pulldown resistance enable
#define R8_PA_PD_0          (*((PUINT8V)0x00401054))  // RW, GPIO PA output open-drain & input pulldown resistance enable byte 0
#define R8_PA_PD_1          (*((PUINT8V)0x00401055))  // RW, GPIO PA output open-drain & input pulldown resistance enable byte 1
#define R32_PA_DRV          (*((PUINT32V)0x00401058)) // RW, GPIO PA driving capability: 0=8mA, 1=16mA
#define R8_PA_DRV_0         (*((PUINT8V)0x00401058))  // RW, GPIO PA driving capability byte 0
#define R8_PA_DRV_1         (*((PUINT8V)0x00401059))  // RW, GPIO PA driving capability byte 1
#define R32_PA_SMT          (*((PUINT32V)0x0040105C)) // RW, GPIO PA output slew rate & input schmitt trigger: 0=fast,normal, 1=slow,schmitt
#define R8_PA_SMT_0         (*((PUINT8V)0x0040105C))  // RW, GPIO PA output slew rate & input schmitt trigger byte 0
#define R8_PA_SMT_1         (*((PUINT8V)0x0040105D))  // RW, GPIO PA output slew rate & input schmitt trigger byte 1

/* GPIO PB register */
#define R32_PB_DIR          (*((PUINT32V)0x00401060)) // RW, GPIO PB I/O direction: 0=in, 1=out
#define R8_PB_DIR_0         (*((PUINT8V)0x00401060))  // RW, GPIO PB I/O direction byte 0
#define R8_PB_DIR_1         (*((PUINT8V)0x00401061))  // RW, GPIO PB I/O direction byte 1
#define R32_PB_PIN          (*((PUINT32V)0x00401064)) // RO, GPIO PB input
#define R8_PB_PIN_0         (*((PUINT8V)0x00401064))  // RO, GPIO PB input byte 0
#define R8_PB_PIN_1         (*((PUINT8V)0x00401065))  // RO, GPIO PB input byte 1
#define R32_PB_OUT          (*((PUINT32V)0x00401068)) // RW, GPIO PB output
#define R8_PB_OUT_0         (*((PUINT8V)0x00401068))  // RW, GPIO PB output byte 0
#define R8_PB_OUT_1         (*((PUINT8V)0x00401069))  // RW, GPIO PB output byte 1
#define R32_PB_CLR          (*((PUINT32V)0x0040106C)) // WZ, GPIO PB clear output: 0=keep, 1=clear
#define R8_PB_CLR_0         (*((PUINT8V)0x0040106C))  // WZ, GPIO PB clear output byte 0
#define R8_PB_CLR_1         (*((PUINT8V)0x0040106D))  // WZ, GPIO PB clear output byte 1
#define R32_PB_PU           (*((PUINT32V)0x00401070)) // RW, GPIO PB pullup resistance enable
#define R8_PB_PU_0          (*((PUINT8V)0x00401070))  // RW, GPIO PB pullup resistance enable byte 0
#define R8_PB_PU_1          (*((PUINT8V)0x00401071))  // RW, GPIO PB pullup resistance enable byte 1
#define R32_PB_PD           (*((PUINT32V)0x00401074)) // RW, GPIO PB output open-drain & input pulldown resistance enable
#define R8_PB_PD_0          (*((PUINT8V)0x00401074))  // RW, GPIO PB output open-drain & input pulldown resistance enable byte 0
#define R8_PB_PD_1          (*((PUINT8V)0x00401075))  // RW, GPIO PB output open-drain & input pulldown resistance enable byte 1
#define R32_PB_DRV          (*((PUINT32V)0x00401078)) // RW, GPIO PB driving capability: 0=8mA, 1=16mA
#define R8_PB_DRV_0         (*((PUINT8V)0x00401078))  // RW, GPIO PB driving capability byte 0
#define R8_PB_DRV_1         (*((PUINT8V)0x00401079))  // RW, GPIO PB driving capability byte 1
#define R32_PB_SMT          (*((PUINT32V)0x0040107C)) // RW, GPIO PB output slew rate & input schmitt trigger: 0=fast,normal, 1=slow,schmitt
#define R8_PB_SMT_0         (*((PUINT8V)0x0040107C))  // RW, GPIO PB output slew rate & input schmitt trigger byte 0
#define R8_PB_SMT_1         (*((PUINT8V)0x0040107D))  // RW, GPIO PB output slew rate & input schmitt trigger byte 1

/* GPIO register address offset and bit define */
#define BA_PA               ((PUINT8V)0x00401040)     // point GPIO PA base address
#define BA_PB               ((PUINT8V)0x00401060)     // point GPIO PB base address
#define GPIO_DIR            0x00
#define GPIO_DIR_0          0x00
#define GPIO_DIR_1          0x01
#define GPIO_PIN            0x04
#define GPIO_PIN_0          0x04
#define GPIO_PIN_1          0x05
#define GPIO_OUT            0x08
#define GPIO_OUT_0          0x08
#define GPIO_OUT_1          0x09
#define GPIO_CLR            0x0C
#define GPIO_CLR_0          0x0C
#define GPIO_CLR_1          0x0D
#define GPIO_PU             0x10
#define GPIO_PU_0           0x10
#define GPIO_PU_1           0x11
#define GPIO_PD             0x14
#define GPIO_PD_0           0x14
#define GPIO_PD_1           0x15
#define GPIO_DRV            0x18
#define GPIO_DRV_0          0x18
#define GPIO_DRV_1          0x19
#define GPIO_SMT            0x1C
#define GPIO_SMT_0          0x1C
#define GPIO_SMT_1          0x1D

/* GPIO alias name */
#define  bSCK1              (1<<0)                    // PA0
#define  bLED0              (1<<0)                    // PA0
#define  bCMD1              (1<<0)                    // PA0
#define  bSDO               (1<<1)                    // PA1
#define  bMOSI1             bSDO
#define  bLED1              (1<<1)                    // PA1
#define  bCMD2              (1<<1)                    // PA1
#define  bSDI               (1<<2)                    // PA2
#define  bMISO1             bSDI
#define  bLED2              (1<<2)                    // PA2
#define  bCMD3              (1<<2)                    // PA2
#define  bINT0              (1<<3)                    // PA3
#define  bTMR0              (1<<3)                    // PA3
#define  bCAP0              bTMR0
#define  bPWM4              bTMR0
#define  bLED3              (1<<3)                    // PA3
#define  bINT1              (1<<4)                    // PA4
#define  bRXD3              (1<<4)                    // PA4
#define  bPWM0              (1<<4)                    // PA4
#define  bLEDC              (1<<4)                    // PA4
#define  bTXD3              (1<<5)                    // PA5
#define  bPWM1              (1<<5)                    // PA5
#define  bINT2              (1<<6)                    // PA6
#define  bRXD2              (1<<6)                    // PA6
#define  bSDCK              (1<<6)                    // PA6
#define  bTXD2              (1<<7)                    // PA7
#define  bCMD0              (1<<7)                    // PA7
#define  bRXD1              (1<<8)                    // PA8
#define  bPWM2              (1<<8)                    // PA8
#define  bSD00              (1<<8)                    // PA8
#define  bTXD1              (1<<9)                    // PA9
#define  bPWM3              (1<<9)                    // PA9
#define  bSD01              (1<<9)                    // PA9
#define  bINT3              (1<<10)                   // PA10
#define  bTMR1              (1<<10)                   // PA10
#define  bCAP1              bTMR1
#define  bPWM5              bTMR1
#define  bSD02              (1<<10)                   // PA10
#define  bINT4              (1<<11)                   // PA11
#define  bTMR2              (1<<11)                   // PA11
#define  bCAP2              bTMR2
#define  bPWM6              bTMR2
#define  bSD03              (1<<11)                   // PA11
#define  bINT5              (1<<12)                   // PA12
#define  bSCS               (1<<12)                   // PA12
#define  bSD10              (1<<12)                   // PA12
#define  bSCK0              (1<<13)                   // PA13
#define  bSD11              (1<<13)                   // PA13
#define  bMOSI              (1<<14)                   // PA14
#define  bTXD0_             (1<<14)                   // PA14
#define  bSD12              (1<<14)                   // PA14
#define  bMISO              (1<<15)                   // PA15
#define  bRXD0_             (1<<15)                   // PA15
#define  bSD13              (1<<15)                   // PA15
#define  bED0BUS            (0xFF<<8)                 // PA8~PA15
#define  bCTS               (1<<0)                    // PB0
#define  bSD20              (1<<0)                    // PB0
#define  bDSR               (1<<1)                    // PB1
#define  bSD21              (1<<1)                    // PB1
#define  bTMR1_             (1<<2)                    // PB2
#define  bCAP1_             bTMR1_
#define  bPWM5_             bTMR1_
#define  bRI                (1<<2)                    // PB2
#define  bSD22              (1<<2)                    // PB2
#define  bDCD               (1<<3)                    // PB3
#define  bSD23              (1<<3)                    // PB3
#define  bINT6              (1<<4)                    // PB4
#define  bRXD0              (1<<4)                    // PB4
#define  bSD30              (1<<4)                    // PB4
#define  bDTR               (1<<5)                    // PB5
#define  bSD31              (1<<5)                    // PB5
#define  bRTS               (1<<6)                    // PB6
#define  bSD32              (1<<6)                    // PB6
#define  bTXD0              (1<<7)                    // PB7
#define  bSD33              (1<<7)                    // PB7
#define  bED1BUS            (0xFF<<0)                 // PB0~PB7
#define  bRXD1_             (1<<8)                    // PB8
#define  bRD                (1<<8)                    // PB8
#define  bTXD1_             (1<<9)                    // PB9
#define  bWR                (1<<9)                    // PB9
#define  bINT7              (1<<10)                   // PB10
#define  bTIO               (1<<10)                   // PB10
#define  bTMR2_             (1<<11)                   // PB11
#define  bCAP2_             bTMR2_
#define  bPWM6_             bTMR2_
#define  bA14               (1<<11)                   // PB11
#define  bA12               (1<<12)                   // PB12
#define  bA13               (1<<13)                   // PB13
#define  bTCK               (1<<15)                   // PB15
#define  bRST               (1<<15)                   // PB15

/* Timer0 register */
#define R32_TMR0_CONTROL    (*((PUINT32V)0x00402000)) // RW, TMR0 control
#define R8_TMR0_CTRL_MOD    (*((PUINT8V)0x00402000))  // RW, TMR0 mode control
#define R8_TMR0_INTER_EN    (*((PUINT8V)0x00402002))  // RW, TMR0 interrupt enable
#define R32_TMR0_STATUS     (*((PUINT32V)0x00402004)) // RW, TMR0 status
#define R8_TMR0_INT_FLAG    (*((PUINT8V)0x00402006))  // RW1, TMR0 interrupt flag
#define R8_TMR0_FIFO_COUNT  (*((PUINT8V)0x00402007))  // RO, TMR0 FIFO count status
#define R32_TMR0_COUNT      (*((PUINT32V)0x00402008)) // RO, TMR0 current count
#define R16_TMR0_COUNT      (*((PUINT16V)0x00402008)) // RO, TMR0 current count
#define R8_TMR0_COUNT       (*((PUINT8V)0x00402008))  // RO, TMR0 current count
#define R32_TMR0_CNT_END    (*((PUINT32V)0x0040200C)) // RW, TMR0 end count value, only low 26 bit
#define R32_TMR0_FIFO       (*((PUINT32V)0x00402010)) // RO/WO, TMR0 FIFO register, only low 26 bit
#define R16_TMR0_FIFO       (*((PUINT16V)0x00402010)) // RO/WO, TMR0 FIFO register
#define R8_TMR0_FIFO        (*((PUINT8V)0x00402010))  // RO/WO, TMR0 FIFO register

/* Timer1 register */
#define R32_TMR1_CONTROL    (*((PUINT32V)0x00402400)) // RW, TMR1 control
#define R8_TMR1_CTRL_MOD    (*((PUINT8V)0x00402400))  // RW, TMR1 mode control
#define R8_TMR1_CTRL_DMA    (*((PUINT8V)0x00402401))  // RW, TMR1 DMA control
#define R8_TMR1_INTER_EN    (*((PUINT8V)0x00402402))  // RW, TMR1 interrupt enable
#define R32_TMR1_STATUS     (*((PUINT32V)0x00402404)) // RW, TMR1 status
#define R8_TMR1_INT_FLAG    (*((PUINT8V)0x00402406))  // RW1, TMR1 interrupt flag
#define R8_TMR1_FIFO_COUNT  (*((PUINT8V)0x00402407))  // RO, TMR1 FIFO count status
#define R32_TMR1_COUNT      (*((PUINT32V)0x00402408)) // RO, TMR1 current count
#define R16_TMR1_COUNT      (*((PUINT16V)0x00402408)) // RO, TMR1 current count
#define R8_TMR1_COUNT       (*((PUINT8V)0x00402408))  // RO, TMR1 current count
#define R32_TMR1_CNT_END    (*((PUINT32V)0x0040240C)) // RW, TMR1 end count value, only low 26 bit
#define R32_TMR1_FIFO       (*((PUINT32V)0x00402410)) // RO/WO, TMR1 FIFO register, only low 26 bit
#define R16_TMR1_FIFO       (*((PUINT16V)0x00402410)) // RO/WO, TMR1 FIFO register
#define R8_TMR1_FIFO        (*((PUINT8V)0x00402410))  // RO/WO, TMR1 FIFO register
#define R32_TMR1_DMA_NOW    (*((PUINT32V)0x00402414)) // RW, TMR1 DMA current address
#define R16_TMR1_DMA_NOW    (*((PUINT16V)0x00402414)) // RW, TMR1 DMA current address
#define R32_TMR1_DMA_BEG    (*((PUINT32V)0x00402418)) // RW, TMR1 DMA begin address
#define R16_TMR1_DMA_BEG    (*((PUINT16V)0x00402418)) // RW, TMR1 DMA begin address
#define R32_TMR1_DMA_END    (*((PUINT32V)0x0040241C)) // RW, TMR1 DMA end address
#define R16_TMR1_DMA_END    (*((PUINT16V)0x0040241C)) // RW, TMR1 DMA end address

/* Timer2 register */
#define R32_TMR2_CONTROL    (*((PUINT32V)0x00402800)) // RW, TMR2 control
#define R8_TMR2_CTRL_MOD    (*((PUINT8V)0x00402800))  // RW, TMR2 mode control
#define R8_TMR2_CTRL_DMA    (*((PUINT8V)0x00402801))  // RW, TMR2 DMA control
#define R8_TMR2_INTER_EN    (*((PUINT8V)0x00402802))  // RW, TMR2 interrupt enable
#define R32_TMR2_STATUS     (*((PUINT32V)0x00402804)) // RW, TMR2 status
#define R8_TMR2_INT_FLAG    (*((PUINT8V)0x00402806))  // RW1, TMR2 interrupt flag
#define R8_TMR2_FIFO_COUNT  (*((PUINT8V)0x00402807))  // RO, TMR2 FIFO count status
#define R32_TMR2_COUNT      (*((PUINT32V)0x00402808)) // RO, TMR2 current count
#define R16_TMR2_COUNT      (*((PUINT16V)0x00402808)) // RO, TMR2 current count
#define R8_TMR2_COUNT       (*((PUINT8V)0x00402808))  // RO, TMR2 current count
#define R32_TMR2_CNT_END    (*((PUINT32V)0x0040280C)) // RW, TMR2 end count value, only low 26 bit
#define R32_TMR2_FIFO       (*((PUINT32V)0x00402810)) // RO/WO, TMR2 FIFO register, only low 26 bit
#define R16_TMR2_FIFO       (*((PUINT16V)0x00402810)) // RO/WO, TMR2 FIFO register
#define R8_TMR2_FIFO        (*((PUINT8V)0x00402810))  // RO/WO, TMR2 FIFO register
#define R32_TMR2_DMA_NOW    (*((PUINT32V)0x00402814)) // RW, TMR2 DMA current address
#define R16_TMR2_DMA_NOW    (*((PUINT16V)0x00402814)) // RW, TMR2 DMA current address
#define R32_TMR2_DMA_BEG    (*((PUINT32V)0x00402818)) // RW, TMR2 DMA begin address
#define R16_TMR2_DMA_BEG    (*((PUINT16V)0x00402818)) // RW, TMR2 DMA begin address
#define R32_TMR2_DMA_END    (*((PUINT32V)0x0040281C)) // RW, TMR2 DMA end address
#define R16_TMR2_DMA_END    (*((PUINT16V)0x0040281C)) // RW, TMR2 DMA end address

/* Timer register address offset and bit define */
#define TMR_FIFO_SIZE       8                         // timer FIFO size (depth)
#define BA_TMR0             ((PUINT8V)0x00402000)     // point TMR0 base address
#define BA_TMR1             ((PUINT8V)0x00402400)     // point TMR1 base address
#define BA_TMR2             ((PUINT8V)0x00402800)     // point TMR2 base address
#define TMR_CTRL_MOD        0
#define  RB_TMR_MODE_IN     0x01                      // RW, timer in mode: 0=timer/PWM, 1=capture/count
#define  RB_TMR_ALL_CLEAR   0x02                      // RW, force clear timer FIFO and count
#define  RB_TMR_COUNT_EN    0x04                      // RW, timer count enable
#define  RB_TMR_OUT_EN      0x08                      // RW, timer output enable
#define  RB_TMR_OUT_POLAR   0x10                      // RW, timer PWM output polarity: 0=default low and high action, 1=default high and low action
#define  RB_TMR_CAP_COUNT   0x10                      // RW, count sub-mode if RB_TMR_MODE_IN=1: 0=capture, 1=count
#define  RB_TMR_PWM_REPEAT  0xC0                      // RW, timer PWM repeat mode: 00=1, 01=4, 10=8, 11-16
#define  RB_TMR_CAP_EDGE    0xC0                      // RW, timer capture edge mode: 00=disable, 01=edge change, 10=fall to fall, 11-rise to rise
#define TMR_CTRL_DMA        1
#define  RB_TMR_DMA_ENABLE  0x01                      // RW, timer1/2 DMA enable
#define  RB_TMR_DMA_LOOP    0x04                      // RW, timer1/2 DMA address loop enable
#define TMR_INTER_EN        2
#define  RB_TMR_IE_CYC_END  0x01                      // RW, enable interrupt for timer capture count timeout or PWM cycle end
#define  RB_TMR_IE_DATA_ACT 0x02                      // RW, enable interrupt for timer capture input action or PWM trigger
#define  RB_TMR_IE_FIFO_HF  0x04                      // RW, enable interrupt for timer FIFO half (capture fifo >=4 or PWM fifo <=3)
#define  RB_TMR_IE_DMA_END  0x08                      // RW, enable interrupt for timer1/2 DMA completion
#define  RB_TMR_IE_FIFO_OV  0x10                      // RW, enable interrupt for timer FIFO overflow
#define TMR_INT_FLAG        6
#define  RB_TMR_IF_CYC_END  0x01                      // RW1, interrupt flag for timer capture count timeout or PWM cycle end
#define  RB_TMR_IF_DATA_ACT 0x02                      // RW1, interrupt flag for timer capture input action or PWM trigger
#define  RB_TMR_IF_FIFO_HF  0x04                      // RW1, interrupt flag for timer FIFO half (capture fifo >=4 or PWM fifo <=3)
#define  RB_TMR_IF_DMA_END  0x08                      // RW1, interrupt flag for timer1/2 DMA completion
#define  RB_TMR_IF_FIFO_OV  0x10                      // RW1, interrupt flag for timer FIFO overflow
#define TMR_FIFO_COUNT      7
#define TMR_COUNT           0x08
#define TMR_CNT_END         0x0C
#define TMR_FIFO            0x10
#define TMR_DMA_NOW         0x14
#define TMR_DMA_BEG         0x18
#define TMR_DMA_END         0x1C

/* UART0 register */
#define R32_UART0_CTRL      (*((PUINT32V)0x00403000)) // RW, UART0 control
#define R8_UART0_MCR        (*((PUINT8V)0x00403000))  // RW, UART0 modem control
#define R8_UART0_IER        (*((PUINT8V)0x00403001))  // RW, UART0 interrupt enable
#define R8_UART0_FCR        (*((PUINT8V)0x00403002))  // RW, UART0 FIFO control
#define R8_UART0_LCR        (*((PUINT8V)0x00403003))  // RW, UART0 line control
#define R32_UART0_STAT      (*((PUINT32V)0x00403004)) // RO, UART0 status
#define R8_UART0_IIR        (*((PUINT8V)0x00403004))  // RO, UART0 interrupt identification
#define R8_UART0_LSR        (*((PUINT8V)0x00403005))  // RO, UART0 line status
#define R8_UART0_MSR        (*((PUINT8V)0x00403006))  // RO, UART0 modem status
#define R32_UART0_FIFO      (*((PUINT32V)0x00403008)) // RW, UART0 data or FIFO port
#define R8_UART0_RBR        (*((PUINT8V)0x00403008))  // RO, UART0 receiver buffer, receiving byte
#define R8_UART0_THR        (*((PUINT8V)0x00403008))  // WO, UART0 transmitter holding, transmittal byte
#define R8_UART0_RFC        (*((PUINT8V)0x0040300A))  // RO, UART0 receiver FIFO count
#define R8_UART0_TFC        (*((PUINT8V)0x0040300B))  // RO, UART0 transmitter FIFO count
#define R32_UART0_SETUP     (*((PUINT32V)0x0040300C)) // RW, UART0 setup
#define R16_UART0_DL        (*((PUINT16V)0x0040300C)) // RW, UART0 divisor latch
#define R8_UART0_DLL        (*((PUINT8V)0x0040300C))  // RW, UART0 divisor latch LSB byte
#define R8_UART0_DLM        (*((PUINT8V)0x0040300D))  // RW, UART0 divisor latch MSB byte
#define R8_UART0_DIV        (*((PUINT8V)0x0040300E))  // RW, UART0 pre-divisor latch byte, only low 7 bit, from 1 to 0/128
#define R8_UART0_ADR        (*((PUINT8V)0x0040300F))  // RW, UART0 slave address: 0xFF=disable, other=enable

/* UART1 register */
#define R32_UART1_CTRL      (*((PUINT32V)0x00403400)) // RW, UART1 control
#define R8_UART1_MCR        (*((PUINT8V)0x00403400))  // RW, UART1 modem control
#define R8_UART1_IER        (*((PUINT8V)0x00403401))  // RW, UART1 interrupt enable
#define R8_UART1_FCR        (*((PUINT8V)0x00403402))  // RW, UART1 FIFO control
#define R8_UART1_LCR        (*((PUINT8V)0x00403403))  // RW, UART1 line control
#define R32_UART1_STAT      (*((PUINT32V)0x00403404)) // RO, UART1 status
#define R8_UART1_IIR        (*((PUINT8V)0x00403404))  // RO, UART1 interrupt identification
#define R8_UART1_LSR        (*((PUINT8V)0x00403405))  // RO, UART1 line status
#define R32_UART1_FIFO      (*((PUINT32V)0x00403408)) // RW, UART1 data or FIFO port
#define R8_UART1_RBR        (*((PUINT8V)0x00403408))  // RO, UART1 receiver buffer, receiving byte
#define R8_UART1_THR        (*((PUINT8V)0x00403408))  // WO, UART1 transmitter holding, transmittal byte
#define R8_UART1_RFC        (*((PUINT8V)0x0040340A))  // RO, UART1 receiver FIFO count
#define R8_UART1_TFC        (*((PUINT8V)0x0040340B))  // RO, UART1 transmitter FIFO count
#define R32_UART1_SETUP     (*((PUINT32V)0x0040340C)) // RW, UART1 setup
#define R16_UART1_DL        (*((PUINT16V)0x0040340C)) // RW, UART1 divisor latch
#define R8_UART1_DLL        (*((PUINT8V)0x0040340C))  // RW, UART1 divisor latch LSB byte
#define R8_UART1_DLM        (*((PUINT8V)0x0040340D))  // RW, UART1 divisor latch MSB byte
#define R8_UART1_DIV        (*((PUINT8V)0x0040340E))  // RW, UART1 pre-divisor latch byte, only low 7 bit, from 1 to 0/128

/* UART2 register */
#define R32_UART2_CTRL      (*((PUINT32V)0x00403800)) // RW, UART2 control
#define R8_UART2_MCR        (*((PUINT8V)0x00403800))  // RW, UART2 modem control
#define R8_UART2_IER        (*((PUINT8V)0x00403801))  // RW, UART2 interrupt enable
#define R8_UART2_FCR        (*((PUINT8V)0x00403802))  // RW, UART2 FIFO control
#define R8_UART2_LCR        (*((PUINT8V)0x00403803))  // RW, UART2 line control
#define R32_UART2_STAT      (*((PUINT32V)0x00403804)) // RO, UART2 status
#define R8_UART2_IIR        (*((PUINT8V)0x00403804))  // RO, UART2 interrupt identification
#define R8_UART2_LSR        (*((PUINT8V)0x00403805))  // RO, UART2 line status
#define R32_UART2_FIFO      (*((PUINT32V)0x00403808)) // RW, UART2 data or FIFO port
#define R8_UART2_RBR        (*((PUINT8V)0x00403808))  // RO, UART2 receiver buffer, receiving byte
#define R8_UART2_THR        (*((PUINT8V)0x00403808))  // WO, UART2 transmitter holding, transmittal byte
#define R8_UART2_RFC        (*((PUINT8V)0x0040380A))  // RO, UART2 receiver FIFO count
#define R8_UART2_TFC        (*((PUINT8V)0x0040380B))  // RO, UART2 transmitter FIFO count
#define R32_UART2_SETUP     (*((PUINT32V)0x0040380C)) // RW, UART2 setup
#define R16_UART2_DL        (*((PUINT16V)0x0040380C)) // RW, UART2 divisor latch
#define R8_UART2_DLL        (*((PUINT8V)0x0040380C))  // RW, UART2 divisor latch LSB byte
#define R8_UART2_DLM        (*((PUINT8V)0x0040380D))  // RW, UART2 divisor latch MSB byte
#define R8_UART2_DIV        (*((PUINT8V)0x0040380E))  // RW, UART2 pre-divisor latch byte, only low 7 bit, from 1 to 0/128

/* UART3 register */
#define R32_UART3_CTRL      (*((PUINT32V)0x00403C00)) // RW, UART3 control
#define R8_UART3_MCR        (*((PUINT8V)0x00403C00))  // RW, UART3 modem control
#define R8_UART3_IER        (*((PUINT8V)0x00403C01))  // RW, UART3 interrupt enable
#define R8_UART3_FCR        (*((PUINT8V)0x00403C02))  // RW, UART3 FIFO control
#define R8_UART3_LCR        (*((PUINT8V)0x00403C03))  // RW, UART3 line control
#define R32_UART3_STAT      (*((PUINT32V)0x00403C04)) // RO, UART3 status
#define R8_UART3_IIR        (*((PUINT8V)0x00403C04))  // RO, UART3 interrupt identification
#define R8_UART3_LSR        (*((PUINT8V)0x00403C05))  // RO, UART3 line status
#define R32_UART3_FIFO      (*((PUINT32V)0x00403C08)) // RW, UART3 data or FIFO port
#define R8_UART3_RBR        (*((PUINT8V)0x00403C08))  // RO, UART3 receiver buffer, receiving byte
#define R8_UART3_THR        (*((PUINT8V)0x00403C08))  // WO, UART3 transmitter holding, transmittal byte
#define R8_UART3_RFC        (*((PUINT8V)0x00403C0A))  // RO, UART3 receiver FIFO count
#define R8_UART3_TFC        (*((PUINT8V)0x00403C0B))  // RO, UART3 transmitter FIFO count
#define R32_UART3_SETUP     (*((PUINT32V)0x00403C0C)) // RW, UART3 setup
#define R16_UART3_DL        (*((PUINT16V)0x00403C0C)) // RW, UART3 divisor latch
#define R8_UART3_DLL        (*((PUINT8V)0x00403C0C))  // RW, UART3 divisor latch LSB byte
#define R8_UART3_DLM        (*((PUINT8V)0x00403C0D))  // RW, UART3 divisor latch MSB byte
#define R8_UART3_DIV        (*((PUINT8V)0x00403C0E))  // RW, UART3 pre-divisor latch byte, only low 7 bit, from 1 to 0/128

/* UART register address offset and bit define */
#define UART_FIFO_SIZE      8                         // UART FIFO size (depth)
#define UART_RECV_RDY_SZ    7                         // the max FIFO trigger level for UART receiver data available
#define BA_UART0            ((PUINT8V)0x00403000)     // point UART0 base address
#define BA_UART1            ((PUINT8V)0x00403400)     // point UART1 base address
#define BA_UART2            ((PUINT8V)0x00403800)     // point UART2 base address
#define BA_UART3            ((PUINT8V)0x00403C00)     // point UART3 base address
#define UART_MCR            0
#define  RB_MCR_DTR         0x01                      // RW, UART0 control DTR
#define  RB_MCR_RTS         0x02                      // RW, UART0 control RTS
#define  RB_MCR_OUT1        0x04                      // RW, UART0 control OUT1
#define  RB_MCR_OUT2        0x08                      // RW, UART control OUT2
#define  RB_MCR_INT_OE      0x08                      // RW, UART interrupt output enable
#define  RB_MCR_LOOP        0x10                      // RW, UART0 enable local loop back
#define  RB_MCR_AU_FLOW_EN  0x20                      // RW, UART0 enable autoflow control
#define  RB_MCR_TNOW        0x40                      // RW, UART0 enable TNOW output on DTR pin
#define  RB_MCR_HALF        0x80                      // RW, UART0 enable half-duplex
#define UART_IER            1
#define  RB_IER_RECV_RDY    0x01                      // RW, UART interrupt enable for receiver data ready
#define  RB_IER_THR_EMPTY   0x02                      // RW, UART interrupt enable for THR empty
#define  RB_IER_LINE_STAT   0x04                      // RW, UART interrupt enable for receiver line status
#define  RB_IER_MODEM_CHG   0x08                      // RW, UART0 interrupt enable for modem status change
#define  RB_IER_DTR_EN      0x10                      // RW, UART0 DTR/TNOW output pin enable
#define  RB_IER_RTS_EN      0x20                      // RW, UART0 RTS output pin enable
#define  RB_IER_TXD_EN      0x40                      // RW, UART TXD pin enable
#define  RB_IER_RESET       0x80                      // WZ, UART software reset control, high action, auto clear
#define UART_FCR            2
#define  RB_FCR_FIFO_EN     0x01                      // RW, UART FIFO enable
#define  RB_FCR_RX_FIFO_CLR 0x02                      // WZ, clear UART receiver FIFO, high action, auto clear
#define  RB_FCR_TX_FIFO_CLR 0x04                      // WZ, clear UART transmitter FIFO, high action, auto clear
#define  RB_FCR_FIFO_TRIG   0xC0                      // RW, UART receiver FIFO trigger level: 00-1byte, 01-2bytes, 10-4bytes, 11-7bytes
#define UART_LCR            3
#define  RB_LCR_WORD_SZ     0x03                      // RW, UART word bit length: 00-5bit, 01-6bit, 10-7bit, 11-8bit
#define  RB_LCR_STOP_BIT    0x04                      // RW, UART stop bit length: 0-1bit, 1-2bit
#define  RB_LCR_PAR_EN      0x08                      // RW, UART parity enable
#define  RB_LCR_PAR_MOD     0x30                      // RW, UART parity mode: 00-odd, 01-even, 10-mark, 11-space
#define  RB_LCR_BREAK_EN    0x40                      // RW, UART break control enable
#define  RB_LCR_DLAB        0x80                      // RW, UART reserved bit
#define  RB_LCR_GP_BIT      0x80                      // RW, UART general purpose bit
#define UART_IIR            4
#define  RB_IIR_NO_INT      0x01                      // RO, UART no interrupt flag: 0=interrupt action, 1=no interrupt
#define  RB_IIR_INT_MASK    0x0F                      // RO, UART interrupt flag bit mask
#define  RB_IIR_FIFO_ID     0xC0                      // RO, UART FIFO enabled flag
#define UART_LSR            5
#define  RB_LSR_DATA_RDY    0x01                      // RO, UART receiver fifo data ready status
#define  RB_LSR_OVER_ERR    0x02                      // RZ, UART receiver overrun error
#define  RB_LSR_PAR_ERR     0x04                      // RZ, UART receiver parity error
#define  RB_LSR_FRAME_ERR   0x08                      // RZ, UART receiver frame error
#define  RB_LSR_BREAK_ERR   0x10                      // RZ, UART receiver break error
#define  RB_LSR_TX_FIFO_EMP 0x20                      // RO, UART transmitter fifo empty status
#define  RB_LSR_TX_ALL_EMP  0x40                      // RO, UART transmitter all empty status
#define  RB_LSR_ERR_RX_FIFO 0x80                      // RO, indicate error in UART receiver fifo
#define UART_MSR            6
#define  RB_MSR_CTS_CHG     0x01                      // RZ, UART0 CTS changed status, high action
#define  RB_MSR_DSR_CHG     0x02                      // RZ, UART0 DSR changed status, high action
#define  RB_MSR_RI_CHG      0x04                      // RZ, UART0 RI changed status, high action
#define  RB_MSR_DCD_CHG     0x08                      // RZ, UART0 DCD changed status, high action
#define  RB_MSR_CTS         0x10                      // RO, UART0 CTS action status
#define  RB_MSR_DSR         0x20                      // RO, UART0 DSR action status
#define  RB_MSR_RI          0x40                      // RO, UART0 RI action status
#define  RB_MSR_DCD         0x80                      // RO, UART0 DCD action status
#define UART_RBR            8
#define UART_THR            8
#define UART_RFC            0x0A
#define UART_TFC            0x0B
#define UART_DLL            0x0C
#define UART_DLM            0x0D
#define UART_DIV            0x0E
#define UART_ADR            0x0F

/* UART interrupt identification values for IIR bits 3:0 */
#define UART_II_SLV_ADDR    0x0E                      // RO, UART0 slave address match
#define UART_II_LINE_STAT   0x06                      // RO, UART interrupt by receiver line status
#define UART_II_RECV_RDY    0x04                      // RO, UART interrupt by receiver data available
#define UART_II_RECV_TOUT   0x0C                      // RO, UART interrupt by receiver fifo timeout
#define UART_II_THR_EMPTY   0x02                      // RO, UART interrupt by THR empty
#define UART_II_MODEM_CHG   0x00                      // RO, UART0 interrupt by modem status change
#define UART_II_NO_INTER    0x01                      // RO, no UART interrupt is pending

/* SPI0 register */
#define R32_SPI0_CONTROL    (*((PUINT32V)0x00404000)) // RW, SPI0 control
#define R8_SPI0_CTRL_MOD    (*((PUINT8V)0x00404000))  // RW, SPI0 mode control
#define R8_SPI0_CTRL_CFG    (*((PUINT8V)0x00404001))  // RW, SPI0 configuration control
#define R8_SPI0_INTER_EN    (*((PUINT8V)0x00404002))  // RW, SPI0 interrupt enable
#define R8_SPI0_CLOCK_DIV   (*((PUINT8V)0x00404003))  // RW, SPI0 master clock divisor
#define R8_SPI0_SLAVE_PRE   (*((PUINT8V)0x00404003))  // RW, SPI0 slave preset value
#define R32_SPI0_STATUS     (*((PUINT32V)0x00404004)) // RW, SPI0 status
#define R8_SPI0_BUFFER      (*((PUINT8V)0x00404004))  // RO, SPI0 data buffer
#define R8_SPI0_RUN_FLAG    (*((PUINT8V)0x00404005))  // RO, SPI0 work flag
#define R8_SPI0_INT_FLAG    (*((PUINT8V)0x00404006))  // RW1, SPI0 interrupt flag
#define R8_SPI0_FIFO_COUNT  (*((PUINT8V)0x00404007))  // RO, SPI0 FIFO count status
#define R32_SPI0_TOTAL_CNT  (*((PUINT32V)0x0040400C)) // RW, SPI0 total byte count, only low 12 bit
#define R16_SPI0_TOTAL_CNT  (*((PUINT16V)0x0040400C)) // RW, SPI0 total byte count, only low 12 bit
#define R32_SPI0_FIFO       (*((PUINT32V)0x00404010)) // RW, SPI0 FIFO register
#define R8_SPI0_FIFO        (*((PUINT8V)0x00404010))  // RO/WO, SPI0 FIFO register
#define R8_SPI0_FIFO_COUNT1 (*((PUINT8V)0x00404013))  // RO, SPI0 FIFO count status
#define R32_SPI0_DMA_NOW    (*((PUINT32V)0x00404014)) // RW, SPI0 DMA current address
#define R16_SPI0_DMA_NOW    (*((PUINT16V)0x00404014)) // RW, SPI0 DMA current address
#define R32_SPI0_DMA_BEG    (*((PUINT32V)0x00404018)) // RW, SPI0 DMA begin address
#define R16_SPI0_DMA_BEG    (*((PUINT16V)0x00404018)) // RW, SPI0 DMA begin address
#define R32_SPI0_DMA_END    (*((PUINT32V)0x0040401C)) // RW, SPI0 DMA end address
#define R16_SPI0_DMA_END    (*((PUINT16V)0x0040401C)) // RW, SPI0 DMA end address

/* SPI1 register */
#define R32_SPI1_CONTROL    (*((PUINT32V)0x00404400)) // RW, SPI1 control
#define R8_SPI1_CTRL_MOD    (*((PUINT8V)0x00404400))  // RW, SPI1 mode control
#define R8_SPI1_CTRL_CFG    (*((PUINT8V)0x00404401))  // RW, SPI1 configuration control
#define R8_SPI1_INTER_EN    (*((PUINT8V)0x00404402))  // RW, SPI1 interrupt enable
#define R8_SPI1_CLOCK_DIV   (*((PUINT8V)0x00404403))  // RW, SPI1 master clock divisor
#define R32_SPI1_STATUS     (*((PUINT32V)0x00404404)) // RW, SPI1 status
#define R8_SPI1_BUFFER      (*((PUINT8V)0x00404404))  // RO, SPI1 data buffer
#define R8_SPI1_RUN_FLAG    (*((PUINT8V)0x00404405))  // RO, SPI1 work flag
#define R8_SPI1_INT_FLAG    (*((PUINT8V)0x00404406))  // RW1, SPI1 interrupt flag
#define R8_SPI1_FIFO_COUNT  (*((PUINT8V)0x00404407))  // RO, SPI1 FIFO count status
#define R32_SPI1_TOTAL_CNT  (*((PUINT32V)0x0040440C)) // RW, SPI1 total byte count, only low 12 bit
#define R16_SPI1_TOTAL_CNT  (*((PUINT16V)0x0040440C)) // RW, SPI1 total byte count, only low 12 bit
#define R32_SPI1_FIFO       (*((PUINT32V)0x00404410)) // RW, SPI1 FIFO register
#define R8_SPI1_FIFO        (*((PUINT8V)0x00404410))  // RO/WO, SPI1 FIFO register
#define R8_SPI1_FIFO_COUNT1 (*((PUINT8V)0x00404413))  // RO, SPI1 FIFO count status

/* SPI register address offset and bit define */
#define SPI_FIFO_SIZE       8                         // SPI FIFO size (depth)
#define BA_SPI0             ((PUINT8V)0x00404000)     // point SPI0 base address
#define BA_SPI1             ((PUINT8V)0x00404400)     // point SPI1 base address
#define SPI_CTRL_MOD        0
#define  RB_SPI_MODE_SLAVE  0x01                      // RW, SPI0 slave mode: 0=master/host, 1=slave/device
#define  RB_SPI_ALL_CLEAR   0x02                      // RW, force clear SPI FIFO and count
#define  RB_SPI_2WIRE_MOD   0x04                      // RW, SPI0 enable 2 wire mode: 0=3wire(SCK,MOSI,MISO), 1=2wire(SCK,MISO=MXSX)
#define  RB_SPI_MST_SCK_MOD 0x08                      // RW, SPI master clock mode: 0=mode 0, 1=mode 3
#define  RB_SPI_SLV_CMD_MOD 0x08                      // RW, SPI0 slave command mode: 0=byte stream, 1=first byte command
#define  RB_SPI_FIFO_DIR    0x10                      // RW, SPI FIFO direction: 0=out(write @master mode), 1=in(read @master mode)
#define  RB_SPI_SCK_OE      0x20                      // RW, SPI SCK output enable
#define  RB_SPI_MOSI_OE     0x40                      // RW, SPI MOSI output enable
#define  RB_SPI1_SDO_OE     0x40                      // RW, SPI1 SDO output enable
#define  RB_SPI_MISO_OE     0x80                      // RW, SPI MISO output enable
#define  RB_SPI1_SDI_OE     0x80                      // RW, SPI1 SDI output enable, SPI1 enable 2 wire mode: 0=3wire(SCK1,SDO,SDI), 1=2wire(SCK1,SDI=SDX)
#define SPI_CTRL_CFG        1
#define  RB_SPI_DMA_ENABLE  0x01                      // RW, SPI0 DMA enable
#define  RB_SPI_DMA_LOOP    0x04                      // RW, SPI0 DMA address loop enable
#define  RB_SPI_AUTO_IF     0x10                      // RW, enable buffer/FIFO accessing to auto clear RB_SPI_IF_BYTE_END interrupt flag
#define  RB_SPI_BIT_ORDER   0x20                      // RW, SPI bit data order: 0=MSB first, 1=LSB first
#define SPI_INTER_EN        2
#define  RB_SPI_IE_CNT_END  0x01                      // RW, enable interrupt for SPI total byte count end
#define  RB_SPI_IE_BYTE_END 0x02                      // RW, enable interrupt for SPI byte exchanged
#define  RB_SPI_IE_FIFO_HF  0x04                      // RW, enable interrupt for SPI FIFO half
#define  RB_SPI_IE_DMA_END  0x08                      // RW, enable interrupt for SPI0 DMA completion
#define  RB_SPI_IE_FIFO_OV  0x10                      // RW, enable interrupt for SPI0 FIFO overflow
#define  RB_SPI_IE_FST_BYTE 0x80                      // RW, enable interrupt for SPI0 slave mode first byte received
#define SPI_CLOCK_DIV       3
#define SPI_SLAVE_PRESET    3
#define SPI_BUFFER          4
#define SPI_RUN_FLAG        5
#define  RB_SPI_SLV_CMD_ACT 0x10                      // RO, SPI0 slave command flag
#define  RB_SPI_FIFO_READY  0x20                      // RO, SPI FIFO ready status
#define  RB_SPI_SLV_CS_LOAD 0x40                      // RO, SPI0 slave chip-select loading status
#define  RB_SPI_SLV_SELECT  0x80                      // RO, SPI0 slave selection status
#define SPI_INT_FLAG        6
#define  RB_SPI_IF_CNT_END  0x01                      // RW1, interrupt flag for SPI total byte count end
#define  RB_SPI_IF_BYTE_END 0x02                      // RW1, interrupt flag for SPI byte exchanged
#define  RB_SPI_IF_FIFO_HF  0x04                      // RW1, interrupt flag for SPI FIFO half (RB_SPI_FIFO_DIR ? >=4bytes : <4bytes)
#define  RB_SPI_IF_DMA_END  0x08                      // RW1, interrupt flag for SPI0 DMA completion
#define  RB_SPI_IF_FIFO_OV  0x10                      // RW1, interrupt flag for SPI0 FIFO overflow
#define  RB_SPI_FREE        0x40                      // RO, current SPI free status
#define  RB_SPI_IF_FST_BYTE 0x80                      // RW1, interrupt flag for SPI0 slave mode first byte received
#define SPI_FIFO_COUNT      7
#define SPI_TOTAL_CNT       0x0C
#define SPI_FIFO            0x10
#define SPI_DMA_NOW         0x14
#define SPI_DMA_BEG         0x18
#define SPI_DMA_END         0x1C

/* PWM0/1/2/3 register */
#define R32_PWM_CONTROL     (*((PUINT32V)0x00405000)) // RW, PWM control
#define R8_PWM_CTRL_MOD     (*((PUINT8V)0x00405000))  // RW, PWM mode control
#define R8_PWM_CTRL_CFG     (*((PUINT8V)0x00405001))  // RW, PWM configuration control
#define R8_PWM_CLOCK_DIV    (*((PUINT8V)0x00405002))  // RW, PWM clock divisor
#define R32_PWM_DATA        (*((PUINT32V)0x00405004)) // RW, PWM data holding
#define R8_PWM0_DATA        (*((PUINT8V)0x00405004))  // RW, PWM0 data holding
#define R8_PWM1_DATA        (*((PUINT8V)0x00405005))  // RW, PWM1 data holding
#define R8_PWM2_DATA        (*((PUINT8V)0x00405006))  // RW, PWM2 data holding
#define R8_PWM3_DATA        (*((PUINT8V)0x00405007))  // RW, PWM3 data holding

/* PWM0/1/2/3 register address offset and bit define */
#define BA_PWMX             ((PUINT8V)0x00405000)     // point PWM0/1/2/3 base address
#define PWM_CTRL_MOD        0
#define  RB_PWM0_OUT_EN     0x01                      // RW, PWM0 output enable
#define  RB_PWM1_OUT_EN     0x02                      // RW, PWM1 output enable
#define  RB_PWM2_OUT_EN     0x04                      // RW, PWM2 output enable
#define  RB_PWM3_OUT_EN     0x08                      // RW, PWM3 output enable
#define  RB_PWM0_POLAR      0x10                      // RW, PWM0 output polarity: 0=default low and high action, 1=default high and low action
#define  RB_PWM1_POLAR      0x20                      // RW, PWM1 output polarity: 0=default low and high action, 1=default high and low action
#define  RB_PWM2_POLAR      0x40                      // RW, PWM2 output polarity: 0=default low and high action, 1=default high and low action
#define  RB_PWM3_POLAR      0x80                      // RW, PWM3 output polarity: 0=default low and high action, 1=default high and low action
#define PWM_CTRL_CFG        1
#define  RB_PWM_CYCLE_SEL   0x01                      // RW, PWM cycle selection: 0=256 clocks, 1=255 clocks
#define PWM_CLOCK_DIV       2
#define PWM0_DATA_HOLD      4
#define PWM1_DATA_HOLD      5
#define PWM2_DATA_HOLD      6
#define PWM3_DATA_HOLD      7

/* LED register */
#define R32_LED_CONTROL     (*((PUINT32V)0x00406000)) // RW, LED control
#define R8_LED_CTRL_MOD     (*((PUINT8V)0x00406000))  // RW, LED mode control
#define R8_LED_CLOCK_DIV    (*((PUINT8V)0x00406001))  // RW, LED serial clock divisor
#define R8_LED_STATUS       (*((PUINT8V)0x00406004))  // RO, LED status
#define R32_LED_FIFO        (*((PUINT32V)0x00406008)) // WO, LED FIFO register, width is half-word, only low 16 bit
#define R16_LED_FIFO        (*((PUINT16V)0x00406008)) // WO, LED FIFO register, width is half-word
#define R32_LED_DMA_CNT     (*((PUINT32V)0x00406010)) // RW, LED DMA main buffer remainder half-word count, exclude auxiliary buffer, automatic decreasing after DMA, only low 12 bit
#define R16_LED_DMA_CNT     (*((PUINT16V)0x00406010)) // RW, LED DMA main buffer remainder half-word count, exclude auxiliary buffer, automatic decreasing after DMA, only low 12 bit
#define R32_LED_DMA_MAIN    (*((PUINT32V)0x00406014)) // RW, LED main buffer DMA begin & current address, automatic increasing after DMA
#define R16_LED_DMA_MAIN    (*((PUINT16V)0x00406014)) // RW, LED main buffer DMA begin & current address, automatic increasing after DMA
#define R32_LED_DMA_AUX     (*((PUINT32V)0x00406018)) // RW, LED auxiliary buffer DMA begin & current address, automatic increasing after DMA
#define R16_LED_DMA_AUX     (*((PUINT16V)0x00406018)) // RW, LED auxiliary buffer DMA begin & current address, automatic increasing after DMA

/* LED register address offset and bit define */
#define LED_FIFO_SIZE       2                         // LED FIFO size (depth), width is half-word
#define BA_LED              ((PUINT8V)0x00406000)     // point LED base address
#define LED_CTRL_MOD        0
#define  RB_LED_BIT_ORDER   0x01                      // RW, LED bit data order: 0=LSB first, 1=MSB first
#define  RB_LED_ALL_CLEAR   0x02                      // RW, force clear LED FIFO and count
#define  RB_LED_OUT_POLAR   0x04                      // RW, LED output polarity: 0=pass, 1=invert
#define  RB_LED_OUT_EN      0x08                      // RW, LED output enable
#define  RB_LED_DMA_EN      0x10                      // RW, LED DMA enable and DMA interrupt enable
#define  RB_LED_IE_FIFO     0x20                      // RW, enable interrupt for LED FIFO <=2
#define  RB_LED_CHAN_MOD    0xC0                      // RW, LED channel mode: 00=LED0, 01=LED0/1, 10=LED0~3, 11=LED0~3 and LED2/3 from auxiliary buffer
// RB_LED_CHAN_MOD: LED channel mode
//   00: single channel output, LED0
//   01: dual channels output, LED0/1
//   10: 4 channels output, LED0~3
//   11: 4 channels output and LED2/3 from aux buffer, LED0~3
#define LED_CLOCK_DIV       1
#define LED_STATUS          4
#define  RB_LED_FIFO_COUNT  0x07                      // RO, LED FIFO byte count status, must divided by 2 for width half-word
#define  RB_LED_CLOCK       0x10                      // RO, current LED clock level
#define  RB_LED_IF_FIFO     0x20                      // RW1, interrupt flag for LED FIFO <=2, cleared by RW1 or write R32/R16_LED_FIFO
#define  RB_LED_FIFO_EMPTY  0x40                      // RO: indicate FIFO empty status
#define  RB_LED_IF_DMA_END  0x80                      // RW1, interrupt flag for LED DMA completion, cleared by RW1 or write R32/R16_LED_DMA_CNT
#define LED_FIFO            8
#define LED_DMA_CNT         0x10
#define LED_DMA_BEG         0x14
#define LED_DMA_END         0x18

/* Address space define */
#define BA_CODE             ((PUINT32)0x00000000)     // point code base address
#define SZ_CODE             0x00080000                // code size
#define BA_SFR              ((PUINT32)0x00400000)     // point SFR base address
#define SZ_SFR              0x00010000                // SFR size
#define BA_RAM              ((PUINT32)0x00200000)     // point RAM base address
#define SZ_RAM              0x00008000                // RAM size
#define BA_XBUS             ((PUINT32)0x00800000)     // point XBUS base address
#define SZ_XBUS             0x00008000                // XBUS size

/*----- Reference Information --------------------------------------------*/
#define ID_CH568            0x68                      // chip ID
#define ID_CH567            0x67                      // chip ID
#define ID_CH566            0x66                      // chip ID
#define ID_CH565            0x65                      // chip ID

/* Interrupt routine address and interrupt number */
#define INT_ID_SW           NDS32_INT_H0  // interrupt number for software
#define INT_ID_TMR0         NDS32_INT_H1  // interrupt number for timer0
#define INT_ID_GPIO         NDS32_INT_H2  // interrupt number for GPIO
#define INT_ID_SPI0         NDS32_INT_H3  // interrupt number for SPI0
#define INT_ID_SATA         NDS32_INT_H4  // interrupt number for SATA
#define INT_ID_USB0         NDS32_INT_H4  // interrupt number for USB0
#define INT_ID_TMR1         NDS32_INT_H5  // interrupt number for timer1
#define INT_ID_TMR2         NDS32_INT_H6  // interrupt number for timer2
#define INT_ID_UART0        NDS32_INT_H7  // interrupt number for UART0
#define INT_ID_USB1         NDS32_INT_H8  // interrupt number for USB1
#define INT_ID_SDC          NDS32_INT_H9  // interrupt number for SDC
#define INT_ID_ECDC         NDS32_INT_H10 // interrupt number for ECDC
#define INT_ID_LED          NDS32_INT_H11 // interrupt number for LED
#define INT_ID_SPI1         NDS32_INT_H12 // interrupt number for SPI1
#define INT_ID_UART1        NDS32_INT_H13 // interrupt number for UART1
#define INT_ID_UART2        NDS32_INT_H14 // interrupt number for UART2
#define INT_ID_UART3        NDS32_INT_H15 // interrupt number for UART3
#define INT_VEC_ENTRY_SZ    16            // size of each interrupt vector entry
#define INT_ADDR_SW         (INT_ID_SW*INT_VEC_ENTRY_SZ+144)    // interrupt vector address for software
#define INT_ADDR_TMR0       (INT_ID_TMR0*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for timer0
#define INT_ADDR_GPIO       (INT_ID_GPIO*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for GPIO
#define INT_ADDR_SPI0       (INT_ID_SPI0*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for SPI0
#define INT_ADDR_SATA       (INT_ID_SATA*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for SATA
#define INT_ADDR_USB0       (INT_ID_USB0*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for USB0
#define INT_ADDR_TMR1       (INT_ID_TMR1*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for timer1
#define INT_ADDR_TMR2       (INT_ID_TMR2*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for timer2
#define INT_ADDR_UART0      (INT_ID_UART0*INT_VEC_ENTRY_SZ+144) // interrupt vector address for UART0
#define INT_ADDR_USB1       (INT_ID_USB1*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for USB1
#define INT_ADDR_SDC        (INT_ID_SDC*INT_VEC_ENTRY_SZ+144)   // interrupt vector address for SDC
#define INT_ADDR_ECDC       (INT_ID_ECDC*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for ECDC
#define INT_ADDR_LED        (INT_ID_LED*INT_VEC_ENTRY_SZ+144)   // interrupt vector address for LED
#define INT_ADDR_SPI1       (INT_ID_SPI1*INT_VEC_ENTRY_SZ+144)  // interrupt vector address for SPI1
#define INT_ADDR_UART1      (INT_ID_UART1*INT_VEC_ENTRY_SZ+144) // interrupt vector address for UART1
#define INT_ADDR_UART2      (INT_ID_UART2*INT_VEC_ENTRY_SZ+144) // interrupt vector address for UART2
#define INT_ADDR_UART3      (INT_ID_UART3*INT_VEC_ENTRY_SZ+144) // interrupt vector address for UART3

#ifdef __cplusplus
}
#endif

#endif  // __CH568GSFR_H__


#ifndef __CH568USBSFR_H__
#define __CH568USBSFR_H__

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************/
/*                         Peripheral memory map                              */
/******************************************************************************/
/* usb addresses                                                              */
#define USB0_BASE_ADDR              (0x00408000)
#define USB1_BASE_ADDR              (0x00409000)


/*       USB0  */
#define		R32_USB0_CONTROL	(*((PUINT32V)(USB0_BASE_ADDR + 0x000)))	// USB control & interrupt enable & device address
#define		R8_USB0_CTRL		(*((PUINT8V)(USB0_BASE_ADDR + 0x000)))	// USB base control
#define		R8_UHOST_CTRL0		(*((PUINT8V)(USB0_BASE_ADDR + 0x001)))	// USB actual speed
#define		R8_USB0_INT_EN		(*((PUINT8V)(USB0_BASE_ADDR + 0x002)))	// USB interrupt enable
#define		R8_USB0_DEV_AD		(*((PUINT8V)(USB0_BASE_ADDR + 0x003)))	// USB device address
#define		R32_USB0_FRAME_NO   (*((PUINT32V)(USB0_BASE_ADDR + 0x004)))
#define 	R16_USB0_SOF_NO   	(*((PUINT16V)(USB0_BASE_ADDR + 0x004)))
#define     R8_USB0_SUSPEND		(*((PUINT8V)(USB0_BASE_ADDR + 0x006)))
#define		R32_USB0_STATUS		(*((PUINT32V)(USB0_BASE_ADDR + 0x008)))	// USB miscellaneous status & interrupt flag & interrupt status
#define		R8_USB0_SPD_TYPE	(*((PUINT8V)(USB0_BASE_ADDR + 0x008)))	// USB actual speed
#define		R8_USB0_MIS_ST		(*((PUINT8V)(USB0_BASE_ADDR + 0x009)))	// USB miscellaneous status
#define		R8_USB0_INT_FG		(*((PUINT8V)(USB0_BASE_ADDR + 0x00a)))	// USB interrupt flag
#define		R8_USB0_INT_ST		(*((PUINT8V)(USB0_BASE_ADDR + 0x00b)))	// USB interrupt status
#define		R16_USB0_RX_LEN		(*((PUINT16V)(USB0_BASE_ADDR + 0x00c)))	// USB receiving length
#define		R32_USB0_BUF_MODE	(*((PUINT32V)(USB0_BASE_ADDR + 0x010)))	// USB endpoint buffer mode
#define		R8_UEP4_1_MOD0		(*((PUINT8V)(USB0_BASE_ADDR + 0x010)))	// endpoint 4/1 mode
#define		R8_UEP2_3_MOD0		(*((PUINT8V)(USB0_BASE_ADDR + 0x011)))	// endpoint 2/3 mode
#define		R8_UH_EP_MOD0		R8_UEP2_3_MOD0	// host endpoint mode
#define		R16_UEP0_DMA0		(*((PUINT16V)(USB0_BASE_ADDR + 0x014)))	// endpoint 0 DMA buffer address
#define		R16_UEP1_DMA0		(*((PUINT16V)(USB0_BASE_ADDR + 0x018)))	// endpoint 1 DMA buffer address
#define		R16_UEP2_DMA0		(*((PUINT16V)(USB0_BASE_ADDR + 0x01c)))	// endpoint 2 DMA buffer address
#define		R16_UEP3_DMA0		(*((PUINT16V)(USB0_BASE_ADDR + 0x020)))	// endpoint 3 DMA buffer address
#define		R16_UH_RX_DMA0		R16_UEP2_DMA0	// host rx endpoint buffer high address
#define		R16_UH_TX_DMA0		R16_UEP3_DMA0	// host tx endpoint buffer high address

#define 	R16_UEP0_MAX_LEN0	(*((PUINT16V)(USB0_BASE_ADDR + 0x024)))
#define		R16_UEP1_MAX_LEN0	(*((PUINT16V)(USB0_BASE_ADDR + 0x028)))
#define		R16_UEP2_MAX_LEN0	(*((PUINT16V)(USB0_BASE_ADDR + 0x02C)))
#define		R16_UEP3_MAX_LEN0	(*((PUINT16V)(USB0_BASE_ADDR + 0x030)))
#define		R16_UEP4_MAX_LEN0	(*((PUINT16V)(USB0_BASE_ADDR + 0x034)))
#define		R16_UH_RX_MAX_LEN0	R16_UEP2_MAX_LEN0	

#define		R32_USB0_EP0_CTRL	(*((PUINT32V)(USB0_BASE_ADDR + 0x038)))	// endpoint 0 control & transmittal length
#define		R16_UEP0_T_LEN0		(*((PUINT16V)(USB0_BASE_ADDR + 0x038)))	// endpoint 0 transmittal length
#define		R8_UEP0_TX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x03a)))	// endpoint 0 tx control
#define		R8_UEP0_RX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x03b)))	// endpoint 0 rx control
#define		R32_USB0_EP1_CTRL	(*((PUINT32V)(USB0_BASE_ADDR + 0x03c)))	// endpoint 1 control & transmittal length
#define		R16_UEP1_T_LEN0		(*((PUINT16V)(USB0_BASE_ADDR + 0x03c)))	// endpoint 1 transmittal length
#define		R8_UEP1_TX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x03e)))	// endpoint 1 control
#define		R8_UEP1_RX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x03f)))	// endpoint 1 control
#define		R8_UH_SETUP0		R8_UEP1_TX_CTRL0	// host aux setup
#define		R32_USB0_EP2_CTRL	(*((PUINT32V)(USB0_BASE_ADDR + 0x040)))	// endpoint 2 control & transmittal length
#define		R16_UEP2_T_LEN0		(*((PUINT16V)(USB0_BASE_ADDR + 0x040)))	// endpoint 2 transmittal length
#define		R8_UEP2_TX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x042)))	// endpoint 2 control
#define		R8_UEP2_RX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x043)))	// endpoint 2 control
#define		R8_UH_EP_PID0		(*((PUINT8V)(USB0_BASE_ADDR + 0x040)))	//R16_UEP2_T_LEN0	// host endpoint and PID
#define		R8_UH_RX_CTRL0		R8_UEP2_RX_CTRL0						// host receiver endpoint control
#define		R32_USB0_EP3_CTRL	(*((PUINT32V)(USB0_BASE_ADDR + 0x044)))	// endpoint 3 control & transmittal length
#define		R16_UEP3_T_LEN0		(*((PUINT16V)(USB0_BASE_ADDR + 0x044)))	// endpoint 3 transmittal length
#define		R8_UEP3_TX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x046)))	// endpoint 3 control
#define		R8_UEP3_RX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x047)))	// endpoint 3 control
#define		R16_UH_TX_LEN0		R16_UEP3_T_LEN0							// host transmittal endpoint transmittal length
#define		R8_UH_TX_CTRL0		R8_UEP3_TX_CTRL0						// host transmittal endpoint control
#define		R32_USB0_EP4_CTRL	(*((PUINT32V)(USB0_BASE_ADDR + 0x048)))	// endpoint 4 control & transmittal length
#define		R16_UEP4_T_LEN0		(*((PUINT16V)(USB0_BASE_ADDR + 0x048)))	// endpoint 4 transmittal length
#define		R8_UEP4_TX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x04a)))	// endpoint 4 control
#define		R8_UEP4_RX_CTRL0	(*((PUINT8V)(USB0_BASE_ADDR + 0x04b)))	// endpoint 4 control
#define     R16_UH_SPLIT_DATA0   R16_UEP4_T_LEN0	

/*       USB1  */
#define		R32_USB1_CONTROL	(*((PUINT32V)(USB1_BASE_ADDR + 0x000)))	// USB control & interrupt enable & device address
#define		R8_USB1_CTRL		(*((PUINT8V)(USB1_BASE_ADDR + 0x000)))	// USB base control
#define		R8_UHOST_CTRL1		(*((PUINT8V)(USB1_BASE_ADDR + 0x001)))	// USB actual speed
#define		R8_USB1_INT_EN		(*((PUINT8V)(USB1_BASE_ADDR + 0x002)))	// USB interrupt enable
#define		R8_USB1_DEV_AD		(*((PUINT8V)(USB1_BASE_ADDR + 0x003)))	// USB device address
#define		R32_USB1_FRAME_NO   (*((PUINT32V)(USB1_BASE_ADDR + 0x004)))
#define 	R16_USB1_SOF_NO   	(*((PUINT16V)(USB1_BASE_ADDR + 0x004)))
#define     R8_USB1_SUSPEND		(*((PUINT8V)(USB1_BASE_ADDR + 0x006)))
#define		R32_USB1_STATUS		(*((PUINT32V)(USB1_BASE_ADDR + 0x008)))	// USB miscellaneous status & interrupt flag & interrupt status
#define		R8_USB1_SPD_TYPE	(*((PUINT8V)(USB1_BASE_ADDR + 0x008)))	// USB actual speed
#define		R8_USB1_MIS_ST		(*((PUINT8V)(USB1_BASE_ADDR + 0x009)))	// USB miscellaneous status
#define		R8_USB1_INT_FG		(*((PUINT8V)(USB1_BASE_ADDR + 0x00a)))	// USB interrupt flag
#define		R8_USB1_INT_ST		(*((PUINT8V)(USB1_BASE_ADDR + 0x00b)))	// USB interrupt status
#define		R16_USB1_RX_LEN		(*((PUINT16V)(USB1_BASE_ADDR + 0x00c)))	// USB receiving length
#define		R32_USB1_BUF_MODE	(*((PUINT32V)(USB1_BASE_ADDR + 0x010)))	// USB endpoint buffer mode
#define		R8_UEP4_1_MOD1		(*((PUINT8V)(USB1_BASE_ADDR + 0x010)))	// endpoint 4/1 mode
#define		R8_UEP2_3_MOD1		(*((PUINT8V)(USB1_BASE_ADDR + 0x011)))	// endpoint 2/3 mode
#define		R8_UH_EP_MOD1		R8_UEP2_3_MOD1							// host endpoint mode
#define		R16_UEP0_DMA1		(*((PUINT16V)(USB1_BASE_ADDR + 0x014)))	// endpoint 0 DMA buffer address
#define		R16_UEP1_DMA1		(*((PUINT16V)(USB1_BASE_ADDR + 0x018)))	// endpoint 1 DMA buffer address
#define		R16_UEP2_DMA1		(*((PUINT16V)(USB1_BASE_ADDR + 0x01c)))	// endpoint 2 DMA buffer address
#define		R16_UEP3_DMA1		(*((PUINT16V)(USB1_BASE_ADDR + 0x020)))	// endpoint 3 DMA buffer address
#define		R16_UH_RX_DMA1		R16_UEP2_DMA1							// host rx endpoint buffer high address
#define		R16_UH_TX_DMA1		R16_UEP3_DMA1							// host tx endpoint buffer high address

#define 	R16_UEP0_MAX_LEN1	(*((PUINT16V)(USB1_BASE_ADDR + 0x024)))
#define		R16_UEP1_MAX_LEN1	(*((PUINT16V)(USB1_BASE_ADDR + 0x028)))
#define		R16_UEP2_MAX_LEN1	(*((PUINT16V)(USB1_BASE_ADDR + 0x02C)))
#define		R16_UEP3_MAX_LEN1	(*((PUINT16V)(USB1_BASE_ADDR + 0x030)))
#define		R16_UEP4_MAX_LEN1	(*((PUINT16V)(USB1_BASE_ADDR + 0x034)))
#define		R16_UH_RX_MAX_LEN1	R16_UEP2_MAX_LEN1	

#define		R32_USB1_EP0_CTRL	(*((PUINT32V)(USB1_BASE_ADDR + 0x038)))	// endpoint 0 control & transmittal length
#define		R16_UEP0_T_LEN1		(*((PUINT16V)(USB1_BASE_ADDR + 0x038)))	// endpoint 0 transmittal length
#define		R8_UEP0_TX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x03a)))	// endpoint 0 tx control
#define		R8_UEP0_RX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x03b)))	// endpoint 0 rx control
#define		R32_USB1_EP1_CTRL	(*((PUINT32V)(USB1_BASE_ADDR + 0x03c)))	// endpoint 1 control & transmittal length
#define		R16_UEP1_T_LEN1		(*((PUINT16V)(USB1_BASE_ADDR + 0x03c)))	// endpoint 1 transmittal length
#define		R8_UEP1_TX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x03e)))	// endpoint 1 control
#define		R8_UEP1_RX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x03f)))	// endpoint 1 control
#define		R8_UH_SETUP1		R8_UEP1_TX_CTRL1	// host aux setup
#define		R32_USB1_EP2_CTRL	(*((PUINT32V)(USB1_BASE_ADDR + 0x040)))	// endpoint 2 control & transmittal length
#define		R16_UEP2_T_LEN1		(*((PUINT16V)(USB1_BASE_ADDR + 0x040)))	// endpoint 2 transmittal length
#define		R8_UEP2_TX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x042)))	// endpoint 2 control
#define		R8_UEP2_RX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x043)))	// endpoint 2 control
#define		R8_UH_EP_PID1		(*((PUINT8V)(USB1_BASE_ADDR + 0x040)))	//R16_UEP2_T_LEN0	// host endpoint and PID
#define		R8_UH_RX_CTRL1		R8_UEP2_RX_CTRL1						// host receiver endpoint control
#define		R32_USB1_EP3_CTRL	(*((PUINT32V)(USB1_BASE_ADDR + 0x044)))	// endpoint 3 control & transmittal length
#define		R16_UEP3_T_LEN1		(*((PUINT16V)(USB1_BASE_ADDR + 0x044)))	// endpoint 3 transmittal length
#define		R8_UEP3_TX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x046)))	// endpoint 3 control
#define		R8_UEP3_RX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x047)))	// endpoint 3 control
#define		R16_UH_TX_LEN1		R16_UEP3_T_LEN1							// host transmittal endpoint transmittal length
#define		R8_UH_TX_CTRL1		R8_UEP3_TX_CTRL1						// host transmittal endpoint control
#define		R32_USB1_EP4_CTRL	(*((PUINT32V)(USB1_BASE_ADDR + 0x048)))	// endpoint 4 control & transmittal length
#define		R16_UEP4_T_LEN1		(*((PUINT16V)(USB1_BASE_ADDR + 0x048)))	// endpoint 4 transmittal length
#define		R8_UEP4_TX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x04a)))	// endpoint 4 control
#define		R8_UEP4_RX_CTRL1	(*((PUINT8V)(USB1_BASE_ADDR + 0x04b)))	// endpoint 4 control
#define     R16_UH_SPLIT_DATA1   R16_UEP4_T_LEN1

/*  USB register address offset and bit define */


/* R8_USB0_SPD_TYPE / R8_USB1_SPD_TYPE */
#define UST_FS 0x00  // USB_SPD_TYPE is full speed
#define UST_HS 0x01  // USB_SPD_TYPE is high speed
#define UST_LS 0x02  // USB_SPD_TYPE is low speed


/* R8_USB0_SUSPEND / R8_USB1_SUSPEND */
//#define bUS_WAKE_UP 0x04  // wake up from suspend
#define bUS_RESUME  0x02  //remote resume
#define bUS_SUSPEND 0x01


/* R8_UEP0_TX_CTRL0 / R8_UEP0_TX_CTRL1 / R8_UEP1_TX_CTRL0 / R8_UEP1_TX_CTRL1 / R8_UEP2_TX_CTRL0 / R8_UEP2_TX_CTRL1 / R8_UEP3_TX_CTRL0 / R8_UEP3_TX_CTRL1 / R8_UEP4_TX_CTRL0 / R8_UEP4_TX_CTRL1  */
#define bUEP_AUTO_TOG     0x20      // enable automatic toggle after successful transfer completion on endpoint 1/2/3: 0=manual toggle, 1=automatic toggle
#define MASK_UEP_T_TOG    0x18      // prepared data toggle flag of USB endpoint X transmittal (IN): 00=DATA0, 01=DATA1, 10=DATA2, 11=MDATA
#define bUEP_T_RES_NO     0x04      // expected no response, 1=enable, 0=disable, for non-zero endpoint isochronous transactions
#define MASK_UEP_T_RES    0x03      // bit mask of handshake response type for USB endpoint X transmittal (IN)

#define UEP_T_RES_ACK     0x00
#define UEP_T_RES_NAK     0x02
#define UEP_T_RES_STALL   0x03
// bUEP_T_RES1 & bUEP_T_RES0: handshake response type for USB endpoint X transmittal (IN)
//   00: ACK (ready) 
//   10: NAK (busy)
//   11: STALL (error)


/* R8_UEP0_RX_CTRL0 / R8_UEP0_RX_CTRL1 / R8_UEP1_RX_CTRL0 / R8_UEP1_RX_CTRL1 / R8_UEP2_RX_CTRL0 / R8_UEP2_RX_CTRL1 / R8_UEP3_RX_CTRL0 / R8_UEP3_RX_CTRL1 / R8_UEP4_RX_CTRL0 / R8_UEP4_RX_CTRL1  */
//#define bUEP_AUTO_TOG     0x20    // enable automatic toggle after successful transfer completion on endpoint 1/2/3: 0=manual toggle, 1=automatic toggle
#define MASK_UEP_R_TOG    0x18      // expected data toggle flag of USB endpoint X receiving (OUT): 00s=DATA0, 01=DATA1, 10=DATA2, 11=MDATA
#define bUEP_R_RES_NO     0x04      // prepared no response, 1=enable, 0=disable, for non-zero endpoint isochronous transactions
#define MASK_UEP_R_RES    0x03      // bit mask of handshake response type for USB endpoint X receiving (OUT)

#define UEP_R_RES_ACK     0x00
#define UEP_R_RES_NAK     0x02
#define UEP_R_RES_STALL   0x03
#define UEP_R_RES_NYET	  0x01
// bUEP_R_RES1 & bUEP_R_RES0: handshake response type for USB endpoint X receiving (OUT)
//   00: ACK (ready) 
//   10: NAK (busy)
//   11: STALL (error)
//   01: NYET

// bUEP_T_TOG1 & bUEP_T_TOG0 / bUEP_R_TOG1 & bUEP_R_TOG0 : toggle flag
#define UEP_DATA0   (0x00<<3)
#define UEP_DATA1   (0x01<<3)
#define UEP_DATA2   (0x02<<3)
#define UEP_MDATA	(0x03<<3)


/* USB interrupt flag registers:  R8_USB0_INT_FG  R8_USB1_INT_FG */
#define UIF_FIFO_OV   0x10 // FIFO overflow interrupt flag for USB, direct bit address clear or write 1 to clear
#define UIF_HST_SOF   0x08 // host SOF timer interrupt flag for USB host, direct bit address clear or write 1 to clear
#define UIF_SUSPEND   0x04 // USB suspend or resume event interrupt flag, direct bit address clear or write 1 to clear
#define UIF_TRANSFER  0x02 // USB transfer completion interrupt flag, direct bit address clear or write 1 to clear
#define UIF_DETECT    0x01 // device detected event interrupt flag for USB host mode, direct bit address clear or write 1 to clear
#define UIF_BUS_RST   0x01 // bus reset event interrupt flag for USB device mode, direct bit address clear or write 1 to clear

/* ReadOnly, USB interrupt status registers:  R8_USB0_INT_ST / R8_USB1_INT_ST */
#define bUIS_IS_NAK       0x80      // ReadOnly: indicate current USB transfer is NAK received for USB device mode
#define bUIS_TOG_OK       0x40      // ReadOnly: indicate current USB transfer toggle is OK
#define MASK_UIS_TOKEN    0x30      // ReadOnly: bit mask of current token PID code received for USB device mode
#define MASK_UIS_ENDP     0x0F      // ReadOnly: bit mask of current transfer endpoint number for USB device mode
#define MASK_UIS_H_RES    0x0F      // ReadOnly: bit mask of current transfer handshake response for USB host mode: 0000=no response, time out from device, others=handshake response PID received

#define UIS_TOKEN_OUT     0x00
#define UIS_TOKEN_SOF     0x10
#define UIS_TOKEN_IN      0x20
#define UIS_TOKEN_SETUP   0x30
// bUIS_TOKEN1 & bUIS_TOKEN0: current token PID code received for USB device mode
//   00: OUT token PID received
//   01: SOF token PID received
//   10: IN token PID received
//   11: SETUP token PID received


/* ReadOnly, USB miscellaneous status registers:  R8_USB0_MIS_ST / R8_USB1_MIS_ST */
#define bUMS_SOF_PRES     0x80      // ReadOnly: indicate host SOF timer presage status
#define bUMS_SOF_ACT      0x40      // ReadOnly: indicate host SOF timer action status for USB host
#define bUMS_SIE_FREE     0x20      // ReadOnly: indicate USB SIE free status
#define bUMS_R_FIFO_RDY   0x10      // ReadOnly: indicate USB receiving FIFO ready status (not empty)
#define bUMS_BUS_RESET    0x08      // ReadOnly: indicate USB bus reset status
#define bUMS_SUSPEND      0x04      // ReadOnly: indicate USB suspend status
#define bUMS_ATTACH    	  0x02      // ReadOnly: indicate device attached status on USB hub DP/DM
#define bUMS_SPLIT_CAN    0x01      // ReadOnly: indicate host allow SPLIT packet

/* USB interrupt enable registers: R8_USB0_INT_EN / R8_USB1_INT_EN */
#define bUIE_DEV_SOF      0x80      // enable interrupt for SOF received for USB device mode
#define bUIE_DEV_NAK      0x40      // enable interrupt for NAK responded for USB device mode
#define bUIE_FIFO_OV      0x10      // enable interrupt for FIFO overflow
#define bUIE_HST_SOF      0x08      // enable interrupt for host SOF timer action for USB host mode
#define bUIE_SUSPEND      0x04      // enable interrupt for USB suspend or resume event
#define bUIE_TRANSFER     0x02      // enable interrupt for USB transfer completion
#define bUIE_DETECT       0x01      // enable interrupt for USB device detected event for USB host mode
#define bUIE_BUS_RST      0x01      // enable interrupt for USB bus reset event for USB device mode

/* USB base control registers: R8_USB0_CTRL / R8_USB1_CTRL */
#define bUC_HOST_MODE     0x80      // enable USB host mode: 0=device mode, 1=host mode
#define UC_SPEED_TYPE     0x60      // enable USB low speed: 00=full speed, 01=high speed, 10 =low speed
#define bUC_DEV_PU_EN     0x10      // USB device enable and internal pullup resistance enable
#define bUC_INT_BUSY      0x08      // enable automatic responding busy for device mode or automatic pause for host mode during interrupt flag UIF_TRANSFER valid
#define bUC_RESET_SIE     0x04      // force reset USB SIE, need software clear
#define bUC_CLR_ALL       0x02      // force clear FIFO and count of USB
#define bUC_DMA_EN        0x01      // DMA enable and DMA interrupt enable for USB

//UC_SPEED_TYPE
#define UCST_FS 0x00
#define UCST_HS 0x20
#define UCST_LS 0x40



/* USB host setup registers:  R8_UH_SETUP0 / R8_UH_SETUP1 */
#define bUH_SOF_EN        0x40      // USB host automatic SOF enable

/* USB host RX control registers: R8_UH_RX_CTRL0 / R8_UH_RX_CTRL1  */
#define bUH_R_DATA_NO   0x40       // expect no data packet, for high speed hub in host mode
#define bUH_R_AUTO_TOG	0x20       // enable automatic toggle after successful receiver completion: 0=manual toggle, 1=automatic toggle
#define MASK_UH_R_TOG	0x18       // expected data toggle flag of host receiving (IN): 00=DATA0, 01=DATA1, 10=DATA2, 11=MDATA
#define bUH_R_RES_NO	0x04       // prepared no response, 1=enable, 0=disable, for non-zero endpoint isochronous transactions
#define MASK_UH_R_RES   0x03       // prepared handshake response type for host receiving (IN)
// bUH_R_RES1 & bUH_R_RES0: handshake response type for host receiving (IN)
//   00: ACK (ready) 
//   10: NAK (busy)
//   11: STALL (error)


/* USB host TX control registers: R8_UH_TX_CTRL0 / R8_UH_TX_CTRL1 */
#define bUH_T_DATA_NO   0x40	  // prepared no data packet, for high speed hub in host mode
#define bUH_T_AUTO_TOG 	0x20      // enable automatic toggle after successful transfer completion: 0=manual toggle, 1=automatic toggle
#define MASK_UH_T_TOG 	0x18      // prepared data toggle flag of host transmittal (SETUP/OUT): 00=DATA0, 01=DATA1, 10=DATA2, 11=MDATA
#define bUH_T_RES_NO	0x04      // expected no response, 1=enable, 0=disable, for non-zero endpoint isochronous transactions
#define MASK_UH_T_RES   0x03      // expected handshake response type for host transmittal (SETUP/OUT)
// bUH_T_RES1 & bUH_T_RES0: expected handshake response type for host transmittal (SETUP/OUT)
//   00: ACK (ready) 
//   10: NAK (busy)
//   11: STALL (error)
//   01: NYET

// toggle flag
#define UH_DATA0	(0x00<<3)
#define UH_DATA1	(0x01<<3)
#define UH_DATA2	(0x02<<3)
#define UH_MDATA	(0x03<<3)

// handshake response
#define UH_ACK		0x00
#define UH_NAK		0x02
#define UH_STALL	0x03
#define UH_NYET		0x01


/* USB host PID registers:  R16_UH_EP_PID0 / R16_UH_EP_PID1  */
#define MASK_UH_TOKEN     0xF0      // bit mask of token PID for USB host transfer
#define MASK_UH_ENDP      0x0F      // bit mask of endpoint number for USB host transfer

/*   R8_UHOST_CTRL0 / R8_UHOST_CTRL1 */
#define bUH_TX_BUS_RESUME  0x04
#define bUH_TX_BUS_SUSPEND 0x02
#define bUH_TX_BUS_RESET   0x01



/* USB device endpoint 1/4 buffer mode registers: R8_UEP4_1_MOD0 / R8_UEP4_1_MOD1 */
#define bUEP1_RX_EN       0x80      // enable USB endpoint 1 receiving (OUT)
#define bUEP1_TX_EN       0x40      // enable USB endpoint 1 transmittal (IN)
#define bUEP1_BUF_MOD     0x10      // buffer mode of USB endpoint 1
// bUEPn_RX_EN & bUEPn_TX_EN & bUEPn_BUF_MOD: USB endpoint 1/2/3 buffer mode, buffer start address is UEPn_DMA
//   0 0 x:  disable endpoint and disable buffer
//   1 0 0:  512 bytes buffer for receiving (OUT endpoint)
//   1 0 1:  dual 512 bytes buffer by toggle bit bUEP_R_TOG selection for receiving (OUT endpoint), total=1024bytes
//   0 1 0:  512 bytes buffer for transmittal (IN endpoint)
//   0 1 1:  dual 512 bytes buffer by toggle bit bUEP_T_TOG selection for transmittal (IN endpoint), total=1024bytes
//   1 1 0:  512 bytes buffer for receiving (OUT endpoint) + 512 bytes buffer for transmittal (IN endpoint), total=1024bytes
//   1 1 1:  dual 512 bytes buffer by bUEP_R_TOG selection for receiving (OUT endpoint) + dual 512 bytes buffer by bUEP_T_TOG selection for transmittal (IN endpoint), total=2048bytes
#define bUEP4_RX_EN       0x08      // enable USB endpoint 4 receiving (OUT)
#define bUEP4_TX_EN       0x04      // enable USB endpoint 4 transmittal (IN)
// bUEP4_RX_EN & bUEP4_TX_EN: USB endpoint 4 buffer mode, buffer start address is UEP0_DMA
//   0 0:  single 64 bytes buffer for endpoint 0 receiving & transmittal (OUT & IN endpoint)
//   1 0:  single 64 bytes buffer for endpoint 0 receiving & transmittal (OUT & IN endpoint) + 512 bytes buffer for endpoint 4 receiving (OUT endpoint), total=576bytes
//   0 1:  single 64 bytes buffer for endpoint 0 receiving & transmittal (OUT & IN endpoint) + 512 bytes buffer for endpoint 4 transmittal (IN endpoint), total=576bytes
//   1 1:  single 64 bytes buffer for endpoint 0 receiving & transmittal (OUT & IN endpoint)
//           + 512 bytes buffer for endpoint 4 receiving (OUT endpoint) + 512 bytes buffer for endpoint 4 transmittal (IN endpoint), total=1088bytes

/* USB device endpoint 2/3 buffer mode registers: R8_UEP2_3_MOD0 / R8_UEP2_3_MOD1 */
#define bUEP3_RX_EN       0x80      // enable USB endpoint 3 receiving (OUT)
#define bUEP3_TX_EN       0x40      // enable USB endpoint 3 transmittal (IN)
#define bUEP3_BUF_MOD     0x10      // buffer mode of USB endpoint 3
#define bUEP2_RX_EN       0x08      // enable USB endpoint 2 receiving (OUT)
#define bUEP2_TX_EN       0x04      // enable USB endpoint 2 transmittal (IN)
#define bUEP2_BUF_MOD     0x01      // buffer mode of USB endpoint 2

/* USB host buffer mode registers: R8_UH_EP_MOD0 / R8_UH_EP_MOD1 */
#define bUH_EP_TX_EN      0x40      // enable USB host OUT endpoint transmittal
#define bUH_EP_TBUF_MOD   0x10      // buffer mode of USB host OUT endpoint
// bUH_EP_TX_EN & bUH_EP_TBUF_MOD: USB host OUT endpoint buffer mode, buffer start address is UH_TX_DMA
//   0 x:  disable endpoint and disable buffer
//   1 0:  512 bytes buffer for transmittal (OUT endpoint)
//   1 1:  dual 512 bytes buffer by toggle bit bUH_T_TOG selection for transmittal (OUT endpoint), total=1024bytes
#define bUH_EP_RX_EN      0x08      // enable USB host IN endpoint receiving
#define bUH_EP_RBUF_MOD   0x01      // buffer mode of USB host IN endpoint
// bUH_EP_RX_EN & bUH_EP_RBUF_MOD: USB host IN endpoint buffer mode, buffer start address is UH_RX_DMA
//   0 x:  disable endpoint and disable buffer
//   1 0:  512 bytes buffer for receiving (IN endpoint)
//   1 1:  dual 512 bytes buffer by toggle bit bUH_R_TOG selection for receiving (IN endpoint), total=1024bytes



#ifdef __cplusplus
}
#endif

#endif		// __CH568USBSFR_H__


#ifndef __USB_TYPE__
#define __USB_TYPE__

#ifdef __cplusplus
extern "C" {
#endif

/* 连接设备速度定义 */
#define USB_DEV_SPEED_LS		0x01									/* 当前为低速设备 */
#define USB_DEV_SPEED_FS		0x00									/* 当前为全速设备 */
#define USB_DEV_SPEED_HS		0x02

/* USB PID */
#ifndef USB_PID_SETUP
#define USB_PID_NULL            0x00    /* reserved PID */
#define USB_PID_PING    		0x04
#define USB_PID_SOF             0x05
#define USB_PID_SETUP           0x0D
#define USB_PID_IN              0x09
#define USB_PID_OUT             0x01
#define USB_PID_ACK             0x02
#define USB_PID_NAK             0x0A
#define USB_PID_STALL           0x0E
#define USB_PID_NYET	        0x06
#define USB_PID_DATA0           0x03
#define USB_PID_DATA1           0x0B
#define USB_PID_PRE             0x0C
#define USB_PID_ERR     		0x0C
#define USB_PID_SPLIT           0x08
#define USB_PID_DATA0   		0x03
#define USB_PID_DATA1   		0x0B
#define USB_PID_DATA2   		0x07
#define USB_PID_MDATA   		0x0F
#endif

/* USB standard device request code */
#ifndef USB_GET_DESCRIPTOR
#define USB_GET_STATUS          0x00
#define USB_CLEAR_FEATURE       0x01
#define USB_SET_FEATURE         0x03
#define USB_SET_ADDRESS         0x05
#define USB_GET_DESCRIPTOR      0x06
#define USB_SET_DESCRIPTOR      0x07
#define USB_GET_CONFIGURATION   0x08
#define USB_SET_CONFIGURATION   0x09
#define USB_GET_INTERFACE       0x0A
#define USB_SET_INTERFACE       0x0B
#define USB_SYNCH_FRAME         0x0C
#endif

/* USB hub class request code */
#ifndef HUB_GET_DESCRIPTOR
#define HUB_GET_STATUS          0x00
#define HUB_CLEAR_FEATURE       0x01
#define HUB_GET_STATE           0x02
#define HUB_SET_FEATURE         0x03
#define HUB_GET_DESCRIPTOR      0x06
#define HUB_SET_DESCRIPTOR      0x07
#endif

/* USB HID class request code */
#ifndef HID_GET_REPORT
#define HID_GET_REPORT          0x01
#define HID_GET_IDLE            0x02
#define HID_GET_PROTOCOL        0x03
#define HID_SET_REPORT          0x09
#define HID_SET_IDLE            0x0A
#define HID_SET_PROTOCOL        0x0B
#endif

/* Bit define for USB request type */
#ifndef USB_REQ_TYP_MASK
#define USB_REQ_TYP_IN          0x80            /* control IN, device to host */
#define USB_REQ_TYP_OUT         0x00            /* control OUT, host to device */
#define USB_REQ_TYP_READ        0x80            /* control read, device to host */
#define USB_REQ_TYP_WRITE       0x00            /* control write, host to device */
#define USB_REQ_TYP_MASK        0x60            /* bit mask of request type */
#define USB_REQ_TYP_STANDARD    0x00
#define USB_REQ_TYP_CLASS       0x20
#define USB_REQ_TYP_VENDOR      0x40
#define USB_REQ_TYP_RESERVED    0x60
#define USB_REQ_RECIP_MASK      0x1F            /* bit mask of request recipient */
#define USB_REQ_RECIP_DEVICE    0x00
#define USB_REQ_RECIP_INTERF    0x01
#define USB_REQ_RECIP_ENDP      0x02
#define USB_REQ_RECIP_OTHER     0x03
#endif

/* USB request type for hub class request */
#ifndef HUB_GET_HUB_DESCRIPTOR
#define HUB_CLEAR_HUB_FEATURE   0x20
#define HUB_CLEAR_PORT_FEATURE  0x23
#define HUB_GET_BUS_STATE       0xA3
#define HUB_GET_HUB_DESCRIPTOR  0xA0
#define HUB_GET_HUB_STATUS      0xA0
#define HUB_GET_PORT_STATUS     0xA3
#define HUB_SET_HUB_DESCRIPTOR  0x20
#define HUB_SET_HUB_FEATURE     0x20
#define HUB_SET_PORT_FEATURE    0x23
#endif

/* Hub class feature selectors */
#ifndef HUB_PORT_RESET
#define HUB_C_HUB_LOCAL_POWER   0
#define HUB_C_HUB_OVER_CURRENT  1
#define HUB_PORT_CONNECTION     0
#define HUB_PORT_ENABLE         1
#define HUB_PORT_SUSPEND        2
#define HUB_PORT_OVER_CURRENT   3
#define HUB_PORT_RESET          4
#define HUB_PORT_POWER          8
#define HUB_PORT_LOW_SPEED      9
#define HUB_C_PORT_CONNECTION   16
#define HUB_C_PORT_ENABLE       17
#define HUB_C_PORT_SUSPEND      18
#define HUB_C_PORT_OVER_CURRENT 19
#define HUB_C_PORT_RESET        20
#endif

/* USB descriptor type */
#ifndef USB_DESCR_TYP_DEVICE
#define USB_DESCR_TYP_DEVICE    0x01
#define USB_DESCR_TYP_CONFIG    0x02
#define USB_DESCR_TYP_STRING    0x03
#define USB_DESCR_TYP_INTERF    0x04
#define USB_DESCR_TYP_ENDP      0x05
#define USB_DESCR_TYP_QUALIF    0x06
#define USB_DESCR_TYP_SPEED     0x07
#define USB_DESCR_TYP_OTG       0x09
#define USB_DESCR_TYP_HID       0x21
#define USB_DESCR_TYP_REPORT    0x22
#define USB_DESCR_TYP_PHYSIC    0x23
#define USB_DESCR_TYP_CS_INTF   0x24
#define USB_DESCR_TYP_CS_ENDP   0x25
#define USB_DESCR_TYP_HUB       0x29
#endif

/* USB device class */
#ifndef USB_DEV_CLASS_HUB
#define USB_DEV_CLASS_RESERVED  0x00
#define USB_DEV_CLASS_AUDIO     0x01
#define USB_DEV_CLASS_COMMUNIC  0x02
#define USB_DEV_CLASS_HID       0x03
#define USB_DEV_CLASS_MONITOR   0x04
#define USB_DEV_CLASS_PHYSIC_IF 0x05
#define USB_DEV_CLASS_POWER     0x06
#define USB_DEV_CLASS_PRINTER   0x07
#define USB_DEV_CLASS_STORAGE   0x08
#define USB_DEV_CLASS_HUB       0x09
#define USB_DEV_CLASS_VEN_SPEC  0xFF
#endif

/* USB endpoint type and attributes */
#ifndef USB_ENDP_TYPE_MASK
#define USB_ENDP_DIR_MASK       0x80
#define USB_ENDP_ADDR_MASK      0x0F
#define USB_ENDP_TYPE_MASK      0x03
#define USB_ENDP_TYPE_CTRL      0x00
#define USB_ENDP_TYPE_ISOCH     0x01
#define USB_ENDP_TYPE_BULK      0x02
#define USB_ENDP_TYPE_INTER     0x03
#endif

#ifndef USB_DEVICE_ADDR
#define	USB_DEVICE_ADDR			0x02	/* 默认的USB设备地址 */
#endif
#ifndef DEFAULT_ENDP0_SIZE
#define DEFAULT_ENDP0_SIZE      64       /* default maximum packet size for endpoint 0 */
#endif
#ifndef MAX_PACKET_SIZE
#define MAX_PACKET_SIZE         512      /* maximum packet size */
#endif
#ifndef USB_BO_CBW_SIZE
#define USB_BO_CBW_SIZE			0x1F	/* 命令块CBW的总长度 */
#define USB_BO_CSW_SIZE			0x0D	/* 命令状态块CSW的总长度 */
#endif
#ifndef USB_BO_CBW_SIG0
#define USB_BO_CBW_SIG0         0x55    /* 命令块CBW识别标志'USBC' */
#define USB_BO_CBW_SIG1         0x53
#define USB_BO_CBW_SIG2         0x42
#define USB_BO_CBW_SIG3         0x43
#define USB_BO_CSW_SIG0         0x55    /* 命令状态块CSW识别标志'USBS' */
#define USB_BO_CSW_SIG1         0x53
#define USB_BO_CSW_SIG2         0x42
#define USB_BO_CSW_SIG3         0x53
#endif

#ifndef USB_SETUP_REQ
typedef struct _USB_SETUP_REQ {
    UINT8 bRequestType;
    UINT8 bRequest;
    UINT8 wValueL;
    UINT8 wValueH;
    UINT8 wIndexL;
    UINT8 wIndexH;
    UINT8 wLengthL;
    UINT8 wLengthH;
} USB_SETUP_REQ, *PUSB_SETUP_REQ;
#endif

#ifndef USB_DEV_DESCR
typedef struct _USB_DEVICE_DESCR {
    UINT8 bLength;
    UINT8 bDescriptorType;
    UINT8 bcdUSBL;
    UINT8 bcdUSBH;
    UINT8 bDeviceClass;
    UINT8 bDeviceSubClass;
    UINT8 bDeviceProtocol;
    UINT8 bMaxPacketSize0;
    UINT8 idVendorL;
    UINT8 idVendorH;
    UINT8 idProductL;
    UINT8 idProductH;
    UINT8 bcdDeviceL;
    UINT8 bcdDeviceH;
    UINT8 iManufacturer;
    UINT8 iProduct;
    UINT8 iSerialNumber;
    UINT8 bNumConfigurations;
} USB_DEV_DESCR, *PUSB_DEV_DESCR;
#endif

#ifndef USB_CFG_DESCR
typedef struct _USB_CONFIG_DESCR {
    UINT8 bLength;
    UINT8 bDescriptorType;
    UINT8 wTotalLengthL;
    UINT8 wTotalLengthH;
    UINT8 bNumInterfaces;
    UINT8 bConfigurationValue;
    UINT8 iConfiguration;
    UINT8 bmAttributes;
    UINT8 MaxPower;
} USB_CFG_DESCR, *PUSB_CFG_DESCR;
#endif

#ifndef USB_ITF_DESCR
typedef struct _USB_INTERF_DESCR {
    UINT8 bLength;
    UINT8 bDescriptorType;
    UINT8 bInterfaceNumber;
    UINT8 bAlternateSetting;
    UINT8 bNumEndpoints;
    UINT8 bInterfaceClass;
    UINT8 bInterfaceSubClass;
    UINT8 bInterfaceProtocol;
    UINT8 iInterface;
} USB_ITF_DESCR, *PUSB_ITF_DESCR;
#endif

#ifndef USB_ENDP_DESCR
typedef struct _USB_ENDPOINT_DESCR {
    UINT8 bLength;
    UINT8 bDescriptorType;
    UINT8 bEndpointAddress;
    UINT8 bmAttributes;
    UINT8 wMaxPacketSizeL;
    UINT8 wMaxPacketSizeH;
    UINT8 bInterval;
} USB_ENDP_DESCR, *PUSB_ENDP_DESCR;
#endif

#ifndef USB_CFG_DESCR_LONG
typedef struct _USB_CONFIG_DESCR_LONG {
    USB_CFG_DESCR   cfg_descr;
    USB_ITF_DESCR   itf_descr;
    USB_ENDP_DESCR  endp_descr[1];
} USB_CFG_DESCR_LONG, *PUSB_CFG_DESCR_LONG;
#endif

#ifndef USB_HUB_DESCR
typedef struct _USB_HUB_DESCR {
    UINT8 bDescLength;
    UINT8 bDescriptorType;
    UINT8 bNbrPorts;
    UINT8 wHubCharacteristicsL;
    UINT8 wHubCharacteristicsH;
    UINT8 bPwrOn2PwrGood;
    UINT8 bHubContrCurrent;
    UINT8 DeviceRemovable;
    UINT8 PortPwrCtrlMask;
} USB_HUB_DESCR, *PUSB_HUB_DESCR;
#endif

#ifndef USB_HID_DESCR
typedef struct _USB_HID_DESCR {
    UINT8 bLength;
    UINT8 bDescriptorType;
    UINT8 bcdHIDL;
    UINT8 bcdHIDH;
    UINT8 bCountryCode;
    UINT8 bNumDescriptors;
    UINT8 bDescriptorTypeX;
    UINT8 wDescriptorLengthL;
    UINT8 wDescriptorLengthH;
} USB_HID_DESCR, *PUSB_HID_DESCR;
#endif

#ifndef USB_DEV_QUAL_DESCR
typedef struct _USB_DEVICE_QUALIFIER_DESCRIPTOR 
{
	UINT8  bLength;
	UINT8  bDescriptorType;
	UINT8  bcdUSBL;
	UINT8  bcdUSBH;
	UINT8  bDeviceClass;
	UINT8  bDeviceSubClass;
	UINT8  bDeviceProtocol;
	UINT8  bMaxPacketSize0;
	UINT8  bNumConfigurations;
	UINT8  bReserved;
}USB_DEV_QUAL_DESCR, *PUSB_DEV_QUAL_DESCR;
#endif

#ifndef USB_CFG_PWR_DESCR
typedef struct _USB_CONFIGURATION_POWER_DESCRIPTOR 
{
	UINT8  bLength;
	UINT8  bDescriptorType;
	UINT8  SelfPowerConsumedD0[3];
	UINT8  bPowerSummaryId;
	UINT8  bBusPowerSavingD1;
	UINT8  bSelfPowerSavingD1;
	UINT8  bBusPowerSavingD2;
	UINT8  bSelfPowerSavingD2; 
	UINT8  bBusPowerSavingD3;
	UINT8  bSelfPowerSavingD3; 
	UINT8  TransitionTimeFromD1L;
	UINT8  TransitionTimeFromD1H;
	UINT8  TransitionTimeFromD2L;
	UINT8  TransitionTimeFromD2H;
	UINT8  TransitionTimeFromD3L;
	UINT8  TransitionTimeFromD3H;
}USB_CFG_PWR_DESCR, *PUSB_CFG_PWR_DESCR;
#endif

#ifndef USB_ITF_PWR_DESCR
typedef struct _USB_INTERFACE_POWER_DESCRIPTOR 
{
	UINT8  bLength;
	UINT8  bDescriptorType;
	UINT8  bmCapabilitiesFlags;
	UINT8  bBusPowerSavingD1;
	UINT8  bSelfPowerSavingD1;
	UINT8  bBusPowerSavingD2;
	UINT8  bSelfPowerSavingD2; 
	UINT8  bBusPowerSavingD3;
	UINT8  bSelfPowerSavingD3; 
	UINT8  TransitionTimeFromD1L;
	UINT8  TransitionTimeFromD1H;
	UINT8  TransitionTimeFromD2L;
	UINT8  TransitionTimeFromD2H;
	UINT8  TransitionTimeFromD3L;
	UINT8  TransitionTimeFromD3H;
}USB_ITF_PWR_DESCR, *PUSB_ITF_PWR_DESCR;
#endif

#define USB_BO_CBW_SIG              0x43425355                                  /* 命令块CBW识别标志'USBC' */
#define USB_BO_CSW_SIG              0x53425355                                  /* 命令状态块CSW识别标志'USBS' */

typedef union _BULK_ONLY_CMD
{
    struct
    {
        UINT32 mCBW_Sig;
        UINT32 mCBW_Tag;
        UINT32 mCBW_DataLen;                                                    /* 输入: 数据传输长度 */
        UINT8  mCBW_Flag;                                                       /* 输入: 传输方向等标志 */
        UINT8  mCBW_LUN;
        UINT8  mCBW_CB_Len;                                                     /* 输入: 命令块的长度,有效值是1到16 */
        UINT8  mCBW_CB_Buf[16];                                                 /* 输入: 命令块,该缓冲区最多为16个字节 */
    } mCBW;                                                                     /* BulkOnly协议的命令块, 输入CBW结构 */
    struct
    {
        UINT32 mCSW_Sig;
        UINT32 mCSW_Tag;
        UINT32 mCSW_Residue;                                                    /* 返回: 剩余数据长度 */
        UINT8  mCSW_Status;                                                     /* 返回: 命令执行结果状态 */
    } mCSW;                                                                     /* BulkOnly协议的命令状态块, 输出CSW结构 */
} BULK_ONLY_CMD, *PBULK_ONLY_CMD;


#ifdef __cplusplus
}
#endif

#endif		// __USB_TYPE__


#ifndef __CH568SATASFR_H__
#define __CH568SATASFR_H__

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************/
/*                         Peripheral memory map                              */
/******************************************************************************/
/* SATA addresses                                                              */
#define SATA_BASE_ADDR          (0x0040B000)

#define R32_SATA_CONTROL		(*((PUINT32V)(SATA_BASE_ADDR + 0x000)))			// SATA control & Power control & buffer mode
#define R8_SATA_CTRL			(*((PUINT8V)(SATA_BASE_ADDR + 0x000)))			// SATA base control
#define R8_SATA_PM_CTRL			(*((PUINT8V)(SATA_BASE_ADDR + 0x001)))			// Power control
#define R8_SATA_MOD				(*((PUINT8V)(SATA_BASE_ADDR + 0x002)))			// buffer mode
#define R16_SATA_INT_EN			(*((PUINT16V)(SATA_BASE_ADDR + 0x004)))			// SATA interrupt enable
#define R8_SATA_INT_EN_L		(*((PUINT8V)(SATA_BASE_ADDR + 0x004)))
#define R8_SATA_INT_EN_H		(*((PUINT8V)(SATA_BASE_ADDR + 0x005)))
#define R16_SATA_RX_LEN			(*((PUINT16V)(SATA_BASE_ADDR + 0x008)))			// SATA receiving length
#define R32_SATA_STATUS			(*((PUINT32V)(SATA_BASE_ADDR + 0x00C)))			// SATA interrupt flag & status
#define R16_SATA_INT_FG			(*((PUINT16V)(SATA_BASE_ADDR + 0x00C)))			// SATA interrupt flag
#define R8_SATA_INT_FG_L		(*((PUINT8V)(SATA_BASE_ADDR + 0x00C)))			
#define R8_SATA_INT_FG_H		(*((PUINT8V)(SATA_BASE_ADDR + 0x00D)))
#define R8_SATA_INT_ST	 		(*((PUINT8V)(SATA_BASE_ADDR + 0x00E)))			// SATA interrupt status
#define R32_SATA_RTX_CONTROL 	(*((PUINT32V)(SATA_BASE_ADDR + 0x010)))
#define R16_SATA_TX_LEN			(*((PUINT16V)(SATA_BASE_ADDR + 0x010)))			// SATA transmittal length
#define R8_SATA_RTX_CTRL	 	(*((PUINT8V)(SATA_BASE_ADDR + 0x012)))			// SATA RX&TX control
#define R32_SATA_DATA0			(*((PUINT32V)(SATA_BASE_ADDR + 0x014)))			// first Dword for buffer0
#define R32_SATA_DATA1			(*((PUINT32V)(SATA_BASE_ADDR + 0x018)))			// first Dword for buffer1
#define R32_SATA_DMA			(*((PUINT32V)(SATA_BASE_ADDR + 0x01C)))
#define R16_SATA_DMA0			(*((PUINT16V)(SATA_BASE_ADDR + 0x01C)))			//  DMA buffer0 address
#define	R16_SATA_DMA1			(*((PUINT16V)(SATA_BASE_ADDR + 0x01E)))			//  DMA buffer1 address

/* R8_SATA_CTRL */
#define bSC_DMA_EN			0x80	/* DMA enable and DMA interrupt enable for SATA */		
#define bSC_CONT_EN			0x40	/* CONT primitive enable */
#define bSC_HOST_MODE		0x20	/* mode select: 0=device mode, 1=host mode */
#define	bSC_FORCE_1P5G		0x10	/* speed select: 0=nomal mode and actual speed read bSMS_SPD_TYPE, 1=force 1.5G */
#define bSC_INT_BUSY		0x08	/* enable automatic pause for transmission during interrupt flag UIF_TRANSFER valid */
#define bSC_RESET_PHY		0x04	/* SATA PHY reset control */
#define bSC_RESET_LINK		0x02	/* SATA LINK reset control */
#define bSC_CLR_ALL			0x01	/* force clear FIFO and count of SATA */

/*  R8_SATA_PM_CTRL	*/
#define bSPC_OFFLINE		0x80
#define bSPC_LISTEN			0x40
#define bSPC_SLUMBER_S		0x20
#define bSPC_SLUMBER		0x10	/* 1=force enter SLUMBER mode, 0=PHYRDY */
#define bSPC_PARTIAL		0x08	/* 1=force enter PARTIAL mode, 0=PHYRDY */
#define bSPC_SEND_PMREQ_P	0x04	/* 1=send request PARTIAL mode, 0=no operation */
#define bSPC_SEND_PMREQ_S	0x02	/* 1=send request SLUMBER mode, 0=no operation */
#define bSPC_DENY			0x01	/* power management control: 0=support, 1=no support */

/* R8_SATA_MOD */
#define bSM_RX_EN			0x04	/* enable SATA receiving  */
#define bSM_TX_EN			0x02	/* enable SATA transmittal  */
#define bSM_BUF_MOD			0x01	/* buffer mode select */
// bSM_RX_EN & bSM_TX_EN & bSM_BUF_MOD: 
//   0 0 x:  disable buffer
//   1 0 0:  2048 Dwords buffer for receiving, buffer start address is SATA_DMA0
//   1 0 1:  dual 2048 Dwords buffer by toggle bit bSRC_R_TOG selection for receiving, total=4096 Dwords, SATA_DMA0 for bSRC_R_TOG=0, SATA_DMA1 for bSRC_R_TOG=1
//   0 1 0:  2048 Dwords buffer for transmittal, buffer start address is SATA_DMA0
//   0 1 1:  dual 2048 Dwords buffer by toggle bit bSRC_T_TOG selection for transmittal, total=4096 Dwords, SATA_DMA0 for bSRC_T_TOG=0, SATA_DMA1 for bSRC_T_TOG=1
//   1 1 x:  2048 Dwords buffer for receiving + 2048 Dwords buffer for transmittal, total=4096 Dwords, receiving from SATA_DMA0, transmittal from SATA_DMA1

/* R16_SATA_INT_EN */
#define bSIE_TRAN_INT		0x0080	/* enable interrupt for transmittal abort event */
#define bSIE_HOLD			0x0040	/* enable interrupt for receiving HOLDp primitive */
#define	bSIE_DMAT			0x0020	/* enable interrupt for receiving DMATp primitive */
#define bSIE_FIFO_OV		0x0010	/* enable interrupt for FIFO overflow */
#define bSIE_COLLIDE		0x0008	/* enable interrupt for bus conflict */
#define bSIE_RECV_OK		0x0004	/* enable interrupt for receiving completion */
#define bSIE_TRAN_OK   	 	0x0002	/* enable interrupt for transmittal completion */
#define bSIE_PHYRDY			0x0001	/* enable interrupt for device detected event */

#define	bSIE_COMWAKE		0x1000	/* enable interrupt for COMWAKE */
#define bSIE_COMINIT		0x0800	/* enable interrupt for COMINIT */
#define bSIE_PHYERR     	0x0400	/* enable interrupt for phy connect error */
#define bSIE_PM_REQ			0x0200	/* enable interrupt for receiving power management request */
#define bSIE_PM_RES			0x0100	/* enable interrupt for power management response */

/* R16_SATA_INT_FG */
#define bSIF_TRAN_INT		0x0080	/* transmittal abort event interrupt flag, write 1 to clear */
#define bSIF_HOLD			0x0040	/* receiving HOLDp primitive */
#define	bSIF_DMAT			0x0020	/* receiving DMATp primitive */
#define bSIF_FIFO_OV		0x0010	/* FIFO overflow interrupt flag, write 1 to clear */
#define bSIF_COLLIDE		0x0008	/* bus conflict interrupt flag, write 1 to clear */
#define bSIF_RECV_OK		0x0004	/* receiving completion interrupt flag, write 1 to clear */
#define bSIF_TRAN_OK   	 	0x0002	/* transmittal completion interrupt flag, write 1 to clear */
#define bSIF_PHYRDY			0x0001	/* device detected event interrupt flag, write 1 to clear */

#define bSIF_COMWAKE		0x2000	/* COMWAKE interrupt flag, write 1 to clear */
#define	bSIF_COMINIT		0x1000	/* COMINIT interrupt flag, write 1 to clear */
#define bSIF_PHYERR     	0x0800	/* phy connect error interrupt flag, write 1 to clear */
#define bSIF_PM_PARTIAL		0x0400	/* receiving PARTIAL mode request interrupt flag, write 1 to clear */
#define bSIF_PM_SLUMBER		0x0200	/* receiving SLUMBER mode request interrupt flag, write 1 to clear */
#define bSIF_PM_NAK			0x0100	/* receiving no support power management interrupt flag, write 1 to clear */


/* R8_SATA_INT_ST */
#define bSMS_SPD_TYPE	    0x40	/* ReadOnly: indicate current actual speed: 0=1.5G, 1=3G */
#define bSMS_R_FIFO_RDY     0x20	/* ReadOnly: indicate receiving FIFO state: 0=empty, 1=non empty */
#define bSIS_LINK_FREE	    0x10	/* ReadOnly: indicate LINK layer free state: 0=busy, 1=free */
#define bSIS_HOLD		    0x08	/* ReadOnly: indicate current HOLD state, set bSRC_SYNC_ESCAPE bit to force abort current transfer */
#define bSIS_RECV_CRC_OK    0x04	/* ReadOnly: indicate current reveiving data packet check status: 0=error(R_ERR), 1=ok(R_OK) */
#define bSIS_TRAN_CRC_OK    0x02	/* ReadOnly: indicate transmittal data packet stauts: 0=error, 1=ok */
#define bSIS_PHYRDY		    0x01	/* ReadOnly: indicate device attached status for phy : 0=disconnect, 1=connect */


/* R8_SATA_RTX_CTRL */
#define bSRC_R_AUTO_TOG		0x80	/* enable automatic toggle after successful receiver completion: 0=manual toggle, 1=automatic toggle */
#define bSRC_T_AUTO_TOG		0x40	/* enable automatic toggle after successful transfer completion: 0=manual toggle, 1=automatic toggle */
#define bSRC_R_TOG			0x20	/* 0=receive data stored in SATA_DMA0, 1=receive data stored in SATA_DMA1 */
#define	bSRC_T_TOG			0x10	/* 0=transmission data from SATA_DMA0, 1=transmission data from SATA_DMA1 */
#define bSRC_R_READY		0x04	/* ready to receive data */
#define bSRC_T_READY		0x02	/* ready to transmit data, start to transmit */
#define bSRC_SYNC_ESCAPE	0x01	/* force abort current transfer */


#ifdef __cplusplus
}
#endif

#endif      // __CH568SATASFR_H__


#ifndef __SATA_TYPE__
#define __SATA_TYPE__

#ifdef __cplusplus
extern "C" {
#endif

/* FIStype */
#ifndef SATA_FIS_TYPE
#define SATA_FIS_REGHOST	  	0x27   /* Host to Device */ 
#define SATA_FIS_REGDEVICE	  	0x34   /* Device to Host */ 
#define SATA_FIS_PIOSETUP	  	0x5f   /* PIO Setup FIS           Device to Host */ 
#define SATA_FIS_DMAACT		  	0x39   /* DMA Activate FIS        Device to Host*/
#define SATA_FIS_DMASETUP	  	0x41   /* DMA Setup FIS           Bidirectonal */ 
#define SATA_FIS_TRANDATA	  	0x46   /* DATA FIS                Bidirectonal */     
#define SATA_FIS_BIST		  	0x58   /* BIST Activate FIS       Bidirectonal */
#define SATA_FIS_SETBIT		  	0xA1   /* Set Devive Bits FIS     Device to Host*/ 
#endif

/* Part Command */
#ifndef SATA_CMD_TYPE
/* PIO data-in command */
#define SATA_CMD_IDENTIFY  		0xEC
#define SATA_CMD_IEFYPDEV  		0xA1
#define SATA_CMD_READBUFF       0xE4
#define SATA_CMD_READLOGE       0x2F
#define SATA_CMD_READMUL       	0xC4
#define SATA_CMD_READMULE       0x29
#define SATA_CMD_READSECS       0x20
#define SATA_CMD_READSECSE      0x24

/* PIO data-out command */
#define SATA_CMD_SCYUNLOCK	    0xF2
#define SATA_CMD_WRITEBUFF	    0xE8
#define SATA_CMD_WRITELOGE      0x3F
#define SATA_CMD_WRITEMUL	    0xC5
#define SATA_CMD_WRITEMULE	    0x39
#define SATA_CMD_WRITESECS	    0x30
#define SATA_CMD_WRITESECSE	    0x34

/* DMA data in */
#define SATA_CMD_READDMA	    0xC8
#define SATA_CMD_READDMAE       0x25

/* DAM data out */
#define SATA_CMD_WRITEDMA       0xCA
#define SATA_CMD_WRITEDMAE      0x35

/* Non data */
#define SATA_CMD_IDLE		    0xE3
#define SATA_CMD_SETFEATURE     0xEF
#define SATA_CMD_SETMULMODE     0xC6
#define SATA_CMD_SECYLOCK       0xF5
#define SATA_CMD_IDLEIMMDE      0xE1
#define SATA_CMD_DEVCONFG       0xB1
#define SATA_CMD_SMART          0xB0
#define SATA_CMD_STANDBY        0xE2
#define SATA_CMD_STANDBYIMMDE   0xE0
#define SATA_CMD_READVRYSEC     0x40
#define SATA_CMD_FLUSHCACHE     0xE7
#endif

#ifndef SATA_PACKET_STRUCT
typedef union _CTRLPARAM {
	struct
	{
	    UINT8 bFIStype;
	    UINT8 bPMport;
	    UINT8 bCommand;
	    UINT8 bFeathres;
	}SHADOW_HOST_CTRL;
	struct
	{
		UINT8 bFIStype;
		UINT8 bPMport;						
	    UINT8 bStatus;
	    UINT8 bError;
	}SHADOW_Device_CTRL;
	struct
	{
		UINT8 bFIStype;
		UINT8 bPMport;				//I-中断标志位 D-方向标志位
	    UINT8 bStatus;
	    UINT8 bError;
	}PIO_SETUP_CTRL;
	struct
	{
	    UINT8 bFIStype;
	    UINT8 bPMport;
	    UINT8 bReserved[2];
	}DMA_ACTIVE_CTRL;
	struct
	{
		UINT8 bFIStype;
		UINT8 bPMport;					//I-中断标志位 D-方向标志位 A-主动标志位
	    UINT8 bReserved[2];
	}DMA_SETUP_CTRL;
	struct
	{
		UINT8 bFIStype;
		UINT8 bPMport;					//I-中断标志位 D-方向标志位 A-主动标志位
	    UINT8 bReserved[2];
	}DATA_CTRL;
	struct
	{
		UINT8 bFIStype;
		UINT8 bPMport;	
	    UINT8 bPattern;					// T A S L F P R V
	    UINT8 bReserved;
	}BIST_AVT_CTRL;
	struct
	{
		UINT8 bFIStype;
		UINT8 bPMport;					// N-通知位  I-中断位 
	    UINT8 bStatus;					
	    UINT8 bError;
	}SET_DEV_CTRL;	
}CTRLPARAM;

typedef union _DATAPARAM {
	struct
	{
	    UINT8 bLBALow;
		UINT8 bLBAMid;
		UINT8 bLBAHigh;
		UINT8 bDevice;
		UINT8 bLBALowE;
		UINT8 bLBAMidE;
		UINT8 bLBAHighE;
		UINT8 bFeathresE;
		UINT16 bSecCount;
		UINT8 bReserved_0;
		UINT8 bControl;
		UINT8 bReserved[4];
	}SHADOW_HOST_DATA;
	
	struct
	{
		UINT8 bLBALow;
	    UINT8 bLBAMid;
	    UINT8 bLBAHigh;
	    UINT8 bDevice;
	    UINT8 bLBALowE;
	    UINT8 bLBAMidE;
	    UINT8 bLBAHighE;
	    UINT8 bReserved_0;
	    UINT16 bSecCount;
	    UINT8 bReserved[6];
	}SHADOW_Device_DATA;
	
	struct
	{
		UINT8 bLBALow;
	    UINT8 bLBAMid;
	    UINT8 bLBAHigh;
	    UINT8 bDevice;
	    UINT8 bLBALowE;
	    UINT8 bLBAMidE;
	    UINT8 bLBAHighE;
	    UINT8 bReserved_0;
	    UINT16 bSecCount;
	    UINT8 bReserved_1;
	    UINT8 bEStatus;
	    UINT16 bTranCount;
	    UINT8 bReserved[2];
	}PIO_SETUP_DATA;
	
	struct
	{
		UINT32 bDMABuff_L;
	    UINT32 bDMABuff_H;
	    UINT32 bReserved2;
	    UINT32 bDMABuffOFFET;
	    UINT32 bDMATRANCOUNT;
	    UINT32 bReserved3;
	}DMA_SETUP_DATA;
	
	struct
	{
		UINT32 bData[2];
	}BIST_AVT_DATA;
	
	struct
	{
		UINT32 bReserved;
	}SET_DEV_DATA;	
}DATAPARAM;		
#endif	

#ifndef SATA_REG_GROUP
typedef struct _TRANS_REG_GROUP {
	struct{
	    UINT8 bCommand;					//命令寄存器
	    UINT8 bDevice;					//设备寄存器
	    UINT8 bFeature;					//特征寄存器
	    UINT8 E_bFeature;				//扩展特征寄存器
	    UINT8 bLBALow;					//lab地址低位
	    UINT8 bLBAMid;					//lab地址中位
	    UINT8 bLBAHigh;					//lab地址高位
	    UINT8 E_bLBALow;				//扩展lab地址低位
	    UINT8 E_bLBAMid;				//扩展lab地址中位
	    UINT8 E_bLBAHigh;				//扩展lab地址高位
	    UINT8 bSecCount;				//扇区数寄存器
	    UINT8 E_bSecCount;				//扩展扇区数寄存器
	    UINT8 bError;					//错误寄存器
	}CmdBlock;
	
    struct{
	    UINT8 bControl;					//设备控制寄存器  
	    UINT8 bStatus;					//状态寄存器
	    UINT8 bAlterStatus;				//状态替换寄存器 
	}CtrBlock;
	
	struct{
		UINT32 bDMABuff_L;
		UINT32 bDMABuff_H;
	    UINT32 bDMABuffOFFET;
	    UINT32 bDMATRANCOUNT;
	}DMABlock;
	
	UINT8 bFlagReg;						//标志寄存器	
	
} TRANS_REG_GROUP, *PTRANS_REG_GROUP;

//bControl Register 设备控制寄存器，only host write
#define bHOB	 0x80	// 0-扩展lba地址无效,28位寻址  1-lba地址支持48寻址(扩展位有效)
#define bSRST	 0x04	// bit2-SRST 复位操作
#define bNIEN	 0x02	// bit1-nIEN 1-向主机发出中断请求  0-释放中断

//bDevice Register  设备寄存器
#define bDEV	 0x20	// 设备选择位 0-选择设备0  1-选择设备1
#define bADDR	 0x0f	// 在28位寻址，表示最高4位地址，和lba寄存器构成28位地址

//bStatus/bAlterStatus Register 状态/替换状态寄存器  only device write
#define bBSY	 0x80	// 1-表示设备忙，收到主机端命令后置1
#define bDRDY	 0x40	// 0-设备空闲  1-设备正在执行命令
#define bDF		 0x20	// 1-设备出错
#define bDRQ	 0x08	// 数据请求位， 0-主机端只能通过DMA进行数据传输  1-PIO和DMA方式皆可
#define bERR	 0x01	// 0-无错误  1-有错误，错误寄存器生效 

//bError Register   错误寄存器  only device write
#define bABRT	 0x04	// 1-命令无效或不支持

//bFlagReg Register   标志寄存器
#define bFlag_A	 0x80	// 主动位:主机端向设备端传输数据是否发送DMA活动信息帧  0-需要   1-不需要
#define bFlag_C	 0x08	// 更新位  0-对设备控制寄存器更新   1-对命令寄存器更新
#define bFlag_I  0x04	// 中断位 = nIEN
#define bFlag_D  0x02	// 方向位  0-host to device     1-device to host
#define bFlag_N	 0x01	// 通知位  0-不需要主机端关注   1-需要主机端关注

#endif

#ifdef __cplusplus
}
#endif

#endif     // __SATA_TYPE__
 

#ifndef __CH568SDC4SFR_H__
#define __CH568SDC4SFR_H__

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************/
/*                         Peripheral memory map                              */
/******************************************************************************/
/* sdc4 addresses                                                              */
#define SD0_BASE_ADDR               (0x0040A000)
#define SD1_BASE_ADDR               (0x0040A040)
#define SD2_BASE_ADDR               (0x0040A080)
#define SD3_BASE_ADDR               (0x0040A0c0)

/*   CLK DIV  */
#define     R16_SD_CLK_CFG    	(*((PUINT16V)(SD0_BASE_ADDR+0x03c))) // SD clock divider register, [4:0]:divisor, sdclk1 = 480m/[4:0], default divisor is 20
                                                        // [8]:clk_oe, [8]==1 indicate supply SDCLK to sdcard, [8]==0 indicate close SDCLK, default clk_oe is 0
                                                        // [9]:sel_25m_100m, select 400k or 25~100m as SDCLK, [9]==1 indicate SDCLK = 480m/[4:0], [9]==0 indicate SDCLK = 480/[4:0]/64, default is 1                
/*       SD0  */
#define     R32_SD0_ARGUMENT    (*((PUINT32V)(SD0_BASE_ADDR+0x000))) // SD 32bits command argument register
#define     R16_SD0_CMD_SET     (*((PUINT16V)(SD0_BASE_ADDR+0x004))) // SD 16bits cmd setting register, [5:0] is the command index, [9:8] indicate the response tye, [10]==1 indicate need to check the response CRC7, [11]==1 indicate need to check the index of the response 
#define     R32_SD0_RESPONSE0   (*((PUINT32V)(SD0_BASE_ADDR+0x008))) // SD 128bits response register, [31:0] 32bits 
#define     R32_SD0_RESPONSE1   (*((PUINT32V)(SD0_BASE_ADDR+0x00c))) // SD 128bits response register, [63:32] 32bits
#define     R32_SD0_RESPONSE2   (*((PUINT32V)(SD0_BASE_ADDR+0x010))) // SD 128bits response register, [95:64] 32bits
#define     R32_SD0_RESPONSE3   (*((PUINT32V)(SD0_BASE_ADDR+0x014))) // SD 128bits response register, [127:96] 32bits, when response length is 48, the 32bit response data is stored in this register       
#define     R32_SD0_WRITE_CONT	(*((PUINT32V)(SD0_BASE_ADDR+0x014))) 
#define     R8_SD0_CTRL         (*((PUINT8V )(SD0_BASE_ADDR+0x018))) // SD 8bits control register, [1:0] number of data line will operate, [2]==1 soft reset all the inner logic, [3]==1 enable DMA, [4]==1 reset inner data tran/recv logic, [5]==1 @negedge sample cmd/dat[7:0] lines                             
#define     R8_SD0_TOCNT        (*((PUINT8V )(SD0_BASE_ADDR+0x01c))) // SD 8bits data timeout value, the real timeout time is 4194304*Tsdclk*value
#define     R32_SD0_STATUS      (*((PUINT32V)(SD0_BASE_ADDR+0x020))) // SD status:[15:0] number of current successfull tran/recv blocks, [16] current sd cmd line level, [17] current sd dat[0] line level   
#define     R16_SD0_INT_FG      (*((PUINT16V)(SD0_BASE_ADDR+0x024))) // SD 16bits interrupt flag register
#define     R16_SD0_INT_EN      (*((PUINT16V)(SD0_BASE_ADDR+0x028))) // SD 16bits interrupt enable register 
#define     R16_SD0_DMA         (*((PUINT16V)(SD0_BASE_ADDR+0x02c))) // SD 16bits DMA start address register when to operate
#define     R32_SD0_BLOCK_CFG   (*((PUINT32V)(SD0_BASE_ADDR+0x030))) // SD 32bits data counter, [15:0] number of blocks this time will tran/recv, [27:16] block sise(byte number) of every block in this time tran/recv
#define     R8_SD0_TRAN_MODE    (*((PUINT8V )(SD0_BASE_ADDR+0x034))) // SD TRANSFER MODE register, [0] direction of this time, 1-write sd, 0-read sd, [1] stop at block gap when multiple wr/rd, [2] this time is eMMC boot mode operation 

/*       SD1  */
#define     R32_SD1_ARGUMENT    (*((PUINT32V)(SD1_BASE_ADDR+0x000))) // SD 32bits command argument register                                                                                                                                                                            
#define     R16_SD1_CMD_SET     (*((PUINT16V)(SD1_BASE_ADDR+0x004))) // SD 16bits cmd setting register, [5:0] is the command index, [9:8] indicate the response tye, [10]==1 indicate need to check the response CRC7, [11]==1 indicate need to check the index of the response        
#define     R32_SD1_RESPONSE0   (*((PUINT32V)(SD1_BASE_ADDR+0x008))) // SD 128bits response register, [31:0] 32bits                                                                                                                                                                    
#define     R32_SD1_RESPONSE1   (*((PUINT32V)(SD1_BASE_ADDR+0x00c))) // SD 128bits response register, [63:32] 32bits                                                                                                                                                                   
#define     R32_SD1_RESPONSE2   (*((PUINT32V)(SD1_BASE_ADDR+0x010))) // SD 128bits response register, [95:64] 32bits                                                                                                                                                                   
#define     R32_SD1_RESPONSE3   (*((PUINT32V)(SD1_BASE_ADDR+0x014))) // SD 128bits response register, [127:96] 32bits, when response length is 48, the 32bit response data is stored in this register                                                                                  
#define     R32_SD1_WRITE_CONT	(*((PUINT32V)(SD0_BASE_ADDR+0x014)))
#define     R8_SD1_CTRL         (*((PUINT8V )(SD1_BASE_ADDR+0x018))) // SD 8bits control register, [1:0] number of data line will operate, [2]==1 soft reset all the inner logic, [3]==1 enable DMA, [4]==1 reset inner data tran/recv logic, [5]==1 @negedge sample cmd/dat[7:0] lines
#define     R8_SD1_TOCNT        (*((PUINT8V )(SD1_BASE_ADDR+0x01c))) // SD 8bits data timeout value, the real timeout time is 4194304*Tsdclk*value                                                                                                                                     
#define     R32_SD1_STATUS      (*((PUINT32V)(SD1_BASE_ADDR+0x020))) // SD status:[15:0] number of current successfull tran/recv blocks, [16] current sd cmd line level, [17] current sd dat[0] line level                                                                             
#define     R16_SD1_INT_FG      (*((PUINT16V)(SD1_BASE_ADDR+0x024))) // SD 16bits interrupt flag register                                                                                                                                                                              
#define     R16_SD1_INT_EN      (*((PUINT16V)(SD1_BASE_ADDR+0x028))) // SD 16bits interrupt enable register                                                                                                                                                                            
#define     R16_SD1_DMA         (*((PUINT16V)(SD1_BASE_ADDR+0x02c))) // SD 16bits DMA start address register when to operate                                                                                                                                                                       
#define     R32_SD1_BLOCK_CFG   (*((PUINT32V)(SD1_BASE_ADDR+0x030))) // SD 32bits data counter, [15:0] number of blocks this time will tran/recv, [27:16] block sise(byte number) of every block in this time tran/recv                                                                
#define     R8_SD1_TRAN_MODE    (*((PUINT8V )(SD1_BASE_ADDR+0x034))) // SD TRANSFER MODE register, [0] direction of this time, 1-write sd, 0-read sd, [1] stop at block gap when multiple wr/rd, [2] this time is eMMC boot mode operation                                             
                                         
/*       SD2  */
#define     R32_SD2_ARGUMENT    (*((PUINT32V)(SD2_BASE_ADDR+0x000))) // SD 32bits command argument register                                                                                                                                                                            
#define     R16_SD2_CMD_SET     (*((PUINT16V)(SD2_BASE_ADDR+0x004))) // SD 16bits cmd setting register, [5:0] is the command index, [9:8] indicate the response tye, [10]==1 indicate need to check the response CRC7, [11]==1 indicate need to check the index of the response        
#define     R32_SD2_RESPONSE0   (*((PUINT32V)(SD2_BASE_ADDR+0x008))) // SD 128bits response register, [31:0] 32bits                                                                                                                                                                    
#define     R32_SD2_RESPONSE1   (*((PUINT32V)(SD2_BASE_ADDR+0x00c))) // SD 128bits response register, [63:32] 32bits                                                                                                                                                                   
#define     R32_SD2_RESPONSE2   (*((PUINT32V)(SD2_BASE_ADDR+0x010))) // SD 128bits response register, [95:64] 32bits                                                                                                                                                                   
#define     R32_SD2_RESPONSE3   (*((PUINT32V)(SD2_BASE_ADDR+0x014))) // SD 128bits response register, [127:96] 32bits, when response length is 48, the 32bit response data is stored in this register                                                                                  
#define     R32_SD2_WRITE_CONT	(*((PUINT32V)(SD0_BASE_ADDR+0x014)))
#define     R8_SD2_CTRL         (*((PUINT8V )(SD2_BASE_ADDR+0x018))) // SD 8bits control register, [1:0] number of data line will operate, [2]==1 soft reset all the inner logic, [3]==1 enable DMA, [4]==1 reset inner data tran/recv logic, [5]==1 @negedge sample cmd/dat[7:0] lines
#define     R8_SD2_TOCNT        (*((PUINT8V )(SD2_BASE_ADDR+0x01c))) // SD 8bits data timeout value, the real timeout time is 4194304*Tsdclk*value                                                                                                                                     
#define     R32_SD2_STATUS      (*((PUINT32V)(SD2_BASE_ADDR+0x020))) // SD status:[15:0] number of current successfull tran/recv blocks, [16] current sd cmd line level, [17] current sd dat[0] line level                                                                             
#define     R16_SD2_INT_FG      (*((PUINT16V)(SD2_BASE_ADDR+0x024))) // SD 16bits interrupt flag register                                                                                                                                                                              
#define     R16_SD2_INT_EN      (*((PUINT16V)(SD2_BASE_ADDR+0x028))) // SD 16bits interrupt enable register                                                                                                                                                                            
#define     R16_SD2_DMA         (*((PUINT16V)(SD2_BASE_ADDR+0x02C))) // SD 16bits DMA start address register when to operate                                                                                                                                                                              
#define     R32_SD2_BLOCK_CFG   (*((PUINT32V)(SD2_BASE_ADDR+0x030))) // SD 32bits data counter, [15:0] number of blocks this time will tran/recv, [27:16] block sise(byte number) of every block in this time tran/recv                                                                                                   
#define     R8_SD2_TRAN_MODE    (*((PUINT8V )(SD2_BASE_ADDR+0x034))) // SD TRANSFER MODE register, [0] direction of this time, 1-write sd, 0-read sd, [1] stop at block gap when multiple wr/rd, [2] this time is eMMC boot mode operation                                             
                                                          
/*       SD3  */
#define     R32_SD3_ARGUMENT    (*((PUINT32V)(SD3_BASE_ADDR+0x000))) // SD 32bits command argument register                                                                                                                                                                            
#define     R16_SD3_CMD_SET     (*((PUINT16V)(SD3_BASE_ADDR+0x004))) // SD 16bits cmd setting register, [5:0] is the command index, [9:8] indicate the response tye, [10]==1 indicate need to check the response CRC7, [11]==1 indicate need to check the index of the response        
#define     R32_SD3_RESPONSE0   (*((PUINT32V)(SD3_BASE_ADDR+0x008))) // SD 128bits response register, [31:0] 32bits                                                                                                                                                                    
#define     R32_SD3_RESPONSE1   (*((PUINT32V)(SD3_BASE_ADDR+0x00c))) // SD 128bits response register, [63:32] 32bits                                                                                                                                                                   
#define     R32_SD3_RESPONSE2   (*((PUINT32V)(SD3_BASE_ADDR+0x010))) // SD 128bits response register, [95:64] 32bits                                                                                                                                                                   
#define     R32_SD3_RESPONSE3   (*((PUINT32V)(SD3_BASE_ADDR+0x014))) // SD 128bits response register, [127:96] 32bits, when response length is 48, the 32bit response data is stored in this register                                                                                  
#define     R32_SD3_WRITE_CONT	(*((PUINT32V)(SD0_BASE_ADDR+0x014)))
#define     R8_SD3_CTRL         (*((PUINT8V )(SD3_BASE_ADDR+0x018))) // SD 8bits control register, [1:0] number of data line will operate, [2]==1 soft reset all the inner logic, [3]==1 enable DMA, [4]==1 reset inner data tran/recv logic, [5]==1 @negedge sample cmd/dat[7:0] lines
#define     R8_SD3_TOCNT        (*((PUINT8V )(SD3_BASE_ADDR+0x01c))) // SD 8bits data timeout value, the real timeout time is 4194304*Tsdclk*value                                                                                                                                     
#define     R32_SD3_STATUS      (*((PUINT32V)(SD3_BASE_ADDR+0x020))) // SD status:[15:0] number of current successfull tran/recv blocks, [16] current sd cmd line level, [17] current sd dat[0] line level                                                                             
#define     R16_SD3_INT_FG      (*((PUINT16V)(SD3_BASE_ADDR+0x024))) // SD 16bits interrupt flag register                                                                                                                                                                              
#define     R16_SD3_INT_EN      (*((PUINT16V)(SD3_BASE_ADDR+0x028))) // SD 16bits interrupt enable register                                                                                                                                                                            
#define     R16_SD3_DMA         (*((PUINT16V)(SD3_BASE_ADDR+0x02C))) // SD 16bits DMA start address register when to operate                                                                                                                                                                              
#define     R32_SD3_BLOCK_CFG   (*((PUINT32V)(SD3_BASE_ADDR+0x030))) // SD 32bits data counter, [15:0] number of blocks this time will tran/recv, [27:16] block sise(byte number) of every block in this time tran/recv                                                                
#define     R8_SD3_TRAN_MODE    (*((PUINT8V )(SD3_BASE_ADDR+0x034))) // SD TRANSFER MODE register, [0] direction of this time, 1-write sd, 0-read sd, [1] stop at block gap when multiple wr/rd, [2] this time is eMMC boot mode operation                                             

/* R16_SD_CLK_CFG */
#define bSDCLK_Mode 	    0x200 // SDCLK mode select:	0=slow, 400KHz, 1=faster, 25~100MHz;
#define bSDCLK_OE		    0x100 // supply SDCLK to sdcard: 0=disable, 1=enable;
#define MASK_SD_CLK_PRE		0x01F // divisor of the divider, If bSDCLK_Mode=1, SDCLK=480M/CLK_PRE; if bSDCLK_Mode=0, SDCLK=480M/CLK_PRE/64 

/* R16_SD0_CMD_SET / R16_SD1_CMD_SET / R16_SD2_CMD_SET / R16_SD3_CMD_SET  */
#define bCHK_RESP_IDX	    0x800 // check the index of the response	
#define bCHK_RESP_CRC	    0x400 // check the CRC of the response
#define	MASK_RESP_TYPE		0x300 // response type
	#define RESP_TYPE_NO	    0x000 // current command doesn't have a response
	#define RESP_TYPE_136	    0x100 // current command response length is 136
	#define RESP_TYPE_48	    0x200 // current command response length is 48
	#define RESP_TYPE_R1b	    0x300 // current command response type is R1b	
#define	MASK_CMD_IDX			0x03F // command index of current operation

/* R8_SD0_CTRL / R8_SD1_CTRL / R8_SD2_CTRL / R8_SD3_CTRL  */
#define bSC_NEG_SAMPLE      0x0020 // controller use nagedge sample cmd, dat[7:0], when use UHS-I mode, timing is difficult to meet, use the negedge will be better, default is @posedge sample    
#define bSC_RST_DAT_LGC     0x0010 // reset the data tran/recv logic, this bit is useful when block CRC error in multiple rd/wr situation, default is valid  
#define bSC_DMA_ENABLE      0x0008 // enable the dma, if the 1# 2# 3# controller is to be used, should also open the 0# dma enable, 0# dma work as the whole enable, default dma is closed 
#define bSC_ALL_CLR         0x0004 // reset all the inner logic, default is valid
#define DAT_LINE_WIDTH	    0x0003 // width of the data line will be used  
	#define bSC_OP_DAT8         0x0002 // use data line [7:0], can be set only in 0# 2# controller
	#define bSC_OP_DAT4         0x0001 // use data line [3:0], the default is this value
	#define bSC_OP_DAT0         0x0000 // use data line [0] only

/*  R8_SD0_TOCNT / R8_SD1_TOCNT / R8_SD2_TOCNT / R8_SD3_TOCNT  */
#define MASK_TOCNT		    0xF

/*  R32_SD0_STATUS / R32_SD1_STATUS / R32_SD2_STATUS / R32_SD3_STATUS */
#define bST_CMD_HI          0x10000 // indicate cmd line is high level now   
#define bST_DAT0_HI         0x20000 // indicate dat[0] line is high level now
#define MASK_BLOCK_NUM	    0x0FFFF // number of blocks have be successfully tran/recv
                        
/* R16_SD0_INT_FG / R16_SD1_INT_FG / R16_SD2_INT_FG / R16_SD3_INT_FG */
#define bSIF_SDIO_INT       0x200 // interrupt from SDIO card inside
#define bSIF_FIFO_OF        0x100 // fifo overflow, when write sd, indicate empty overflow, when read sd, indicate full overflow
#define bSIF_BLOCK_GAP      0x080 // every block gap interrupt when multiple read/write, allow drive change the DMA address at this moment 
#define bSIF_TRANS_SC       0x040 // all the blocks have been tran/recv successfully    
#define bSIF_TRANS_ER       0x020 // last block have encountered a CRC error
#define bSIF_DATA_TMO       0x010 // data line busy timeout                    
#define bSIF_CMD_DONE       0x008 // when cmd hasn't response, indicate cmd has been sent, when cmd has a response, indicate cmd has bee sent and has received the response
#define bSIF_RE_IDX_ER      0x004 // indicate INDEX error of the response
#define bSIF_RE_CRC_WR      0x002 // indicate CRC error of the response
#define bSIF_RE_TMOUT       0x001 // indicate when expect the response, timeout              

/* R16_SD0_INT_EN / R16_SD1_INT_EN / R16_SD2_INT_EN / R16_SD3_INT_EN */
#define bSIE_SDIO_INT       0x200    
#define bSIE_FIFO_OF        0x100
#define bSIE_BLOCK_GAP      0x080
#define bSIE_TRANS_SC       0x040
#define bSIE_TRANS_ER       0x020
#define bSIE_DATA_TMO       0x010
#define bSIE_CMD_DONE       0x008
#define bSIE_RE_IDX_ER      0x004
#define bSIE_RE_CRC_WR      0x002
#define bSIE_RE_TMOUT       0x001

/* R32_SD0_BLOCK_CFG / R32_SD1_BLOCK_CFG / R32_SD2_BLOCK_CFG / R32_SD3_BLOCK_CFG */
#define BLOCK_SIZE		    0xFFF0000 // number of bytes in every block
#define BLOCK_NUM		    0x000FFFF // number of blocks will be tran/recv, non-zero value activate tran/recv logic  

/* R8_SD0_TRAN_MODE / R8_SD1_TRAN_MODE / R8_SD2_TRAN_MODE / R8_SD3_TRAN_MODE  */
#define bTM_EMMC_BOOT	    0x04 // current operation is emmc boot mode
#define bTM_GAP_STOP	    0x02 // enable this bit, data tran/recv logic will stop at gap
#define bTM_WR_SD		    0x01 // direction is sram to SD

#ifdef __cplusplus
}
#endif

#endif      // __CH568SDC4SFR_H__


#ifndef __CH568ECDCSFR_H__
#define __CH568ECDCSFR_H__

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************/
/*                         Peripheral memory map                              */
/******************************************************************************/
/* base address */
#define		AES_SM4_BASE_ADDR			(0x0040c400)		

#define		R32_AES_SM4_CTRL		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x00)))	// AES/SM4 control register
   #define   bKEYE_ACT_IE      0x00010000					// enable interrupt for key expansion finished
   #define   MASK_Key_LEN      0x00000C00					// select key length: 00=128bit, 01=192bit, 10=256bit, 11=reserved
   #define   bBCIPHER_MOD      0x00000200					// select block cipher mode: 0=ECB mode, 1=CTR mode
   #define   bALGRM_MOD        0x00000100					// select algorithm: 0=SM4, 1=AES
   #define   MASK_ED_CLK_PRE   0x00000030					// divisor of the divider,  EDCLK=480M/ED_CLK_PRE
   #define   bEDMOD_SELT       0x00000008					// 0=encryption, 1=decryption
   #define   bRDDAT_ED_EN      0x00000004					// enable encrypt/decrypt RAM data to write to SD
   #define   bWRDAT_ED_EN      0x00000002					// enable encrypt/decrypt RAM data read from SD
   #define   bKEYE_EN          0x00000001                   // enable key expansion

#define		R32_AES_SM4_INT_FG		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x04)))   // AES/SM4 interrupt flag register
   #define   bKEYE_ACT_IF      0x00010000					// key expansion finished interrutp flag

/*	the following 8*32-bit registers are used to store 128/192/256-bit key */
#define		R32_AES_SM4_KEY7		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x08)))	
#define		R32_AES_SM4_KEY6		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x0c)))	
#define		R32_AES_SM4_KEY5		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x10)))	
#define		R32_AES_SM4_KEY4		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x14)))	
#define		R32_AES_SM4_KEY3		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x18)))	
#define		R32_AES_SM4_KEY2		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x1c)))	
#define		R32_AES_SM4_KEY1		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x20)))	
#define		R32_AES_SM4_KEY0		(*((PUINT32V)(AES_SM4_BASE_ADDR + 0x24)))
	
/*	the following 4*32-bit registers are used to store initial vector for CTR mode	*/
#define		R32_AES_SM4_IV3		    (*((PUINT32V)(AES_SM4_BASE_ADDR + 0x28)))	
#define		R32_AES_SM4_IV2		    (*((PUINT32V)(AES_SM4_BASE_ADDR + 0x2c)))	
#define		R32_AES_SM4_IV1		    (*((PUINT32V)(AES_SM4_BASE_ADDR + 0x30)))	
#define		R32_AES_SM4_IV0		    (*((PUINT32V)(AES_SM4_BASE_ADDR + 0x34)))	

#ifdef __cplusplus
}

#endif

#endif      // __CH568ECDCSFR_H__


#ifndef __SDIO_TYPE__
#define __SDIO_TYPE__

#ifdef __cplusplus
extern "C" {
#endif

/* 应答方式 */
#define R2    							0x01
#define R1    							0x02
#define R1B   							0x03

/* SD Mode Command List */
/* 复位SD 卡 */
#define CMD0 							0
#define CMD0_R 							R1

/* 激活该卡的初始化处理 */
#define CMD1 							1
#define CMD1_R 							R1

/* 第二版接口识别 */
#define CMD8 							8
#define CMD8_R 							R3

/* 读CSD寄存器 */
#define CMD9 							9
#define CMD9_R 							R1

/* 读CID寄存器 */
#define CMD10 							10
#define CMD10_R 						R1

/* 停止读多块时的数据传输  */
#define CMD12 							12
#define CMD12_R 						R1B

/* 读 Card_Status 寄存器  */
#define CMD13 							13
#define CMD13_R 						R2

/************************************ 块读命令集********************************/
/* 设置块的长度 */
#define CMD16 							16
#define CMD16_R 						R1

/* 读单块*/
#define CMD17 							17
#define CMD17_R 						R1

/* 读多块,直至主机发送CMD12为止 */
#define CMD18 							18
#define CMD18_R 						R1

/*************************************块写命令集 ********************************/
/* 写单块  */
#define CMD24 							24
#define CMD24_R 						R1

/* 写多块 */
#define CMD25 							25
#define CMD25_R 						R1

/* 写CSD寄存器 */
#define CMD27 							27
#define CMD27_R 						R1

/************************************ 写保护 ***********************************/
#define CMD28 							28
#define CMD28_R 						R1B

#define CMD29 							29
#define CMD29_R 						R1B

#define CMD30 							30
#define CMD30_R 						R1

/************************************* 擦除命令 *******************************/
/* 设置擦除块的起始地址(只用于SD卡) */
#define CMD32 							32
#define CMD32_R 						R1

/* 设置擦除块的终止地址(只用于SD卡)  */
#define CMD33 							33
#define CMD33_R 						R1

/* 设置擦除块的起始地址(只用于MMC卡) */
#define CMD35 							35
#define CMD35_R 						R1

/* 设置擦除块的终止地址(只用于MMC卡) */
#define CMD36 							36
#define CMD36_R 						R1

/* 擦除所选择的块 */
#define CMD38 							38
#define CMD38_R 						R1B

/************************************ 锁卡命令 **********************************/
/* 设置/复位密码或上锁/解锁卡 */
#define CMD42 							42
#define CMD42_R							R1B

/************************************ 应用命令  ********************************/
/* 通知下一个命令为应用命令 */
#define CMD55 							55
#define CMD55_R 						R1

/* 向卡发送或从卡接收数据块来获取通用/特殊命令集 */
#define CMD56 							56
#define CMD56_R 						R1

/* 读OCR寄存器 */
#define CMD58 							58
#define CMD58_R 						R3

/* 使能或禁止CRC */
#define CMD59 							59
#define CMD59_R 						R1

/************************************ 应用命令  ********************************/
/* 获取 SD Status寄存器 */
#define ACMD13 							13
#define ACMD13_R 						R2

/* 获取已写入卡中的块的个数 */
#define ACMD22 							22
#define ACMD22_R 						R1

/* 在写之前,设置预先擦除的块的个数  */
#define ACMD23 							23
#define ACMD23_R 						R1

/* 启动此卡的初始化过程  */
#define ACMD41 							41
#define ACMD41_R 						R1

/* 连接/断开CD/DATA[3]引脚上的上拉电阻  */
#define ACMD42 							42
#define ACMD42_R 						R1

/* 读取该卡的配置寄存器SCR  */
#define ACMD51 							51
#define ACMD51_R 						R1

/******************************************************************************/
/* 相关错误码定义 */
		
/* 错误码(error code) */
#define SD_NO_ERR			     		0x00					 /* 函数执行成功 */
#define SD_ERR_NO_CARD		 			0x60					 /* 卡没有完全插入到卡座中 */
#define SD_ERR_USER_PARAM      			0x61					 /* 用户使用API函数时，入口参数有错误 */
#define SD_ERR_CARD_PARAM		 		0x62					 /* 卡中参数有错误（与本模块不兼容） */
#define	SD_ERR_VOL_NOTSUSP        		0x63					 /* 卡不支持3.3V供电 */
#define SD_ERR_OVER_CARDRANGE			0x64					 /* 操作超出卡容量范围 */
#define SD_ERR_UNKNOWN_CARD       		0x65					 /* 无法识别卡型 */
#define SD_ERR_WP_PROTECT         		0x66            		 /* 写保护 */
#define SD_CARD_BUSY              		0x67            		 /* 在读取OCR寄存器时检测到BIT31为0表示忙 */
#define SD_BLOCK_ERROR            		0x68            		 /* 块大小一定要大于等于512,否则无法操作 */

/* SD命令可能返回的错误码 */
#define SD_ERR_CMD_RESPTYPE	 			0x69					 /* 命令类型错误 */
#define SD_ERR_CMD_TIMEOUT     			0x6A					 /* 卡命令响应超时 */
#define SD_ERR_CMD_RESP		 			0x6B					 /* 卡命令响应错误 */
#define SD_Power_Up_Busy          		0x6C					 /* 上电忙 */

/* 数据流错误码 */
#define SD_ERR_DATA_CRC16      			0x6D					 /* 数据流CRC16校验不通过 */
#define SD_ERR_DATA_START_TOK			0x6E					 /* 读单块或多块时,数据开始令牌不正确 */
#define	SD_ERR_DATA_RESP		 		0x6F					 /* 写单块或多块时,卡数据响应令牌不正确 */

/* 等待错误码 */
#define SD_ERR_TIMEOUT_WAIT    			0x70					 /* 写或擦操作时,发生超时错误 */
#define SD_ERR_TIMEOUT_READ    			0x71					 /* 读操作超时错误 */
#define	SD_ERR_TIMEOUT_WRITE	 		0x72					 /* 写操作超时错误 */
#define SD_ERR_TIMEOUT_ERASE   			0x73					 /* 擦除操作超时错误 */
#define	SD_ERR_TIMEOUT_WAITIDLE 		0x74					 /* 初始化卡时,等待卡退出空闲状态超时错误 */

/* 写操作可能返回的错误码 */
#define	SD_ERR_WRITE_BLK				0x75					 /* 写块数据错误 */
#define	SD_ERR_WRITE_BLKNUMS      		0x76				 	 /* 写多块时,想要写入的块与正确写入的块数不一致 */
#define SD_ERR_WRITE_PROTECT			0x77					 /* 卡外壳的写保护开关打在写保护位置 */

/* 使用UCOS-II时可能出现的错误 */
#define SD_ERR_CREATE_SEMSD				0xA0					 /* 创建SD/MMC卡访问信号量失败 */

/********************************************************************************/
/* 其它宏定义 */	
/********************************************************************************/
#define	CARDTYPE_SD						0x00					 /* 卡型为SD  卡 */
#define CARDTYPE_MMC					0x01					 /* 卡型为MMC 卡 */
#define SD_UCOSII_SMALLWAIT				0xFF					 /* 运行于UCOS-II时的小等待时间 */
#define SD_READREG_TIMEOUT				0x08					 /* 读寄存器的超时时间 */
		    	    	    
/* Mask off the bits in the OCR corresponding to voltage range 3.2V to 3.4V, OCR bits 20 and 21 */
#define MSK_OCR_33			 			0xC0

/* 定义在初始化阶段,等待SD卡退出空闲状态的重试次数 */
/* Number of tries to wait for the card to go idle during initialization */
#define SD_IDLE_WAIT_MAX     			10000

/* SD卡命令超时时间(timeout of command) */
#define SD_CMD_TIMEOUT   	 			0x0FFFF
	
/* 超时时间：SD 卡时钟周期 * 4194304 * MASK_TOCNT */
#define READ_TIMEOUT_100MS      		100 * SD_CLOCK / 4194304

#define WRITE_TIMEOUT_250MS				250 * SD_CLOCK / 4194304    


/* CSD中一些域的字节位置(高字节在前) */
#define TAAC_POS 						1						 /* TACC */
#define NSAC_POS						2						 /* NSAC */
#define READ_BL_LEN_POS					5						 /* READ_BL_LEN */
#define C_SIZE_POS1						6						 /* C_SIZE upper  2-bit */
#define C_SIZE_POS2						7						 /* C_SIZE middle 8-bit */
#define C_SIZE_POS3						8						 /* C_SIZE lower	2-bit */
#define C_SIZE_MULT_POS1				9						 /* C_SIZE_MULT upper 2-bit */
#define C_SIZE_MULT_POS2				10						 /* C_SIZE_MULT lower 1-bit */	
#define SECTOR_SIZE_POS1				10						 /* SECTOR_SIZE upper 5-bit */
#define SECTOR_SIZE_POS2				11						 /* SECTOR_SIZE lower 2-bit */
#define R2WFACTOR_POS 					12						 /* R2WFACTOR_POS */

/* MMC擦除相关定义 */
#define ERASE_GRP_SIZE_POS   			10						 /* MMC卡 ERASE_GRP_SIZE 5-bit */
#define ERASE_GRP_MULTI_POS1 			10						 /* MMC卡 ERASE_GRP_MULTI 2-bit */
#define ERASE_GRP_MULTI_POS2 			11						 /* MMC卡 ERASE_GRP_MULTI 3-bit */

/* CSD中一些域的掩码 */
#define TAAC_MSK						0x07					 /* TACC 域掩码 */
#define NSAC_MSK						0x78					 /* NSAC 域掩码 */
#define READ_BL_LEN_MSK				 	0x0F				 	 /* READ_BL_LEN 的掩码 */
#define C_SIZE_MSK1						0x03					 /* C_SIZE 高2位掩码 */
#define C_SIZE_MSK3						0xC0					 /* C_SIZE 低2位掩码 */
#define C_SIZE_MULT_MSK1 				0x03					 /* C_SIZE_MULT 的高2位掩码 */
#define C_SIZE_MULT_MSK2 				0x80					 /* C_SIZE_MULT 的低2位掩码 */
#define R2WFACTOR_MSK					0x1C					 /* R2WFACTOR 掩码 */
#define SECTOR_SIZE_MSK1				0x3F					 /* SECTOR_SIZE 的高5位 */
#define SECTOR_SIZE_MSK2				0x80					 /* SECTOR_SIZE 的低2位 */
#define ERASE_GRP_SIZE_MSK 				0x7C					 /* MMC卡 ERASE_GRP_SIZE 掩码 */
#define ERASE_GRP_MULTI_MSK1 			0x03					 /* MMC卡 ERASE_GRP_MULTI 高2位掩码 */
#define ERASE_GRP_MULTI_MSK2 			0xE0				  	 /* MMC卡 ERASE_GRP_NULTI 低3位掩码 */
#define SD_BLOCK_LEN       				512						 /* 设置SD块大小 */


#ifdef __cplusplus
}
#endif

#endif      // __SDIO_TYPE__


