
#include <stdio.h>
#include <nds32_isr.h>
#include <nds32_intrinsic.h>
#include "CH568SFR.H"
#include "MYdebug.H"

void NDS32ATTR_RESET("vectors=16") reset_handler(void)
{
    __c_init();
    main();
}

void __c_init(void)
{

    extern char _end;
    extern char __bss_start;
    int size;
    extern char __data_lmastart;
    extern char __data_start;
    extern char _edata;

    /* Copy data section to RAM */
    size = &_edata - &__data_start;
    MEMCPY(&__data_start, &__data_lmastart, size);

    /* Clear bss section */
    size = &_end - &__bss_start;
    MEMSET(&__bss_start, 0, size);
    return;
}

void Interrupt_init(UINT16 itnum)/* Initialize interrupt */
{
    __nds32__mtsr(itnum, NDS32_SR_INT_MASK2);                /* enable HW# */
    __nds32__setgie_en();                                    /* enable global interrupt */
    __nds32__dsb();
}

/*******************************************************************************
* Function Name  : mInitSTDIO_UR3
* Description    : 为printf和getkey输入输出初始化串口
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void mInitSTDIO_UR3 (void)                                                //UART3初始化
{
    UINT32    x;

    R8_UART3_DIV = 1;
    x = 10 * FREQ_SYS / R8_UART3_DIV * 2 / 16 / STDIO_BAUD_RATE;
    x = ( x + 5 ) / 10;                                                  // +5是四舍五入                                                         // 预分频
    R16_UART3_DL = x;
    R8_UART3_FCR = RB_FCR_FIFO_TRIG | RB_FCR_TX_FIFO_CLR | RB_FCR_RX_FIFO_CLR | RB_FCR_FIFO_EN;
    R8_UART3_LCR = RB_LCR_WORD_SZ;
    R8_UART3_IER = RB_IER_TXD_EN;                                          // TXD enable
    R32_PA_PU |= bRXD3 | bTXD3;                                          // RXD1 schmitt input, TXD1 slow rate
    R32_PA_DIR |= bTXD3;                                                  // TXD1 output enable(pin_pa[9])

    printf("ch5%02x debug time:%s \n", R8_CHIP_ID, __TIME__);
}

/*******************************************************************************
* Function Name  : putchar
* Description    : 通过串口输出监控信息
* Input          : c-- writes the character specified by c
* Output         : None
* Return         : None
*******************************************************************************/
#undef putchar
int putchar( int c )
{
    while    ( R8_UART3_TFC == UART_FIFO_SIZE );
    R8_UART3_THR = c;
    return c;
}
__attribute__((used))
void nds_write(const unsigned char *buf, int size)
{
    int i;
    for(i=0; i<size; i++)
        putchar(buf[i]);
}



void PowerInit(void)
{
//为降低功耗，如果不用，需要关闭。
    R8_SAFE_ACCESS_SIG = 0x57; //开启安全模式，准备设置系统时钟相关寄存器
    R8_SAFE_ACCESS_SIG = 0xa8;
    R8_SLP_POWER_CTRL = RB_SLP_SATA_PWRDN;
    //R8_SLP_CLK_OFF1 = RB_SLP_CLK_SDC;
}

/*******************************************************************************
* Function Name  : SysClkInit
* Description    : 系统时钟初始化
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SysClkInit(void)
{
// 在安全模式下设置默认系统时钟为xxMHz
    R8_SAFE_ACCESS_SIG = 0x57; //开启安全模式，准备设置系统时钟相关寄存器
    R8_SAFE_ACCESS_SIG = 0xa8;
    R8_CLK_PLL_DIV = 0x40 | 0x05;                // 系统时钟为SYS_FREQ = SRC_FREQ / R8_CLK_PLL_DIV = 480MHz/8=60MHz
    R8_CLK_CFG_CTRL = 0x80 | RB_CLK_SEL_PLL ;    // OSC_FREQ = 30MHz
                                                // PLL_FREQ = 300MHz/480MHz
                                                // SRC_FREQ = RB_CLK_SEL_PLL ? PLL_FREQ : OSC_FREQ, 选择PLL的时钟作为系统源时钟，即 300/480 MHz
    mDelayuS(2);                                //延时2us，跳过不稳定的时钟

    PowerInit();
}

/*******************************************************************************
* Function Name  : mDelayuS
* Description    : us 延时
* Input          : n-- us time
* Output         : None
* Return         : None
*******************************************************************************/
void    mDelayuS( UINT16 n )  // 以uS为单位延时
{
    R32_TMR1_CNT_END = FREQ_SYS/1000000*n-15;
    R8_TMR1_INT_FLAG |= RB_TMR_IF_CYC_END;
    R8_TMR1_CTRL_MOD = RB_TMR_ALL_CLEAR ;

    R8_TMR1_CTRL_MOD = RB_TMR_COUNT_EN;
    while(!(R8_TMR1_INT_FLAG & RB_TMR_IF_CYC_END));
    R8_TMR1_CTRL_MOD = 0;
}

/*******************************************************************************
* Function Name  : mDelaymS
* Description    : ms 延时
* Input          : n-- ms time
* Output         : None
* Return         : None
*******************************************************************************/
void    mDelaymS( UINT16 n )  // 以mS为单位延时
{
    while ( n ) {
        mDelayuS( 1000 );
        -- n;
    }
}


